dnl  ====================================================
dnl             MATLAB library macros
dnl  ====================================================

dnl
dnl  CHECK_MATLAB - find Matlab libraries
dnl  ----------------------------------------------------

AC_DEFUN([AX_CHECK_MATLAB],
[
AC_ARG_WITH(matlab,
  [  --with-matlab=DIR	no, yes, or the directory where Matlab is installed ],
  [have_matlab=$withval],
  [have_matlab=no])

case $have_matlab in 
  no)
    want_matlab=no
    ;;
  yes)
    want_matlab=yes
    AC_PATH_PROG([MEX], [mex], [/usr/local/matlab/bin/mex], [/usr/local/matlab/bin:$PATH])
    ;;
  *)
    want_matlab=yes
    AC_PATH_PROG([MEX], [mex], [$have_matlab/bin/mex], [$have_matlab/bin:$have_matlab/bin/matlab:/usr/local/matlab/bin:$PATH])
    MATLAB_DIR=$have_matlab
    ;;
esac

if test "$want_matlab" = "no"; then
  AM_CONDITIONAL(HAVE_MATLAB, test "xno" = "xyes" )
  have_matlab=no
elif test -z "$MEX"; then
  AC_MSG_WARN([Could not find 'mex' executable for MATLAB.])
  AM_CONDITIONAL(HAVE_MATLAB, test "xno" = "xyes" )
  have_matlab=no
else 
  AM_CONDITIONAL(HAVE_MATLAB, test "xyes" = "xyes" )
  have_matlab=yes
fi


if test $want_matlab = "yes"
then

dnl default values
    MEX="$MEX CC='\$(COMPILE)'"
    MEXOPTS=""
    MEXFLAGS="-g "
    MEXLDFLAGS=""
    MEXLIB=""
    MEXEXT=""

    case $build_os in

    *alpha*linux*)
        MEXEXT=".mexlx"
	;;

    *linux*)
        MEXEXT=.mexglx
	;;

    *aix*)
 	MEXEXT=".mexrs6"
	;;	

    *solaris*)
        MEXEXT=".mexsol"
	;;	

    *sunos*)
        MEXEXT=".mex4"
	;;	

    *linux_without_mex*)
        MEXFLAGS="-I${MATLAB_DIR}/extern/include -I${MATLAB_DIR}/simulink/include -DMATLAB_MEX_FILE -fPIC -ansi -D_GNU_SOURCE -pthread -O -DNDEBUG";
        MEXLDFLAGS="-pthread -shared -W1,--version-script,${MATLAB_DIR}/extern/lib/glnx86/mexFunction.map";
        MEXLIB="-W1,--rpath-link,${MATLAB_DIR}/extern/lib/glnx86,--rpath-link,${MATLAB_DIR}/bin/glnx86 -L${MATLAB_DIR}/bin/glnx86 -lmx -lmex -lmat -lm";
        MEXEXT=.mexglx
	;;

    *cygwin_without_mex*)
        MEXFLAGS="-I${MATLAB_DIR}/extern/include -I${MATLAB_DIR}/simulink/include -fno-exceptions -DMATLAB_MEX_FILE -DNDEBUG";
        MEXLDFLAGS="-shared -W1,--version-script,${MATLAB_DIR}/extern/lib/win32/mexFunction.def";
        MEXLIB="-W1,--rpath-link,${MATLAB_DIR}/extern/lib/win32,--rpath-link,${MATLAB_DIR}/bin/win32 ${MATLAB_DIR}/bin/win32/libmx.a ${MATLAB_DIR}/bin/win32/libmex.a ${MATLAB_DIR}/bin/win32/libmat.a -lm";
        MEXLDFLAGS="-shared -L${MATLAB_DIR}/bin/win32 -W1,--version-script,${MATLAB_DIR}/extern/lib/win32/mexFunction.def";
        MEXLIB="-lmx -lmex -lmat -lm";
        MEXEXT=.dll;
        if test ! -e "${MATLAB_DIR}/bin/win32/libmx.a"
        then
            cd ${MATLAB_DIR}/bin/win32
            libmx=`dlltool -llibmx.a -d${MATLAB_DIR}/extern/include/libmx.def -Dlibmx.dll`
            cd -
        fi
        if test ! -e "${MATLAB_DIR}/bin/win32/libmex.a"
        then
            cd ${MATLAB_DIR}/bin/win32
            libmex=`dlltool -llibmex.a -d${MATLAB_DIR}/extern/include/libmex.def -Dlibmex.dll`
            cd -
        fi
        if test ! -e "${MATLAB_DIR}/bin/win32/libmex.a"
        then
            cd ${MATLAB_DIR}/bin/win32
            libmat=`dlltool -llibmat.a -d${MATLAB_DIR}/extern/include/libmat.def -Dlibmat.dll`
            cd -
        fi
	;;

    *mingw_without_mex*)
        MEXFLAGS="-I${MATLAB_DIR}/extern/include -I${MATLAB_DIR}/simulink/include -fno-exceptions -DMATLAB_MEX_FILE -DNDEBUG";
        MEXLDFLAGS="-shared -W1,--version-script,${MATLAB_DIR}/extern/lib/win32/mexFunction.def";
        MEXLIB="-W1,--rpath-link,${MATLAB_DIR}/extern/lib/win32,--rpath-link,${MATLAB_DIR}/bin/win32 ${MATLAB_DIR}/bin/win32/libmx.a ${MATLAB_DIR}/bin/win32/libmex.a ${MATLAB_DIR}/bin/win32/libmat.a -lm";
        MEXLDFLAGS="-shared -L${MATLAB_DIR}/bin/win32 -W1,--version-script,${MATLAB_DIR}/extern/lib/win32/mexFunction.def";
        MEXLIB="-lmx -lmex -lmat -lm";
        MEXEXT=.dll;
        if test ! -e "${MATLAB_DIR}/bin/win32/libmx.a"
        then
            cd ${MATLAB_DIR}/bin/win32
            libmx=`dlltool -llibmx.a -d${MATLAB_DIR}/extern/include/libmx.def -Dlibmx.dll`
            cd -
        fi
        if test ! -e "${MATLAB_DIR}/bin/win32/libmex.a"
        then
            cd ${MATLAB_DIR}/bin/win32
            libmex=`dlltool -llibmex.a -d${MATLAB_DIR}/extern/include/libmex.def -Dlibmex.dll`
            cd -
        fi
        if test ! -e "${MATLAB_DIR}/bin/win32/libmex.a"
        then
            cd ${MATLAB_DIR}/bin/win32
            libmat=`dlltool -llibmat.a -d${MATLAB_DIR}/extern/include/libmat.def -Dlibmat.dll`
            cd -
        fi
	;;

    *unknown*)
	MEXEXT=""
	;;

    esac

    AC_SUBST(MEX)
    AC_SUBST(MEXOPTS)
    AC_SUBST(MEXFLAGS)
    AC_SUBST(MEXEXT)
    AC_SUBST(MEXLIB)
    AC_SUBST(MEXLDFLAGS)
    AC_SUBST(MATLAB_DIR)
fi

])

