dnl checks for mkoctfile in the user specified location, then checks the version
dnl this macro takes one argument: the required version number.
dnl for example:   AX_CHECK_OCTAVE(2.9)

AC_DEFUN([AX_CHECK_OCTAVE],
[
AC_ARG_WITH(octave,
  [  --with-octave=DIR	no, yes, or the directory where mkoctfile is installed ],
  [have_octave=$withval],
  [have_octave=no])

case $have_octave in 
  no)
    want_octave=no
    ;;
  yes)
    want_octave=yes
    AC_PATH_PROG([MKOCTFILE], [mkoctfile], [], [/usr/bin:$PATH])
    ;;
  *)
    want_octave=yes
    AC_PATH_PROG([MKOCTFILE], [mkoctfile], [], [$have_octave:$PATH])
    if test -z "$MKOCTFILE"; then
      bn=`basename $have_octave`
      if test "$bn" = "mkoctfile"; then
        if test -e "$have_octave"; then
          MKOCTFILE="$have_octave"
        fi
      fi
    fi
    ;;
esac

if test "$want_octave" = "no"; then
  AM_CONDITIONAL(have_octave, test "xno" = "xyes" )
  have_octave=no
elif test -z "$MKOCTFILE"; then
  AC_MSG_ERROR([Could not find 'mkoctfile' executable for Octave.])
else 
  SAVE_CFLAGS=$CFLAGS
  OCTINC=`$MKOCTFILE -p INCFLAGS`
  CFLAGS="-I $OCTINC $SAVE_CFLAGS"
  AC_TRY_RUN([#include <stdio.h>
#include <stdlib.h>

#include "version.h"

int
main(int argc, char *argv[])
{
  FILE *foo;

  foo=fopen("octave_version.txt", "w");

  if(!foo) {
    fprintf(stderr, "could not open 'octave_version.txt' for writing\n");
    exit(-1);
  }

  fprintf(foo, "%s\n", OCTAVE_VERSION);
  fclose(foo);
  exit(0);
}], [oct_ver=`cat octave_version.txt`], [oct_ver=bad], [])

  if test "$oct_ver" = "bad"; then
    AC_MSG_ERROR([Could not determine octave version.])
  fi

  AX_COMPARE_VERSION([$oct_ver], [lt], [$1], [ver_ok="no"], [ver_ok="ok"])

  if test "$ver_ok" = "no"; then
    AC_MSG_ERROR([If Octave is enabled, it must be at least version $1.
You have version $oct_ver.  If you want to build without Octave support,
omit the --with-octave flag.])
  fi

  AM_CONDITIONAL(have_octave, test "xyes" = "xyes" )
  have_octave=yes
  CFLAGS=$SAVE_CFLAGS
fi

AC_SUBST(MKOCTFILE)

])

