#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "netsolveclient.h"

int main(int argc, char** argv){
  int i;
  int length;
  int* inputVec;
  int* outputVec;
  DSI_FILE* dsi_file;
  DSI_OBJECT* dsivec;
  char *dsihost;
  grpc_function_handle_t handle;
  
  /*  You can pass in another DSI host if you have one */
  if(argc == 2)
    dsihost = argv[1];
  else
    dsihost = "acre.cs.utk.edu";

  length = 10;
  printf("Test add 5 to each element of a vector of length %d\n", length);

  inputVec = (int*)malloc(sizeof(int)*length);
  outputVec = (int*)malloc(sizeof(int)*length);

  srand48(100);
  for(i=0; i<length; i++) inputVec[i] = drand48() * 100;
  for(i=0; i<length; i++) outputVec[i] = 0;

  printf("Initial elements: \n");
  for(i=0; i<length; i++)
    printf("%d ", inputVec[i]);
  printf("\n");
  
  dsi_file = ns_dsi_open(dsihost, O_CREAT|O_RDWR , 744 , 3000, GS_DSI_IBP);
  if(dsi_file == NULL)
    fprintf(stderr, "Error in opening Distributed Storage Infrastructure (DSI) at %s\n", dsihost);

  printf("Writing input vector to Distributed Storage Infrastructure (DSI)\n");
  dsivec = ns_dsi_write_vector(dsi_file, inputVec, length, GS_INT);
  if(dsivec == NULL) {
    fprintf(stderr, "Error writing input vector to DSI \n");
    exit(EXIT_FAILURE);
  }

  printf("Calling GridSolve using a blocking call\n");
  if((grpc_initialize(NULL) != GRPC_NO_ERROR) ||
     (grpc_function_handle_default(&handle, "int_vector_add5") != GRPC_NO_ERROR) ||
     (grpc_call(&handle, length, dsivec, outputVec) != GRPC_NO_ERROR)) { 
    grpc_perror("Error calling GridSolve");
    exit(EXIT_FAILURE);
  }

  printf("Final elements: \n");
  for(i=0; i<length; i++)
    printf("%d ", outputVec[i]);
  printf("\n");

  printf("Checking: ");
  for(i=0; i<length; i++) {
    if (inputVec[i] != outputVec[i] - 5) {
      fprintf(stderr, "Bad results in output\n");
      exit(EXIT_FAILURE);
    }
  }

  ns_dsi_close(dsi_file);
  grpc_finalize();

  printf("GridRPC test completed successfully.\n");
  return EXIT_SUCCESS;
}
