//-----------------------------------------------------------------------------
// GA2K
//
// ISDL, Doshisha University Distributed Genetic Algorithm Software
// Version 1.4.1
//-----------------------------------------------------------------------------
// Parallel Genetic Algorithm Team of Intelligent Systems Design Laborato
// ry, Doshisha University.
//
// Copyright (C) 1999           Ikki Ohmukai,      All rights reserved.
// Copyright (C) 1999,2000,2001 Jun-ichi Yoshida,  All rights reserved.
// Copyright (C) 2001,2002,2003 Tomoyuki Hiroyasu, All rights reserved.
// Copyright (C) 2001,2002,2003 Mitsunori Miki,    All rights reserved.
// Copyright (C) 2002           Masaki Sano,       All rights reserved.
// Copyright (C) 2002,2003      Junji Sawada,      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
// 3. The names of the authors should not be used to endorse or promote
//    products derived from this software without specific prior
//    written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
// IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// ga2k_ms.h
//-----------------------------------------------------------------------------

#ifndef _GA2K_MS_H_
#define _GA2K_MS_H_

#include <vector>

#include <mpi.h>
#include "ga2k.h"

#define GENE_DATA     1000
#define FITNESS_DATA  2000
#define VARIABLE_DATA 3000
#define EVAL_FINISH   4000


///////////////////////////////////////////////////////////////////////////////
// class MasterSlaveEvaluation
///////////////////////////////////////////////////////////////////////////////

class MasterSlaveEvaluation : public Evaluation
{
  public:
	//data (for mpi)
	int					root;
	int 				nProcs;
	int					myRank;
	char 				hostname[MPI_MAX_PROCESSOR_NAME];

	//method (constructor)
	MasterSlaveEvaluation(int* pargc, char*** argv);
	~MasterSlaveEvaluation();
	void setting(int argc, char *argv[]);  //

	//method
	void initialize();         //GAλϤ˼¹Ԥ
	void start();              //ɾ˼¹Ԥ
	void fitness(Gene& gene);  //ɾ׻Ԥ
	void end();                //ɾθ˼¹Ԥ
	void finalize();           //GAν˼¹Ԥ


  private:
	//data
	vector<Gene*>			pool;
	int						numSlave;
	vector<int>				numSendMax;
	vector<vector<int> >	slaveTable;
	int						sendOut;
	int						currentIndex;

	//method
	void setNumSendMax(int num);

	void masterPart();
	void sendGenes(int dest);
	void getResults(MPI_Status stat);
	void slavePart();
	void getGenes(MPI_Status stat, vector<Gene>& gene);
	void sendResults(vector<Gene>& gene);
};


///////////////////////////////////////////////////////////////////////////////
// class MasterSlaveGlobe
///////////////////////////////////////////////////////////////////////////////

class MasterSlaveGlobe : public Globe
{
  public:
	//data (for mpi)
	int					root;
	int 				nProcs;
	int					myRank;
	char 				hostname[MPI_MAX_PROCESSOR_NAME];

	//method (constructor)
	void initialize();
	void initialize(int argc, char *argv[]);
	void establish();

	//method (operator)
	void execute();
	void showParameters();
	void finalize(bool b);
};


#endif //_GA2K_MS_H_
