//-----------------------------------------------------------------------------
// GA2K
//
// ISDL, Doshisha University Distributed Genetic Algorithm Software
// Version 1.4.1
//-----------------------------------------------------------------------------
// Parallel Genetic Algorithm Team of Intelligent Systems Design Laborato
// ry, Doshisha University.
//
// Copyright (C) 1999           Ikki Ohmukai,      All rights reserved.
// Copyright (C) 1999,2000,2001 Jun-ichi Yoshida,  All rights reserved.
// Copyright (C) 2001,2002,2003 Tomoyuki Hiroyasu, All rights reserved.
// Copyright (C) 2001,2002,2003 Mitsunori Miki,    All rights reserved.
// Copyright (C) 2002           Masaki Sano,       All rights reserved.
// Copyright (C) 2002,2003      Junji Sawada,      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
// 3. The names of the authors should not be used to endorse or promote
//    products derived from this software without specific prior
//    written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
// IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// interface.cpp
//-----------------------------------------------------------------------------

#include <vector>
#include "interface.h"
using namespace std;

#include <iostream>
#include <fstream>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <numeric>
#include <algorithm>
#include <utility>
#include <assert.h>
#include <unistd.h>

#define PROBLEM_NAME "HIDECS"
#define PARAMS_FILE "params_hidecs.ini"

///////////////////////////////////////////////////////////////////////////////
// function
///////////////////////////////////////////////////////////////////////////////

//gray decode
static int gray_decode(const vector<char>& bitString, int startPoint, int bitLength)
{
	int i, real, mask;
	bool sw = false;

	for(i = real = 0; i < bitLength; i++){
		if(bitString[startPoint + i])
			real |= 1 << (bitLength - i - 1);
	}

	for(mask = 1 << (bitLength - 1); mask != 0; mask >>= 1){
		if(sw){
			if(real & mask) sw = false;
			real ^= mask;
		}
		else if(real & mask) sw = true;
	}
	return real;
}

static long round_j(double x)
{
	if(x > 0.0)
		x = floor(x + 0.5);
	if(x < 0.0)
		x = ceil(x - 0.5);
	return (long)x;
}

static const char *cmd_template = "/home/junjis/local/bin/hidecs %s";
static const char *file_inp_template = "file-%s.inp";
static const char *data_inp_template = "data-%s.inp";
static const char *fuel_inp_template = "fuel-%s.inp";
static const char *result_out_template = "result-%s.out";
static const char *opt_out_template = "opt-%s.out";

static const char *file_inp_text =
"**_input file for general data **\n"
"%s\n"
"**_input file for fuel data **\n"
"%s\n"
"**_output_file_1_**\n"
"data1.out\n"
"** output file 2 **\n"
"data2.out\n"
"** output file 3 **\n"
"data3.out\n"
"**_output_file_4_**\n"
"%s\n"
"**_output_file_for_opt_**\n"
"%s\n";
static const char *data_inp_text =
"** INPUT PRINT **\n"
"1\n"
"** OUTPUT PRINT (analyze = 10, opt= 100) **\n"
"100\n"
"** Computing Step **\n"
"0.5\n"
"** PI Cylinder Pressure (K Pa) **\n"
"101.303\n"
"** TI Cylinder Temperature (T) **\n"
"298\n"
"** TF0 Initial Air Temperature (T) **\n"
" 298\n"
"** RPM Speed (Rpm) **\n"
"1800\n"
"** RS Swirl Ratio **\n"
"2.0\n"
"** CRCOMP Woschni's Ratio Before Burn **\n"
"1\n"
"** CRBURN Woschni's Ratio After Burn **\n"
"1\n"
"** TW Wall Temperature (T) **\n"
"473\n"
"** DLEND Smallest Drop **\n"
"5E-006\n"
"** XB Bore (m) **\n"
"0.102\n"
"** XR Stroke(m) **\n"
"0.0525\n"
"** XL Connecting Rod **\n"
"0.165\n"
"** XT Cavity  **\n"
"0.0567\n"
"** EPS Compress Ratio **\n"
"17\n"
"** ND Nozzle Number **\n"
"4\n"
"** DIA Nozzle Diameter **\n"
"0.00018\n"
"** XIVC Inlet Valve Close **\n"
"-145\n"
"** XEVO Exhast Vale Open **\n"
"145\n"
"** CF Contraction Factor **\n"
"0.8\n"
"** BL ?? **\n"
"0.025\n"
"** ZW Thickness of Spray at Wall **\n"
"0.01\n"
"** CWALL Wall Collison Ratio **\n"
"1\n"
"** CBAIR Coeff. of after Wall Impingement **\n"
"1.1\n"
"** CAAIR Coeff. of before Ignition **\n"
"0.6\n"
"** AIRIN Coeff. of Overall Air Entrainment **\n"
"1\n"
"** CTURB Turbulent Intensity in Spray **\n"
"2.0\n"
"** LPACK Number of Division with along to the radius direction **\n"
"10\n"
"** SMD DISTRIBUTION OF THE RADIAL DIRECTION **\n"
"2.5E-005\n"
"2.5E-005\n"
"2.5E-005\n"
"2.5E-005\n"
"2.5E-005\n"
"3E-005\n"
"3E-005\n"
"3E-005\n"
"3E-005\n"
"3E-005\n"
"0\n"
"0\n"
"0\n"
"0\n"
"0\n"
"0\n"
"0\n"
"0\n"
"0\n"
"0\n";
static const char *fuel_inp_text =
"** START ANGLE **\n"
"-5\n"
"** DURATION ANGLE **\n"
"18\n"
"** TOTAL AMOUNT OF FUEL **\n"
"40\n"
"** XO2 ?? **\n"
"0.21\n"
"** XN2 ?? **\n"
"0.79\n"
"** XCO2 ?? **\n"
"0\n"
"** XH2O ?? **\n"
"0\n"
"** Fuel Ratio **\n";

static int execute_hidecs(const double* x, int num, double eval[3])
{
  int i;
  char cmd[80];
  char file_inp[80], data_inp[80], fuel_inp[80], result_out[80], opt_out[80];
  FILE *in_file, *in_data, *in_fuel, *out_result, *out_opt;
  char host[32];

  gethostname(host, sizeof(host));
  sprintf(file_inp, file_inp_template, host);
  sprintf(data_inp, data_inp_template, host);
  sprintf(fuel_inp, fuel_inp_template, host);
  sprintf(result_out, result_out_template, host);
  sprintf(opt_out, opt_out_template, host);
  sprintf(cmd, cmd_template, file_inp);

  /* ϥեν */
  in_file = fopen(file_inp, "w");
  fprintf(in_file, file_inp_text, data_inp, fuel_inp, result_out, opt_out);
  fclose(in_file);
  in_data = fopen(data_inp, "w");
  fprintf(in_data, data_inp_text);
  fclose(in_data);
  in_fuel = fopen(fuel_inp, "w");
  fprintf(in_fuel, fuel_inp_text);
  for(i = 0; i < num; i++)
    fprintf(in_fuel, "%f\n", x[i]);
  fclose(in_fuel);

  /* HIDECS¹ */
  system(cmd);

  /* Ϥɤ߹ߡɾͤ */
  out_opt = fopen(opt_out, "r");
  fscanf(out_opt, "%lf", &eval[0]); /* SFC */
  fscanf(out_opt, "%lf", &eval[1]); /* NOx */
  fscanf(out_opt, "%lf", &eval[2]); /* Soot */
  fclose(out_opt);

  return 0;
}

//=============================================================================
//class ObjectiveProblem
//=============================================================================

void ObjectiveProblem::initialize()
{
	ifstream fin;
	fin.open(PARAMS_FILE);
	if(!fin.is_open()){
		cerr << __FILE__ << ":(" << __LINE__ << ") can't open file "
				<< PARAMS_FILE << endl;
		exit(1);
	}
	int tmp;
	fin >> tmp;
	codingType = static_cast<CodingType>(tmp);
	fin >> nSchedule;
	fin >> nVariable;
	fin >> varLength;
	fin >> tmp;
	optimizationType = static_cast<OptimizationType>(tmp);
	fin >> limSFC;
	fin >> limNOx;
	fin >> limSoot;
	fin >> wSFC;
	fin >> wNOx;
	fin >> wSoot;
	fin.close();
	if(codingType == Traditional && nSchedule != nVariable){
		cerr << "number of variables is not the same as number of schedules" << endl << "Exit" << endl;
		exit(1);
	}
}

void ObjectiveProblem::finalize()
{

}

vector<int> ObjectiveProblem::gene_to_block(const vector<char>& gene)
{
	vector<int> x(nVariable);

	for(int i = 0; i < nVariable; i++)
		x[i] = gray_decode(gene, i * varLength, varLength);
	return x;
}

class ErrorInfo
{
public:
	double err;
	int idx;

	bool operator<(const ErrorInfo& other) const
	{
		return err > other.err;
	}
};

vector<int> ObjectiveProblem::normalizeWidth(const vector<int>& in)
{
	int dim = in.size();
	vector<double> out_f(dim);
	vector<int> out_i(dim);
	vector<ErrorInfo> error(dim);
	int in_sum, out_i_sum;

	in_sum = accumulate(in.begin(), in.end(), 0);
	for(int i = 0; i < dim; i++){
		out_f[i] = ((double)in[i] / in_sum) * nSchedule;
		out_i[i] = (int)round_j(out_f[i]);
		error[i].err = fabs(out_i[i] - out_f[i]);
		error[i].idx = i;
	}
	out_i_sum = accumulate(out_i.begin(), out_i.end(), 0);
	sort(error.begin(), error.end());
	if(out_i_sum > nSchedule){
		int i = 0;
		while(out_i_sum > nSchedule){
			int idx = error[i].idx;
			out_i[idx]--;
			out_i_sum--;
			i++;
		}
	}
	else if(out_i_sum < nSchedule){
		int i = 0;
		while(out_i_sum < nSchedule){
			int idx = error[i].idx;
			out_i[idx]++;
			out_i_sum++;
			i++;
		}
	}
	return out_i;
}

vector<double> ObjectiveProblem::normalizeHeight(const vector<int>& in)
{
	int dim = in.size();
	vector<double> out(dim);

	for(int i = 0; i < dim; i++){
		out[i] = (double)in[i];
	}
	return out;
}

vector<double> ObjectiveProblem::block_to_schedule(const vector<int>& in)
{
	int in_size = in.size();
	vector<int> in_width(in_size / 2), in_height(in_size / 2);
	vector<int> out_width;
	vector<double> out_height;
	vector<double> out(nSchedule);
	int idx_w, idx_h;

	idx_w = idx_h = 0;
	for(int i = 0; i < in_size; i++){
		if(i % 2 == 0)
			in_width[idx_w++] = in[i];
		else
			in_height[idx_h++] = in[i];
	}
	out_width = normalizeWidth(in_width);
	out_height = normalizeHeight(in_height);
	int idx = 0;
	for(int i = 0; i < in_size/2; i++){
		for(int j = 0; j < out_width[i]; j++)
			out[idx++] = out_height[i];
	}
	return out;
}

vector<double> ObjectiveProblem::traditional_decode(const vector<char>& gene)
{
	vector<double> x(nVariable);

	for(int i = 0; i < nVariable; i++)
		x[i] = gray_decode(gene, i * varLength, varLength);
	return x;
}

vector<double> ObjectiveProblem::decode(const vector<char>& chrom)
{
	vector<double> schedule;

	switch(codingType){
	case Traditional:
		schedule = traditional_decode(chrom);
		break;
	case Block:
		{
			vector<int> block;
			block = gene_to_block(chrom);
			schedule = block_to_schedule(block);
			break;
		}
	default:
		cerr << endl << "invalid coding type" << endl << "Exit" << endl;
		exit(1);
	}
	return schedule;
}

vector<double> ObjectiveProblem::evaluate(const vector<double>& x)
{
	vector<double> result(3);
	execute_hidecs(&(x.front()), nSchedule, &(result.front()));
	return result;
}

double ObjectiveProblem::result_to_fitness(const vector<double>& result)
{
	double f;

	switch(optimizationType){
	case SFC:
		f = -result[0];
		if(result[1] >= limNOx)
			f -= (result[1] / limNOx - 1) * wNOx;
		if(result[2] >= limSoot)
			f -= (result[2] / limSoot - 1) * wSoot;
		break;
	case NOx:
		f = -result[1];
		if(result[0] >= limSFC)
			f -= (result[0] / limSFC - 1) * wSFC;
		if(result[2] >= limSoot)
			f -= (result[2] / limSoot - 1) * wSoot;
		break;
	case Soot:
		f = -result[2];
		if(result[0] >= limSFC)
			f -= (result[0] / limSFC - 1) * wSFC;
		if(result[1] >= limNOx)
			f -= (result[1] / limNOx - 1) * wNOx;
		break;
	}
	return f;
}

char* ObjectiveProblem::problemName()
{
	return PROBLEM_NAME;
}

int ObjectiveProblem::numVariable()
{
	return nVariable;
}
