//-----------------------------------------------------------------------------
// GA2K
//
// ISDL, Doshisha University Distributed Genetic Algorithm Software
// Version 1.4.1
//-----------------------------------------------------------------------------
// Parallel Genetic Algorithm Team of Intelligent Systems Design Laborato
// ry, Doshisha University.
//
// Copyright (C) 1999           Ikki Ohmukai,      All rights reserved.
// Copyright (C) 1999,2000,2001 Jun-ichi Yoshida,  All rights reserved.
// Copyright (C) 2001,2002,2003 Tomoyuki Hiroyasu, All rights reserved.
// Copyright (C) 2001,2002,2003 Mitsunori Miki,    All rights reserved.
// Copyright (C) 2002           Masaki Sano,       All rights reserved.
// Copyright (C) 2002,2003      Junji Sawada,      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
// 3. The names of the authors should not be used to endorse or promote
//    products derived from this software without specific prior
//    written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
// IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// interface.h
//-----------------------------------------------------------------------------

#ifndef _INTERFACE_H_
#define _INTERFACE_H_

#include <vector>

using namespace std;

///////////////////////////////////////////////////////////////////////////////
// global variable
///////////////////////////////////////////////////////////////////////////////

enum CodingType {
	Traditional = 0,
	Block = 1
};

enum OptimizationType {
	SFC = 0,
	NOx = 1,
	Soot = 2
};

///////////////////////////////////////////////////////////////////////////////
// class ObjectiveProblem
///////////////////////////////////////////////////////////////////////////////

class ObjectiveProblem
{
public:
	void initialize();                                      //ץ೫ֵ
	void finalize();                                        //ץཪλֵ
	vector<double> decode(const vector<char>& chrom);       //ǥɽ
	vector<double> evaluate(const vector<double>& x);       //Ϸ׻
	double result_to_fitness(const vector<double>& result); //Ŭٷ׻

	char* problemName();                                    //о̾
	int numVariable();                                      //߷ѿο

private:
	//ɬפʥ᥽åɡѿɲ
	vector<double> traditional_decode(const vector<char>& gene);
	vector<int> gene_to_block(const vector<char>& gene);
	vector<int> normalizeWidth(const vector<int>& in);
	vector<double> normalizeHeight(const vector<int>& in);
	vector<double> block_to_schedule(const vector<int>& in);

	CodingType codingType;
	int nSchedule;
	int nVariable;
	int varLength;
	OptimizationType optimizationType;
	double limSFC;
	double wSFC;
	double limNOx;
	double wNOx;
	double limSoot;
	double wSoot;
};

#endif //_INTERFACE_H_
