//-----------------------------------------------------------------------------
// GA2K
//
// ISDL, Doshisha University Distributed Genetic Algorithm Software
// Version 1.4.1
//-----------------------------------------------------------------------------
// Parallel Genetic Algorithm Team of Intelligent Systems Design Laborato
// ry, Doshisha University.
//
// Copyright (C) 1999           Ikki Ohmukai,      All rights reserved.
// Copyright (C) 1999,2000,2001 Jun-ichi Yoshida,  All rights reserved.
// Copyright (C) 2001,2002,2003 Tomoyuki Hiroyasu, All rights reserved.
// Copyright (C) 2001,2002,2003 Mitsunori Miki,    All rights reserved.
// Copyright (C) 2002           Masaki Sano,       All rights reserved.
// Copyright (C) 2002,2003      Junji Sawada,      All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
// 3. The names of the authors should not be used to endorse or promote
//    products derived from this software without specific prior
//    written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
// IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// randring.h
//-----------------------------------------------------------------------------


#ifndef _RANDRING_H_
#define _RANDRING_H_

#include "mtrand.h"

class RandomRing
{
private:
	int *ar;
	int sz;
	MTRand mt;
	int *work;

public:
	void srand(int n) {mt.srand(n);}
	void shuffle(int deepness = 1)
	{
		if(sz == 1)
			return;

		int i;
		for(i = 0; i < deepness * sz;)
		{
			int ind_a = mt.genrand_int(sz);
			int ind_b = mt.genrand_int(sz);
			int& a = work[ind_a];
			int& b = work[ind_b];

			if(a == ind_b || b == ind_a)
				continue;

			int tmp = a;
			a = b;
			b = tmp;
			i++;
		}
		for(i = 0; i < sz; i++)
		  {
		    ar[work[i]] = work[(i+1) % sz];
		  }
	}

	void resize(int n)
	{
		if(ar) delete[] ar;
		ar = new int[n * 2];
		work = ar + n;
		int i;
		for(i = 0; i < n; i++)
			work[i] = (i + 1) % n;

		sz = n;
		shuffle();
	}

	explicit RandomRing(int n) : ar(NULL) {resize(n);}
	RandomRing(int n, int seed) : ar(NULL) {srand(seed); resize(n);}
	~RandomRing() {delete[] ar;}
	inline const int *array() {return ar;}
	inline int operator[](int n) const {return ar[n];}
	inline int size() const {return sz;}
};

#endif // _RANDRING_H_
