#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "mtrand.h"
#include "classic.h"

//ѿͤž뤿δؿ֤줿̤ϼǤǲ褦ˡ
double *rotate_variable(const double *x, int dim, double arg)
{
	int i, j;
	double tmp1, tmp2;
	double *trans;
	double cos_value = cos(arg);
	double sin_value = sin(arg);
	
	trans = (double *)malloc(sizeof(double)*dim);
	memcpy(trans,x,sizeof(double)*dim);
	for (i=0; i<dim-1; i++){
		for (j=i+1; j<dim; j++) {
			tmp1 = cos_value*trans[i] - sin_value*trans[j];
			tmp2 = sin_value*trans[i] + cos_value*trans[j];
			trans[i] = tmp1;
			trans[j] = tmp2;
		}
	}
	return trans;
}

double gaussian(double m, double s)
{
	int i;
	double f = 0.0;
	
	for(i=0; i<12; i++)
		f += genrand();
	f -= 6;
	f = s * f + m;
	return f;
}

double rastrigin(const double *x, int dim)
{
	int i;
	double f;
	
	f = 10.0 * (double)dim;
	for(i=0; i<dim; i++)
		f += x[i] * x[i] - 10.0 * cos(2.0 * PI * x[i]);
	return -f;
}

double rosenbrock(const double *x, int dim)
{
	int i;
	double f = 0.0;
	
	for(i=0; i<dim-1; i++)
		f += 100.0 * (x[i] * x[i] - x[i+1]) * (x[i] * x[i] - x[i+1]) + (1.0 - x[i]) * (1.0 - x[i]);
	return -f;
}

double simple_rosenbrock(const double *x, int dim)
{
	int i;
	double f = 0.0;
	
	for(i=1; i<dim; i++)
		f += 100.0 * (x[i] * x[i] - x[0]) * (x[i] * x[i] - x[0]) + (1.0 - x[i]) * (1.0 - x[i]);
	return -f;
}

double griewank(const double *x, int dim)
{
	int i;
	double f, f1 = 0.0, f2 = 1.0;
	
	for(i=0; i<dim; i++){
		f1 += x[i] * x[i] / 4000.0;
		f2 *= cos(x[i] / sqrt(i + 1));
	}
	f = 1.0 + f1 - f2;
	return -f;
}

double ridge(const double *x, int dim)
{
	int i, j;
	double f = 0.0, f1;
	
	for(i=0; i<dim; i++){
		f1 = 0.0;
		for(j=0; j<=i; j++)
			f1 += x[j];
		f += f1 * f1;
	}
	return -f;
}

double schwefel(const double *x, int dim)
{
	int i;
	double f, f1 = 0.0;
	static const double schw_ans = -418.98276401614430142217315733432769775390625;
	
	for(i=0; i<dim; i++)
		f1 -= x[i] * sin(sqrt(fabs(x[i])));
	f = f1 - schw_ans * (double)dim;
	return -f;
}

double sphere(const double *x, int dim)
{
	int i;
	double f = 0.0;
	
	for(i=0; i<dim; i++)
		f += x[i] * x[i];
	return -f;
}

double weighted_sphere(const double *x, int dim)
{
	int i;
	double f=0.0;
	
	for(i=0; i<dim; i++)
		f += (double)(i+1) * x[i] * x[i];
	return -f;
}

double step(const double *x, int dim)
{
	int i;
	double f = 0.0;

	for(i=0; i<dim; i++)
		f += floor(x[i]);
	f += 6.0 * (double)dim;
	return -f;
}

double quartic(const double *x, int dim)
{
	int i;
	double f, f1 = 0.0;

	for(i=0; i<dim; i++)
		f1 += ((i+1)*x[i]*x[i]*x[i]*x[i]);

	f = f1 + gaussian(0,1);
	return -f;
}

double ackley(const double *x, int dim)
{
	int i;
	double f, f1=0.0, f2=0.0;
	static const double a = 20.0, b = 0.2, c = 2.0 * PI;
	
	for(i=0; i<dim; i++){
		f1 += x[i]*x[i];
		f2 += cos(c*x[i]);
	}
	f = -a*exp(-b*sqrt(f1/(double)dim)) - exp(f2/(double)dim) + a + exp(1);
	return -f;
}

double bohachevsky(const double *x, int dim)
{
	int i;
	double f = 0.0;
	
	for(i=0; i<dim-1; i++)
		f += x[i]+x[i] + 2.0*x[i+1]*x[i+1] - 0.3*cos(3*PI*x[i]) - 0.4*cos(4*PI*x[i+1]) + 0.7;
	return -f;
}

double rotated_rastrigin(const double *x, int dim)
{
	double f;
	double *trans;
	
	trans = rotate_variable(x,dim,PI/4);
	f = rastrigin(trans,dim);
	free(trans);
	return f;
}

double rotated_schwefel(const double *x, int dim)
{
	double f;
	double *trans;
	
	trans = rotate_variable(x,dim,PI/4);
	f = schwefel(trans,dim);
	free(trans);
	return f;
}

double rotated_ackley(const double *x, int dim)
{
	double f;
	double *trans;
	
	trans = rotate_variable(x,dim,PI/4);
	f = ackley(trans,dim);
	free(trans);
	return f;
}

double rotated_bohachevsky(const double *x, int dim)
{
	double f;
	double *trans;
	
	trans = rotate_variable(x,dim,PI/4);
	f = bohachevsky(trans,dim);
	free(trans);
	return f;
}
