
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <mpi.h>

#include "grpc.h"

#define N 12

int
main (int argc, char *argv[])
{
  grpc_function_handle_t handle;
  double result, correct_result;
  grpc_sessionid_t sessionID;
  int i, status;
  double dx[N];
  double dy[N];

  if(grpc_initialize(NULL) != GRPC_NO_ERROR) {
    grpc_perror("grpc_initialize");
    exit(EXIT_FAILURE);
  }

  if(grpc_function_handle_default(&handle, "simple_mpi") != GRPC_NO_ERROR) {
    fprintf(stderr,"Error creating function handle\n");
    exit(EXIT_FAILURE);
  }

  for(i=0;i<N;i++) {
    dx[i] = i+10.0;
    dy[i] = i+1000.0;
  }

  status = grpc_call_async(&handle, &sessionID, dx, dy, N, &result);

  if(status != GRPC_NO_ERROR) {
    printf("GRPC error status  = %d\n", status);
    grpc_perror("grpc_call_async");
    exit(EXIT_FAILURE);
  }

  printf("Waiting for result.\n");
	    
  status = grpc_wait(sessionID);
	        
  if(status != GRPC_NO_ERROR) {
    grpc_perror("grpc_wait");
    exit(EXIT_FAILURE);
  }

  grpc_function_handle_destruct(&handle);

  correct_result = 0.0;
  for(i=0;i<N;i++)
    correct_result += dx[i] * dy[i];

  printf("result = %g, correct_result = %g\n", result, correct_result);

  exit(EXIT_SUCCESS);
}
