#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <mpi.h>

extern int gs_argc;
extern char **gs_argv;

double
simple_mpi(double *dx, double *dy, int n)
{
  double dtemp = 0.0;
  const int tag = 42;
  int id, ntasks, source_id, dest_id, err, i;
  MPI_Status status;
  double msg;
  int els_per_node;

  err = MPI_Init(&gs_argc, &gs_argv);
  if(err != MPI_SUCCESS) {
    printf ("MPI initialization failed!\n");
    return -1;
  }

  err = MPI_Comm_size (MPI_COMM_WORLD, &ntasks);
  err = MPI_Comm_rank (MPI_COMM_WORLD, &id);

  if(ntasks < 2) {
    for(i=0;i<n;i++)
      dtemp += dx[i] * dy[i];

    MPI_Finalize ();
    return dtemp;
  }

  dtemp = 0.0;
  els_per_node = n/ntasks;

  if(id == 0) {
    int start_idx, end_idx;

    start_idx = els_per_node * (ntasks-1);
    end_idx = n;

    for(i=start_idx;i<end_idx;i++)
      dtemp += dx[i] * dy[i];

    for (i = 1; i < ntasks; i++)
    {
      err = MPI_Recv (&msg, 1, MPI_DOUBLE, MPI_ANY_SOURCE, tag, MPI_COMM_WORLD, &status);
      source_id = status.MPI_SOURCE;
      dtemp += msg;
    }
  }
  else {
    int start_idx, end_idx;

    start_idx = els_per_node * (id-1);
    end_idx = els_per_node * (id);

    for(i=start_idx;i<end_idx;i++)
      dtemp += dx[i] * dy[i];

    dest_id = 0;
    err = MPI_Send (&dtemp, 1, MPI_DOUBLE, dest_id, tag, MPI_COMM_WORLD);
  }

  err = MPI_Finalize ();

  if (id != 0)
    exit(0);

  return dtemp;
}
