#define MONTAGE_BINARY_DEFAULT "montage"
#define CONVERT_BINARY_DEFAULT "convert"
// "/cygdrive/c/Download/GridSolve/ImageMagick-6.2.5/bin/convert.exe" 
#define MPEGENCODE_BINARY_DEFAULT "/home/seymour/stuff/mpeg_encode/mpeg_encode/mpeg_encode"
#define MMV_BINARY_DEFAULT "mmv"

#define RC_FILENAME ".gspovrayrc"

#define FILE_FORMAT "tga"

#define TMP_POVRAY_ALL       ".gs_povray.*"
#define TMP_POVRAY_INI       ".gs_povray.ini"
#define TMP_POVRAY_INC       ".gs_povray.inc"
#define TMP_RETURN_FILE      "return0.tar"

#define ANIMATION_PARM       "animation.parm"
#define CUSTOM_PARM          "custom.parm"

#define GS_MODE_BLOCKING        1
#define GS_MODE_NONBLOCKING     2
#define GS_MODE_FARMING         3

#define POV_MODE_SINGLE_IMG     1
#define POV_MODE_ANIMATION      2

#define DEBUG_OFF               0
#define DEBUG_ON                1

#define RENDER_MODE_REMOTE      1
#define RENDER_MODE_DISTRIBUTED 2

#define RETVAL_STANDARD			0
#define RETVAL_NEXT             1

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <pwd.h>
#include <ctype.h>

#include "grpc.h"

/* this struct holds command line argument info */

typedef struct {
  int server;
  int width_part, height_part;
  int frame_part;
  int first_frame, last_frame;
  double init_clock, final_clock;
  int gs_mode, pov_mode, render_mode;
  int width, height;
  char output[128], localPath[512];
  char povray_options[128];
  int timeout;
  int debug;
} arg_info_t;

FILE
  *open_rc_file();

int 
  parse_rc_file(),
  parseInput(int, char **, arg_info_t *),
  optionHandling(FILE *, int, char **, arg_info_t *),
  compose_output(arg_info_t *),
  make_call(arg_info_t *),
  make_blocking_call(arg_info_t *),
  make_nonblocking_call(arg_info_t *),
  make_farming_call(arg_info_t *);

char 
  *get_home_dir(),
  *getFilename(char *);

void
  clean(),
  version(),
  usage(),
  help(char **),
  addFile(char *, arg_info_t *),
  addDir(char *, arg_info_t *),
  getLocalPath(char *, arg_info_t *);

char *MONTAGE_BINARY = NULL;
char *CONVERT_BINARY = NULL;
char *MPEGENCODE_BINARY = NULL;
char *MMV_BINARY = NULL;

int 
main(int argc, char **argv)
{
  arg_info_t arg_info;
  struct stat buf;

  if(parse_rc_file() < 0) {
    MONTAGE_BINARY = MONTAGE_BINARY_DEFAULT;
    CONVERT_BINARY = CONVERT_BINARY_DEFAULT;
    MPEGENCODE_BINARY = MPEGENCODE_BINARY_DEFAULT;
    MMV_BINARY = MMV_BINARY_DEFAULT;
  }

  if(grpc_initialize(NULL) != GRPC_NO_ERROR) {
    grpc_perror("grpc_initialize");
    exit(EXIT_FAILURE);
  }

  /* create ini-file from the commandline arguments that povray can be
   * invoked by 'povray file.ini' 
   */

  if(parseInput(argc, argv, &arg_info) < 0) {
    fprintf(stderr, "Error: invalid options!\n");
    exit(EXIT_FAILURE);
  }

  // config output
  if (arg_info.debug == DEBUG_ON){
    fprintf(stderr, "-- configuration --\n");
    fprintf(stderr, "montage: %s\n", MONTAGE_BINARY);
	fprintf(stderr, "convert: %s\n", CONVERT_BINARY);
	fprintf(stderr, "mpegencode: %s\n", MPEGENCODE_BINARY);
	fprintf(stderr, "mmv: %s\n", MMV_BINARY);
  }

  if(make_call(&arg_info) < 0) {
    fprintf(stderr, "Error: failed to make call\n");
    exit(EXIT_FAILURE);
  }

  if(compose_output(&arg_info) < 0) {
    fprintf(stderr, "error: failure on composing the image!\n");
    exit(EXIT_FAILURE);
  }

  /* cleaning temporary files and exiting with status message */
  if (arg_info.debug == DEBUG_OFF)
    clean();

  if(stat(arg_info.output, &buf) == 0) {
    printf("GridSolve/PovRay successfully finished;");
    printf(" output saved in %s\n", arg_info.output);
  }
  else
    fprintf(stderr, "error: no output was generated\n");

  exit(EXIT_SUCCESS);
}

int 
parseInput(int argc, char **argv, arg_info_t *a)
{
  int i;
  FILE *iniFile;

  if (argc < 2) {
    usage(argv);
    fprintf(stdout,"Try '%s --help' for more information.\n",argv[0]);
    exit(EXIT_FAILURE);
  }

  // initialize configuration
  a->server = 1;
  a->gs_mode = GS_MODE_BLOCKING;
  a->pov_mode = POV_MODE_SINGLE_IMG;
  a->width = 333;
  a->height = 222;
  a->width_part = 333;
  a->height_part = 222;
  a->frame_part = 1;
//  a->debug = DEBUG_OFF;
  a->debug = DEBUG_ON;
  a->output[0] = 0;

  iniFile = fopen(TMP_POVRAY_INI,"w");
  for(i=1; i<argc; i++)
    if (optionHandling(iniFile, i, argv, a) == RETVAL_NEXT)
      i++;
  fclose(iniFile);

  /* check if it is really an animation */
  if (a->pov_mode == 2) {
    if ((a->first_frame == a->last_frame) && 
		(a->init_clock == 0.0) && (a->final_clock == 0.0))
      a->pov_mode = POV_MODE_SINGLE_IMG;
  }

  if((a->pov_mode == POV_MODE_ANIMATION) && (a->gs_mode == GS_MODE_BLOCKING))
    a->gs_mode = GS_MODE_NONBLOCKING;
  if((a->server > 1) && (a->gs_mode == GS_MODE_BLOCKING))
	a->gs_mode = GS_MODE_NONBLOCKING;

  sprintf(a->povray_options, "-DG +FT +H%d +W%d", a->height, a->width);

  // option output
  if (a->debug == DEBUG_ON){
    fprintf(stderr, "-- options --\n");
    fprintf(stderr, "server: %d\n", a->server);
	fprintf(stderr, "width: %d\n", a->width);
	fprintf(stderr, "height: %d\n", a->height);
	fprintf(stderr, "width-part: %d\n", a->width_part);
	fprintf(stderr, "height-part: %d\n", a->height_part);
  //int frame_part;
  //int first_frame, last_frame;
  //double init_clock, final_clock;
  //int gs_mode, pov_mode, render_mode;
  //char output[128], localPath[512];
  //char povray_options[128];
  //int timeout;
  //int debug;
  }

  return 0;
}

/*
  return value: 
    STANDARD - processed one argument (e.g., +Iicl.pov)
	NEXT     - processed two arguments (e.g., --dist 2)
*/
int 
optionHandling(FILE *iniFile, int i, char **argv, arg_info_t *a) {
  FILE *customIni;
  char *option, *part, *tok1, *tok2, *tok3, *value, *filename;
  char cmd[10000]="";
  char value2[1024]="";
  char **argvtmp;
  int retval = RETVAL_STANDARD;

  option = strdup(argv[i]);
  part = strdup(option);
  tok1 = strtok(part, "=");
  tok2 = strtok(NULL, "=");
  if (a->debug == DEBUG_ON)
    fprintf(stderr,"%s (%s %s)\n",option,tok1 ? tok1 : "null",tok2 ? tok2 : "null");

  /******** gspovray Options ********/
  if(strcmp(option,"--clean") == 0) {
    clean();
    free(option);
    free(part);
    exit(EXIT_SUCCESS);

  } else if(strcmp(option,"--debug") == 0) {
    a->debug = DEBUG_ON;

  } else if(strcmp(option,"--dist") == 0) {
    a->render_mode = RENDER_MODE_DISTRIBUTED;
    a->server = atoi(argv[i+1]);
    if (a->debug == DEBUG_ON)
		fprintf(stderr,"dist: %d %d\n",a->render_mode,a->server);
    retval = RETVAL_NEXT;

  } else if(strcmp(option,"--tile") == 0) {
    part = strdup(argv[i+1]);
    tok1 = strtok(part, "x");
    tok2 = strtok(NULL, "x");
    if (a->debug == DEBUG_ON)
      fprintf(stderr,"%s (%s %s)\n",part,tok1,tok2);
    a->width_part = atoi(tok1);
    a->height_part = atoi(tok2);
    retval = RETVAL_NEXT;

  } else if(strcmp(option,"--frame") == 0) {
    a->frame_part = atoi(argv[i+1]);
    retval = RETVAL_NEXT;

  } else if(strcmp(option,"--timeout") == 0) {
    a->timeout = atoi(argv[i+1]);
    retval = RETVAL_NEXT;

  //} else if(strcmp(option,"--gridsolve") == 0) {
  //  transport = 0;

  //} else if(strcmp(option,"--ssh") == 0) {
  //  transport = 1;
  //  sprintf(sshHostsFile, "%s", argv[i+1]);
  //  retval = 1;

  //} else if(strcmp(option,"--kpm") == 0) {
  //  kpm_mode = 1;

  } else if(strcmp(option,"--help") == 0) {
    usage(argv);
    help(argv);
    free(option);
    free(part);
    exit(0);

  } else if(strcmp(option,"--version") == 0) {
    version();
    fprintf(stdout,"This program comes with NO WARRANTY, to the extent permitted by law.\n");
    fprintf(stdout,"You may redistribute it under the terms of the GNU General Public License;\n");
    fprintf(stdout,"see the file named COPYING for details.\n");
    fprintf(stdout,"Written by Keith Seymour and Christoph Fabianek.\n");
    free(option);
    free(part);
    exit(0);

  } else
  /******** POV-Ray Options ********/
  /* Input_File_Name ---------------------------------------------------------*/
  if ((((option[0] == '+') || (option[0] == '-')) &&
       ((option[1] == 'I') || (option[1] == 'i'))) ||
      (strcmp(tok1,"Input_File_Name") == 0)) {
    if (strcmp(tok1,"Input_File_Name") == 0)
      value = strdup(tok2);
    else
      value = strdup(&option[2]);
    if (value[0] == '/')
      sprintf(value2, "%s", &value[1]);
	else {
	  getLocalPath(option,a);
      sprintf(value2,"%s%s",a->localPath,value);
	}
    addFile(value, a);
    if (value2[0] == '/')
      sprintf(value2, "%s", &value2[1]);
    fprintf(iniFile,"Input_File_Name=%s\n", value2);

    if(strcmp(a->output,"") == 0) {
      filename = getFilename(value2);
      strcpy(a->output,filename);
	  a->output[strlen(filename)-3] = '\0';
      sprintf(a->output,"%s%s",a->output,FILE_FORMAT);
      free(filename);
    }
    free(value);
  } else

  /* Include_Header ----------------------------------------------------------*/
  if ((((option[0] == '+') || (option[0] == '-')) &&
       ((option[1] == 'H') || (option[1] == 'h')) &&
       ((option[2] == 'I') || (option[2] == 'i'))) ||
      (strcmp(tok1,"Include_Header") == 0)) {
    if (strcmp(tok1,"Include_Header") == 0)
      value = strdup(tok2);
    else
      value = strdup(&option[2]);
    if (value[0] == '/')
      sprintf(value2, "%s", &value[1]);
    else
      sprintf(value2,"%s%s",a->localPath,value);

    addFile(value, a);
    if (value2[0] == '/')
      sprintf(value2, "%s", &value2[1]);
    fprintf(iniFile,"Include_Header=%s\n",value2);
    free(value);
  } else

  /* Library_Path ------------------------------------------------------------*/
  if ((((option[0] == '+') || (option[0] == '-')) &&
       ((option[1] == 'L') || (option[1] == 'l'))) ||
      (strcmp(tok1,"Library_Path") == 0)) {
    if (strcmp(tok1,"Library_Path") == 0)
      value = strdup(tok2);
    else
      value = strdup(&option[2]);
    if (value[0] == '/')
      sprintf(value2, "%s", &value[1]);
    else
      sprintf(value2,"%s%s",a->localPath,value);

    addDir(value, a);
    if (value2[0] == '/')
      sprintf(value2, "%s", &value2[1]);
    fprintf(iniFile,"Library_Path=%s\n",value2);
    free(value);
  } else

  /* Output_File_Name --------------------------------------------------------*/
  if ((((option[0] == '+') || (option[0] == '-')) &&
       ((option[1] == 'O') || (option[1] == 'o'))) ||
      (strcmp(tok1,"Output_File_Name") == 0)) {
    if (strcmp(tok1,"Output_File_Name") == 0)
      value = strdup(tok2);
    else
      value = strdup(&option[2]);

    filename = getFilename(value);
    fprintf(iniFile,"Output_File_Name=%s\n",filename);
    sprintf(a->output,"%s",filename);
    free(value);
    free(filename);
  } else

  /* Width -------------------------------------------------------------------*/
  if ((((option[0] == '+') || (option[0] == '-')) &&
       ((option[1] == 'W') || (option[1] == 'w'))) ||
      (strcmp(tok1,"Width") == 0)) {
    if (strcmp(tok1,"Width") == 0)
      value = strdup(tok2);
    else
      value = strdup(&option[2]);

    a->width = atoi(value);
    fprintf(iniFile,"Width=%d\n",a->width);
    free(value);
  } else

  /* Height ------------------------------------------------------------------*/
  if ((((option[0] == '+') || (option[0] == '-')) &&
       ((option[1] == 'H') || (option[1] == 'h'))) ||
      (strcmp(tok1,"Height") == 0)) {
    if (strcmp(tok1,"Height") == 0)
      value = strdup(tok2);
    else
      value = strdup(&option[2]);

    a->height = atoi(value);
    fprintf(iniFile,"Height=%d\n", a->height);
    free(value);
  } else

  /* Initial_Frame -----------------------------------------------------------*/
  if ((((option[0] == '+') || (option[0] == '-')) &&
       ((option[1] == 'K') || (option[1] == 'k')) &&
       ((option[2] == 'F') || (option[2] == 'f')) &&
       ((option[3] == 'I') || (option[3] == 'i'))) ||
      (strcmp(tok1,"Initial_Frame") == 0)) {
    if (strcmp(tok1,"Initial_Frame") == 0)
      value = strdup(tok2);
    else
      value = strdup(&option[4]);

    a->first_frame = atoi(value);
    fprintf(iniFile,"Initial_Frame=%d\n", a->first_frame);
    a->pov_mode=POV_MODE_ANIMATION;
    free(value);
  } else

  /* Final_Frame -------------------------------------------------------------*/
  if ((((option[0] == '+') || (option[0] == '-')) &&
       ((option[1] == 'K') || (option[1] == 'k')) &&
       ((option[2] == 'F') || (option[2] == 'f')) &&
       ((option[3] == 'F') || (option[3] == 'f'))) ||
      (strcmp(tok1,"Final_Frame") == 0)) {
    if (strcmp(tok1,"Final_Frame") == 0)
      value = strdup(tok2);
    else
      value = strdup(&option[4]);

    a->last_frame = atoi(value);
    fprintf(iniFile,"Final_Frame=%d\n", a->last_frame);
    a->pov_mode=POV_MODE_ANIMATION;
    free(value);
  } else

  /* Initial_Clock -----------------------------------------------------------*/
  if ((((option[0] == '+') || (option[0] == '-')) &&
       ((option[1] == 'K') || (option[1] == 'k')) &&
       ((option[2] == 'I') || (option[2] == 'i'))) ||
      (strcmp(tok1,"Initial_Clock") == 0)) {
    if (strcmp(tok1,"Initial_Clock") == 0)
      value = strdup(tok2);
    else
      value = strdup(&option[3]);

    a->init_clock = atof(value);
    fprintf(iniFile,"Initial_Clock=%f\n", (double)a->init_clock);
    a->pov_mode=POV_MODE_ANIMATION;
    free(value);
  } else

  /* Final_Clock -------------------------------------------------------------*/
  if ((((option[0] == '+') || (option[0] == '-')) &&
       ((option[1] == 'K') || (option[1] == 'k')) &&
       ((option[2] == 'F') || (option[2] == 'f'))) ||
      (strcmp(tok1,"Final_Clock") == 0)) {
    if (strcmp(tok1,"Final_Clock") == 0)
      value = strdup(tok2);
    else
      value = strdup(&option[3]);

    a->final_clock = atof(value);
    fprintf(iniFile,"Final_Clock=%f\n", (double)a->final_clock);
    a->pov_mode=POV_MODE_ANIMATION;
    free(value);
  } else

  /* generic +/- options -----------------------------------------------------*/
  if ((option[0] == '+') || (option[0] == '-')) {
    fprintf(iniFile,"%s\n", option);
  } else

  /* if it is an INI file parse it recursively -------------------------------*/
  {
    customIni = fopen(option,"r");
    getLocalPath(option,a);
    if (customIni != NULL) {
      while (fscanf(customIni,"%[^\n]\n",cmd) != EOF) {
        tok1=strtok(cmd, "\r");
        tok2=strdup(tok1);
        tok3=strdup(&cmd[strlen(tok2)+1]);
        for (i=strlen(tok3);i<10000;i++)
          cmd[i] = '\0';
        sprintf(cmd,"%s",tok3);
        while (tok1 != NULL) {
          if (tok2[0] != ';') {
            argvtmp = malloc(sizeof(char *));
            argvtmp[0] = strdup(tok2);
            if (optionHandling(iniFile, 0, argvtmp, a) == RETVAL_NEXT)
              fprintf(iniFile,"%s\n", cmd);
            free(argvtmp[0]);
            free(argvtmp);
          }
          free(tok2);
          free(tok3);
          tok1=strtok(cmd, "\r");
          if (cmd[0] == '\0')
            break;
          if (tok1 != NULL) {
            tok2=strdup(tok1);
            sprintf(cmd,"%s",&cmd[strlen(tok1)+1]);
          }
        }
      }
    } else
      retval = RETVAL_NEXT;
  }
  free(option);
  free(part);

  return retval;
}

int
compose_single_image(arg_info_t *args)
{
  int i,j;
  int rows,columns;
  int tile_width, tile_height;
  char cmd[1024];
  char im_cmd[1024];

  sprintf(im_cmd,"%s &> /dev/null",MONTAGE_BINARY);
  if (args->debug == DEBUG_ON)
    fprintf(stderr, "montage test: %s\n", im_cmd);
  if (system(im_cmd) != 0) {
    fprintf(stderr, "error: executable '%s' not found!\n", im_cmd);
    exit(EXIT_FAILURE);
  }
  sprintf(im_cmd,"%s &> /dev/null",CONVERT_BINARY);
  if (system(im_cmd) != 0) {
    fprintf(stderr, "error: executable '%s' not found!\n", CONVERT_BINARY);
    exit(EXIT_FAILURE);
  }

  columns = ceil((double)args->width / (double)args->width_part);
  rows = ceil((double)args->height / (double)args->height_part);
  if (args->debug == DEBUG_ON)
    fprintf(stderr, "cols/rows: %d/%d\n", columns, rows);

  sprintf(im_cmd,"%s -crop %dx%d+0+0 -geometry +0+0 -tile %dx%d",
          MONTAGE_BINARY,args->width_part,args->height_part,columns,rows);
  if (args->debug == DEBUG_ON)
    fprintf(stderr, "montage: %s\n", im_cmd);

  for(i=0; i<rows; i++){
    for(j=0; j<columns; j++){
	  // unpack
	  if (args->debug == DEBUG_ON) {
	    sprintf(cmd, "tar xf return%d.tar",i*columns+j);
        fprintf(stderr, "%s\n", cmd);
	  } else
	    sprintf(cmd, "tar xf return%d.tar &> /dev/null",i*columns+j);
      system(cmd);

	  // crop
      tile_width = MIN((j+1)*args->width_part,args->width) - j*args->width_part;
      tile_height = MIN((i+1)*args->height_part,args->height) - i*args->height_part;
      sprintf(cmd, "%s outfile.%s -crop %dx%d+%d+0 part%d.%s &> /dev/null",
              CONVERT_BINARY,FILE_FORMAT,tile_width,tile_height,
			  args->width_part*j,i*columns+j,FILE_FORMAT);
      system(cmd);
      if (args->debug == DEBUG_ON)
		fprintf(stderr, "crop: %s\n", cmd);

	  // build composition command
      sprintf(im_cmd,"%s part%d.%s",im_cmd,i*columns+j,FILE_FORMAT);
      if (args->debug == DEBUG_ON)
        fprintf(stderr, "montage: %s\n", im_cmd);
    }
  }

  sprintf(im_cmd,"%s %s &> /dev/null", im_cmd, args->output);
  system(im_cmd);
  if (args->debug == DEBUG_ON)
    fprintf(stderr, "montage: %s\n", im_cmd);

  sprintf(cmd,"%s -crop %dx%d+0+0 -geometry +0+0 %s %s",MONTAGE_BINARY, 
		  args->width, args->height, args->output, args->output);
  system(cmd);

  return 0;
}

int
compose_animation(arg_info_t *args)
{
  FILE *animation_parm, *custom_parm;
  char cmd[1024];
  int i, retval;

  for(i=0; i<args->server; i++){
    sprintf(cmd, "tar xf return%d.tar &> /dev/null",i);
    system(cmd);
  }
  animation_parm = fopen(ANIMATION_PARM, "r");
  custom_parm = fopen(CUSTOM_PARM, "w");
  while (fscanf(animation_parm,"%[^\n]\n",cmd) != EOF) {
    if (strcmp(cmd,"outfile*.tga [001-999]") == 0) {
      fprintf(custom_parm,"outfile*.tga [%03d-%03d]\n",
              args->first_frame,args->last_frame);
    } else {
      fprintf(custom_parm,"%s\n",cmd);
    }
  }
  fclose(animation_parm);
  fclose(custom_parm);

  /* check if first frames have wrong number... 01 -> 001 */
  if(args->last_frame < 10) {
    for(i=args->first_frame; i <= args->last_frame; i++) {
      sprintf(cmd,"mv outfile%d.tga outfile00%d.tga &> /dev/null",
              i ,i);
      system(cmd);
    }
  }
  else if(args->last_frame < 100) {
    for(i=args->first_frame; i < 10; i++) {
      sprintf(cmd,"mv outfile0%d.tga outfile00%d.tga &> /dev/null",
              i ,i);
      system(cmd);
    }
    for(i=10; i <= args->last_frame; i++) {
      sprintf(cmd,"mv outfile%d.tga outfile0%d.tga &> /dev/null",
              i ,i);
      system(cmd);
    }
  }

  /* if the outputfilename is *.mpg or *.mpeg use mpeg_encode otherwise
     use mmv to just rename the files */
  if ((strcmp(&args->output[strlen(args->output) - 4],".mpg") == 0) ||
      (strcmp(&args->output[strlen(args->output) - 5],".mpeg") == 0)) {
    sprintf(cmd,"%s &> /dev/null", MPEGENCODE_BINARY);
    retval=system(cmd);
    if ((retval != 0) && (retval != 256)) {
      fprintf(stderr, "error: executable '%s' not found!\n", MPEGENCODE_BINARY);
      return -1;
    }
    sprintf(cmd, "%s %s", MPEGENCODE_BINARY, CUSTOM_PARM);
    system(cmd);
    sprintf(cmd,"mv .gsanimation.mpg %s", args->output);
    system(cmd);
  } else {
    sprintf(cmd, "%s \"outfile*.tga\" \"%s#1.tga\"",MMV_BINARY,args->output);
    retval = system(cmd);
    if (retval != 0) {
      fprintf(stderr, "error: executable '%s' not found or problems when renaming the files!\n",
              MMV_BINARY);
      return -1;
    }
    if (args->last_frame < 10) {
      sprintf(args->output, "%s00%d.tga",args->output,args->last_frame);
    } else if (args->last_frame < 100) {
      sprintf(args->output, "%s0%d.tga",args->output,args->last_frame);
    } else {
      sprintf(args->output, "%s%d.tga",args->output,args->last_frame);
    }
  }

  return 0;
}

int 
compose_output(arg_info_t *args)
{

  switch(args->pov_mode) {
    case POV_MODE_SINGLE_IMG:
      return compose_single_image(args);

    case POV_MODE_ANIMATION:
      return compose_animation(args);

    default:
      fprintf(stderr, "error: invalid pov_mode!\n");
      return -1;
      break;
  }

  return 0;
}

int
make_call(arg_info_t *args)
{
  int rv = -1;

  if (args->debug == DEBUG_ON)
    fprintf(stderr, "%d\n", args->gs_mode);

  switch(args->gs_mode) {
    case GS_MODE_BLOCKING:
      rv = make_blocking_call(args);
      break;
    case GS_MODE_NONBLOCKING:
      rv = make_nonblocking_call(args);
      break;
    case GS_MODE_FARMING:
      rv = make_farming_call(args);
      break;
    default:
      fprintf(stderr, "Error: invalid gs_mode!\n");
      return -1;
      break;
  }

  return rv;
}

int
make_blocking_call(arg_info_t *args)
{
  grpc_function_handle_t handle;
  int info;

  args->server = 1;
  args->height_part = args->height;
  args->width_part = args->width;

  if(grpc_function_handle_default(&handle, "povray") != GRPC_NO_ERROR) {
    fprintf(stderr,"Error creating function handle\n");
    return -1;
  }

  info = grpc_call(&handle, args->povray_options, 
     strlen(args->povray_options) + 1, TMP_POVRAY_INI, 
     TMP_POVRAY_INC, TMP_RETURN_FILE);

  if(info != GRPC_NO_ERROR) {
    grpc_perror("grpc_call");
    return -1;
  }

  grpc_function_handle_destruct(&handle);

  return 0;
}

int
make_nonblocking_call(arg_info_t *args)
{
  grpc_function_handle_t *handles;
  grpc_sessionid_t sessionID;
  grpc_error_t *status_array;
  char **myretfile = NULL;
  char myinifile[128]="";
  char filename[128]="";
  char cmdstr[1024]="";
  //char povcmd[2048]="";
  double clock_part;
  char cmd[1024];
  FILE *iniFile;
  int i;
  int row, col, rows, columns;
  int sr,er,sc,ec;

  printf ("using nonblocking calls with %d server(s)...\n", args->server);
  status_array = (grpc_error_t *)calloc(args->server, sizeof(grpc_error_t));
  handles = (grpc_function_handle_t *)calloc(args->server, 
     sizeof(grpc_function_handle_t));

  myretfile = (char **)malloc(args->server * sizeof(char *));
      
  for(i=0; i<args->server; i++) {
    sprintf(filename,"%s%d", TMP_POVRAY_INI, i);
    sprintf(cmd,"cp %s %s", TMP_POVRAY_INI, filename);
    system(cmd);
    iniFile = fopen(filename,"a");
    switch(args->pov_mode){
      case POV_MODE_SINGLE_IMG: /* rendering a single image */
        columns = ceil((double)args->width / (double)args->width_part);
        rows = ceil((double)args->height / (double)args->height_part);
        row = i / columns;
        col = i - (row*columns);
        sc = col*args->width_part+1; 
        ec = MIN((col+1)*args->width_part,args->width);
        sr = row*args->height_part+1; 
        er = MIN((row+1)*args->height_part,args->height);
        sprintf(cmdstr,"%s +SC%d +EC%d +SR%d +ER%d",args->povray_options,sc,ec,sr,er);
        break;
      case POV_MODE_ANIMATION: /* rendering an animation */
        args->frame_part=(int)((args->last_frame-args->first_frame+1)/args->server);
        clock_part=(args->final_clock-args->init_clock)/args->server;
        fprintf(iniFile,"Initial_Frame=%d\n",i*args->frame_part+args->first_frame);
        if (i == (args->server-1))
          fprintf(iniFile,"Final_Frame=%d\n",args->last_frame);
        else
          fprintf(iniFile,"Final_Frame=%d\n",(i+1)*args->frame_part+args->first_frame-1);
        fprintf(iniFile,"Initial_Clock=%f\n",(double)i*clock_part);
        fprintf(iniFile,"Final_Clock=%f\n",(double)(i+1)*clock_part);
        break;
      default:
        fprintf(stderr, "error: invalid pov_mode!\n");
        return -1;
        break;
    }
    fclose(iniFile);
    sprintf(myinifile,"%s", filename);

    sprintf(filename,"return%d.tar",i);
    myretfile[i] = strdup(filename);


    if(grpc_function_handle_init(&handles[i], "bind_servers_at_call_time", "povray") != GRPC_NO_ERROR) {
      fprintf(stderr,"Error creating function handle\n");
      return -1;
    }

printf("cmdstr = '%s', myinifile = '%s', INC = '%s', retfile = '%s'\n",
 cmdstr, myinifile, TMP_POVRAY_INC, myretfile[i]);

    status_array[i] = grpc_call_async(&handles[i], &sessionID, 
      cmdstr, strlen(cmdstr)+1, myinifile, 
      TMP_POVRAY_INC, myretfile[i]);

    if(status_array[i] != GRPC_NO_ERROR) {
      grpc_perror("grpc_call");
      return -1;
    }

    printf("Request #%d being processed\n", sessionID);
  }

  if(grpc_wait_all() != GRPC_NO_ERROR) {
    printf("Error: one or more calls failed.\n");
    return -1;
  }

  return 0;
}

int
make_farming_call(arg_info_t *args)
{
  double clock_part;
  FILE *iniFile;
  char filename[128]="";
  char cmd[1024];
  int i;

  char server_itr[10];
  int *status_array;
  char **cmd_ptr_array;
  int *len_array;
  char **ini_ptr_array;
  char **inc_ptr_array;
  char **out_ptr_array;

  printf("using task farming...\n"); fflush(stdout);

  /* allocate memory */
  cmd_ptr_array = (char **)calloc(args->server,sizeof(char*));
  len_array = (int *)calloc(args->server, sizeof(int));
  ini_ptr_array = (char **)calloc(args->server,sizeof(char*));
  inc_ptr_array = (char **)calloc(args->server,sizeof(char*));
  out_ptr_array = (char **)calloc(args->server,sizeof(char*));

  /* build arrays to invoke grpc_farm() */
  for(i=0; i<args->server; i++){
    sprintf(filename,"%s%d", TMP_POVRAY_INI, i);
    sprintf(cmd,"cp %s %s", TMP_POVRAY_INI, filename);
    system(cmd);
    iniFile = fopen(filename,"a");
    switch(args->pov_mode){
      case POV_MODE_SINGLE_IMG: /* rendering a single image */
        if(i != (args->server-1))
          fprintf(iniFile,"+SR%d +ER%d\n",i*args->height_part+1,
                          (i+1)*args->height_part);
        else
          fprintf(iniFile,"+SR%d +ER%d\n",i*args->height_part+1,args->height);

        break;
      case POV_MODE_ANIMATION: /* rendering an animation */
        args->frame_part=(int)((args->last_frame-args->first_frame+1)/args->server);
        clock_part=(args->final_clock-args->init_clock)/args->server;
        fprintf(iniFile,"Initial_Frame=%d\n",i*args->frame_part+args->first_frame);
        if (i == (args->server-1))
          fprintf(iniFile,"Final_Frame=%d\n",args->last_frame);
        else
          fprintf(iniFile,"Final_Frame=%d\n",(i+1)*args->frame_part+args->first_frame-1);
        fprintf(iniFile,"Initial_Clock=%f\n",(double)i*clock_part);
        fprintf(iniFile,"Final_Clock=%f\n",(double)(i+1)*clock_part);
        break;
      default:
        fprintf(stderr, "error: invalid pov_mode!\n");
        return -1;
        break;
    }
    fclose(iniFile);
    sprintf(filename,"%s%d", TMP_POVRAY_INI, i);
    ini_ptr_array[i]=strdup(filename);
    inc_ptr_array[i]=TMP_POVRAY_INC;
    sprintf(filename,"return%d.tar",i);
    out_ptr_array[i]=strdup(filename);

    cmd_ptr_array[i]=strdup(args->povray_options);
    len_array[i] = strlen(cmd_ptr_array[i]) + 1;
  }

  /* invoking GridSolve using task-farming */
  sprintf(server_itr,"i=0,%d",args->server-1);
  status_array = grpc_farm(server_itr, "povray",
      grpc_ptr_array((void**)cmd_ptr_array,"$i"),
      grpc_int_array((void *)len_array, "$i"),
      grpc_ptr_array((void**)ini_ptr_array,"$i"),
      grpc_ptr_array((void**)inc_ptr_array,"$i"),
      grpc_ptr_array((void**)out_ptr_array,"$i"));

  /* error handling */
  if (status_array[0] == -1) {
    for (i=1;i<=args->server;i++)
      fprintf(stderr,"request #%d: %s\n",i,
                     grpc_error_string(status_array[i]));

    return -1;
  }

  /* freeing memory */
  free(cmd_ptr_array);
  free(ini_ptr_array);
  free(inc_ptr_array);
  free(out_ptr_array);

  return 0;
}

char *getFilename(char *allname) {
  int i;
  char *filename;

  filename=malloc(sizeof(char)*128);
  for(i=strlen(allname); i>0; i--){
    if(allname[i] =='/') {
      sprintf(filename,"%s",&allname[i+1]);
      i=-1;
    }
  }
  if(i == 0)
    sprintf(filename, "%s", allname);
  return filename;
}

void getLocalPath(char *allname, arg_info_t *a) {
  int i;
  strcpy(a->localPath,"\0");
  for(i=strlen(allname); i>0; i--)
    if(allname[i] =='/') {
      strncpy(a->localPath,allname,i+1);
      i = 0;
    }
}

void addFile(char *filename, arg_info_t *a) {
  char cmd[1024]="";
printf("adding file '%s%s'\n", a->localPath, filename);

  sprintf(cmd, "tar --no-recursion -rf %s %s%s &> /dev/null", TMP_POVRAY_INC,a->localPath,filename);
  if (a->debug == DEBUG_ON)
    fprintf(stderr, "%s\n", cmd);
  system(cmd);
}

void addDir(char *pathname, arg_info_t *a) {
  char cmd[1024]="";
  sprintf(cmd, "tar --no-recursion -rf %s %s/* &> /dev/null", TMP_POVRAY_INC,pathname);
  if (a->debug != 0)
    fprintf(stderr, "%s\n", cmd);
  system(cmd);
}

void 
clean() {
  char cmd[1024]="";
  sprintf(cmd, "rm -f return*.tar part*.tga %s outfile* %s",
    TMP_POVRAY_ALL,CUSTOM_PARM);
  system(cmd);
}

void 
version() 
{
  fprintf(stdout,"gspovray 0.4pre (4 Jan 2006)\n");
}

void 
usage(char **argv) 
{
  fprintf(stdout,"gspovray provides remote and distributed execution of POV-Ray through GridSolve.\n");
  version();
  fprintf(stdout,"\nUsage: %s [+/-]POV-Ray_Options [gspovray_Options]\n",argv[0]);
}

void 
help(char **argv) {
  fprintf(stdout,"Examples:\n");
  fprintf(stdout,"  %s +iicl.pov\n",argv[0]);
  fprintf(stdout,"  %s +iicl1.pov --dist 2\n",argv[0]);
  fprintf(stdout,"  %s icl_animation.ini +W200 +H200 --dist 4\n",argv[0]);
  fprintf(stdout,"\n");
  fprintf(stdout,"POV-Ray Options:\n");
  fprintf(stdout,"  All the options supported by POV-Ray can be used by gspovray and are just\n");
  fprintf(stdout,"  passed to the server. Some of the most often used options are:\n");
  fprintf(stdout,"  I<name> = input file name\n");
  fprintf(stdout,"  L<name> = library path prefix\n");
  fprintf(stdout,"  Hnnn    = image height\n");
  fprintf(stdout,"  Wnnn    = image width\n");
  fprintf(stdout,"  --- for a complete list look at http://www.povray.org/\n\n");
  fprintf(stdout,"gspovray Options:\n");
  fprintf(stdout,"  --clean     remove temporary files generated by nspovray, then exit\n");
  fprintf(stdout,"  --debug     print debug messages (for troubleshooting only)\n");
  fprintf(stdout,"  --dist n    start n instances to distribute the workload\n");
  fprintf(stdout,"  --frame n   set the number of frames per job\n");
  fprintf(stdout,"  --help      print this help, then exit\n");
  fprintf(stdout,"  --tile nxn  set size of one tile rendered by a server\n");
  fprintf(stdout,"  --timeout n time in seconds after a job is resubmited\n");
  fprintf(stdout,"  --version   print version information, then exit\n");
  fprintf(stdout,"  use +Oname.mpg to compose the frames of an animation with mpeg_encode\n");
  fprintf(stdout,"\nReport bugs to Keith Seymour <seymour@cs.utk.edu>\n");
}

FILE *
open_rc_file()
{
  char *buf;
  FILE *in;

  in = NULL;

  if((in = fopen(RC_FILENAME, "r")) == NULL) {
    char *home;

    home = get_home_dir();

    if(!home) {
      fprintf(stderr, "Failed to determine home area.\n");
      return NULL;
    }

    buf = (char *)malloc(strlen(home) + strlen(RC_FILENAME) + 5);

    if(!buf) {
      free(home);
      fprintf(stderr, "malloc failed.\n");
      return NULL;
    }

    sprintf(buf, "%s/%s", home, RC_FILENAME);

    free(home);

    if((in = fopen(buf, "r")) == NULL) {
      free(buf);
      fprintf(stderr, "Failed to open both rc files\n");
      return NULL;
    }

    free(buf);
  }

  return in;
}

int
parse_rc_file()
{
  FILE *in;
#define BUFSZ 512
  char buf[BUFSZ];

  MONTAGE_BINARY = MONTAGE_BINARY_DEFAULT;
  CONVERT_BINARY = CONVERT_BINARY_DEFAULT;
  MPEGENCODE_BINARY = MPEGENCODE_BINARY_DEFAULT;
  MMV_BINARY = MMV_BINARY_DEFAULT;

  if((in = open_rc_file()) == NULL)
    return -1;

  while(fgets(buf, BUFSZ, in)) {
    char *keytok, *valtok;

    buf[strlen(buf)-1] = '\0';

    valtok = strchr(buf, '=');
    valtok++;
    while(isspace(*valtok))
      valtok++;

    keytok = strtok(buf, " \t=");

    if(!strcasecmp(keytok, "MONTAGE_BINARY")) {
      MONTAGE_BINARY = strdup(valtok);
    }
    else if(!strcasecmp(keytok, "CONVERT_BINARY")) {
      CONVERT_BINARY = strdup(valtok);
    }
    else if(!strcasecmp(keytok, "MPEGENCODE_BINARY")) {
      MPEGENCODE_BINARY = strdup(valtok);
    }
    else if(!strcasecmp(keytok, "MMV_BINARY")) {
      MMV_BINARY = strdup(valtok);
    }
    else
      fprintf(stderr, "Warning: unknown option '%s'\n", keytok);
  }

  fclose(in);
  return 0;
}

char *
get_home_dir()
{
  struct passwd *user_info;
  char *login;

  if((login = getlogin()) == NULL)
    return strdup(getenv("HOME"));

  user_info = getpwnam(login);

  if(!user_info)
    return strdup(getenv("HOME"));

  return strdup(user_info->pw_dir);
}
