import java.lang.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.nio.channels.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

public class GSpovray extends JFrame implements java.awt.event.ActionListener {
  private JTextField infoLabel;
  private JTextField questionField;
  private StatusBar statusField;
  private JTextField povFileField;
  private JButton loadButton, saveButton, renderButton, quitButton;
  private JSlider gridSizeSlider, xResolutionSlider, yResolutionSlider;
  private GSpovrayService gspov;
  private JPanel sliderPanel, buttonPanel, filenamePanel, mainPanel;
  private Container content;
  private GSpovrayImageFrame img = null;

  private int global_rnum = 0;

  static final long serialVersionUID = 42L;

  public GSpovray() {
    super("GSpovray");

    gspov = new GSpovrayService();

    createUI();

    pack();
    setVisible(true);
  }

  protected void createUI() {
    content = getContentPane();
    content.setLayout(new BorderLayout(15, 15));

    mainPanel = new JPanel();
    mainPanel.setLayout(new BorderLayout(15, 15));
    Border empty = BorderFactory.createEmptyBorder(10,5,5,5);
    mainPanel.setBorder(empty);

    buttonPanel = new JPanel();
    buttonPanel.setLayout(new FlowLayout());

    sliderPanel = new JPanel();
    sliderPanel.setLayout(new BoxLayout(sliderPanel, BoxLayout.Y_AXIS));

    filenamePanel = new JPanel();
    filenamePanel.setLayout(new BoxLayout(filenamePanel, BoxLayout.X_AXIS));
    Border title = BorderFactory.createTitledBorder("POV File:");
    filenamePanel.setBorder(title);

    infoLabel = new JTextField(25);
    infoLabel.setEditable(false);
    infoLabel.setFont(infoLabel.getFont().deriveFont(Font.PLAIN, 16));

    loadButton = new JButton();
    loadButton.setText("Browse");
    loadButton.addActionListener(this);

    saveButton = new JButton();
    saveButton.setText("Save current image");
    saveButton.addActionListener(this);

    quitButton = new JButton();
    quitButton.setText("Quit");
    quitButton.addActionListener(this);

    renderButton = new JButton();
    renderButton.setText("Start rendering");
    renderButton.addActionListener(this);

    povFileField = new JTextField("Enter filename here or use browse button");

    gridSizeSlider = new JSlider(1, 8);
    gridSizeSlider.setBorder(BorderFactory.createTitledBorder("Grid Dimension"));
    gridSizeSlider.setMajorTickSpacing(1);
    gridSizeSlider.setMinorTickSpacing(1);
    gridSizeSlider.setPaintTicks(true);
    gridSizeSlider.setPaintLabels(true);
    gridSizeSlider.setSnapToTicks(true);
    gridSizeSlider.setValue(2);
    gridSizeSlider.addChangeListener(new SliderListener());

    xResolutionSlider = new JSlider(128, 1280);
    xResolutionSlider.setBorder(BorderFactory.createTitledBorder("X Resolution"));
    xResolutionSlider.setMajorTickSpacing(128);
    xResolutionSlider.setMinorTickSpacing(8);
    xResolutionSlider.setPaintTicks(true);
    xResolutionSlider.setPaintLabels(true);
    xResolutionSlider.setSnapToTicks(true);
    xResolutionSlider.setValue(640);
    xResolutionSlider.addChangeListener(new SliderListener());

    yResolutionSlider = new JSlider(128, 1280);
    yResolutionSlider.setBorder(BorderFactory.createTitledBorder("Y Resolution"));
    yResolutionSlider.setMajorTickSpacing(128);
    yResolutionSlider.setMinorTickSpacing(8);
    yResolutionSlider.setPaintTicks(true);
    yResolutionSlider.setPaintLabels(true);
    yResolutionSlider.setSnapToTicks(true);
    yResolutionSlider.setValue(480);
    yResolutionSlider.addChangeListener(new SliderListener());

    infoLabel.setText("Grid size: " + 
        gridSizeSlider.getValue() + " x " +
        gridSizeSlider.getValue() + "; Resolution: " + 
        xResolutionSlider.getValue() + " x " +
        yResolutionSlider.getValue() + "       ");

    filenamePanel.add(povFileField);
    filenamePanel.add(loadButton);

    sliderPanel.add(gridSizeSlider);
    sliderPanel.add(xResolutionSlider);
    sliderPanel.add(yResolutionSlider);

    buttonPanel.add(infoLabel);
    buttonPanel.add(renderButton);
    buttonPanel.add(saveButton);
    buttonPanel.add(quitButton);

    mainPanel.add(filenamePanel, BorderLayout.NORTH);
    mainPanel.add(sliderPanel, BorderLayout.CENTER);
    mainPanel.add(buttonPanel, BorderLayout.SOUTH);

    content.add(mainPanel, BorderLayout.CENTER);

    // Exit the application when the window is closed.
    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        System.exit(0);
      }
    });
  }

  public static void main(String[] args) {
    new GSpovray();
  }
    
  class EnterKeyListener implements java.awt.event.KeyListener
  {
    public void keyTyped(KeyEvent e) { }
    public void keyPressed(KeyEvent e) { }
    public void keyReleased(KeyEvent e) { }
  }

  public String getFilename(javax.swing.filechooser.FileFilter f)
  {
    JFileChooser chooser = null;
    int returnVal;

    chooser = new JFileChooser();

    chooser.setCurrentDirectory(new File(System.getProperty("user.dir")));

    if(f != null) {
      chooser.setDialogType(JFileChooser.OPEN_DIALOG);
      chooser.setFileFilter(f);
      returnVal = chooser.showOpenDialog(this);
    }
    else {
      chooser.setDialogType(JFileChooser.SAVE_DIALOG);
      returnVal = chooser.showSaveDialog(this);
    }

    if(returnVal == JFileChooser.APPROVE_OPTION) {
      try {
        return chooser.getSelectedFile().getCanonicalPath();
      } catch(IOException e) {
        return null;
      }
    }

    return null;
  }

  public static String getFileSeparator() {
    return System.getProperties().getProperty("file.separator");
  }

  public void doRendering()
  {
    int i, j, gSz, numReq, chunkXSz, chunkYSz, resX, resY, reqid;
    int sc, ec, sr, er;
    String [] cmd, retFilename;
    String prefix;
    ProgressDialog progress;
    long stime, otime, prep_time, submit_time, wait_time, total_time;

    stime = System.currentTimeMillis();
    otime = stime;

    File f = new File(povFileField.getText());

    if(!f.exists()) {
      JOptionPane.showMessageDialog(this, "File does not exist.");
      return;
    }

    progress = new ProgressDialog("Submitting Image Sections to GridSolve");

    global_rnum++;

    gSz = gridSizeSlider.getValue();
    numReq = gSz * gSz;

    chunkXSz = xResolutionSlider.getValue() / gSz;
    chunkYSz = yResolutionSlider.getValue() / gSz;

    resX = chunkXSz * gSz;
    resY = chunkYSz * gSz;

    cmd = new String[numReq];
    retFilename = new String[numReq];

    System.out.println("going to start rendering, " + chunkXSz + " x " 
       + chunkYSz + " (" + resX + " x " + resY + ")");

    prefix = "-DG +FT +H" + resY + " +W" + resX;

    for(i=0;i<gSz;i++) {
      for(j=0;j<gSz;j++) {
        sc = 1 + i*chunkXSz;
        ec = (i*chunkXSz) + chunkXSz;
        sr = 1 + j*chunkYSz;
        er = (j*chunkYSz) + chunkYSz;

        cmd[i*gSz + j] = prefix + " +SC" + sc + " +EC" + ec + " +SR" + sr + " +ER" + er;
        retFilename[i*gSz + j] = "return_" + i + "_" + j + "_" + global_rnum + ".png";

        File tmpfile = new File(retFilename[i*gSz + j]);
        if(tmpfile.exists())
          tmpfile.delete();
      }
    }

    String fname = f.getName();

    String newname;

    String inifile = "gs_povray.ini";

    if(fname.endsWith(".pov"))
      newname = fname.replaceAll("pov$", "png");
    else
      newname = fname + ".png";

    FileOutputStream out;
    PrintStream p;

    try
    {
      out = new FileOutputStream(inifile);
      p = new PrintStream(out);
      p.println("Input_File_Name=incfile");
      p.println("Height=" + resY);
      p.println("Width=" + resX);
      p.close();
    }
    catch (Exception e)
    {
      JOptionPane.showMessageDialog(this, "Error creating ini file.");
      return;
    }

    prep_time = System.currentTimeMillis() - stime;
    stime = System.currentTimeMillis();

    Hashtable requests = new Hashtable(numReq*3);

    for(i=0;i<gSz;i++) {
      for(j=0;j<gSz;j++) {
        progress.setValue( (int) (((i*gSz + j) / (double) (numReq-1)) * 100));

        if(progress.isCancelled()) {
          gspov.cancel_all();
          progress.setVisible(false);
          return;
        }

        System.out.println("cmd = " + cmd[i*gSz + j] + ", outfile = " + retFilename[i*gSz + j]);

        reqid = gspov.povray(cmd[i*gSz + j], inifile, povFileField.getText(), retFilename[i*gSz + j]);

        if(reqid < 0) {
          JOptionPane.showMessageDialog(this, "GridSolve request failed.  Check your GRIDSOLVE_AGENT setting.");
          progress.setVisible(false);
          return;
        }

        requests.put(String.valueOf(reqid), retFilename[i*gSz + j]);
      }
    }

    progress.setVisible(false);

    if(progress.isCancelled()) {
      gspov.cancel_all();
      progress.setVisible(false);
      return;
    }

    submit_time = System.currentTimeMillis() - stime;
    stime = System.currentTimeMillis();

    System.out.println("submitted all requests.. waiting now");

    img = new GSpovrayImageFrame(resX, resY, gSz, gSz);

    for(i=0;i<numReq;i++) {
      String retfile;
      String [] tmpsplit;
      int ix, iy;

      reqid = gspov.wait_for_any();

      if(reqid < 0) {
        System.err.println("failed to wait for request");
        continue;
      }

      retfile = (String) requests.get(String.valueOf(reqid));
 
      tmpsplit = retfile.split("_");
      ix = Integer.parseInt(tmpsplit[1]);
      iy = Integer.parseInt(tmpsplit[2]);

      System.out.println("return file is " + retfile + ", ix = " + ix + ", iy = " + iy);

      img.addImage(retfile, ix, iy);

      File tmpfile = new File(retfile);
      if(tmpfile.exists())
        tmpfile.delete();
    }

    wait_time = System.currentTimeMillis() - stime;
    total_time = System.currentTimeMillis() - otime;

    System.out.println("Time spent:");
    System.out.println("  preparing inputs:     " + (double)prep_time/1000.0 + " seconds");
    System.out.println("  submitting tasks:     " + (double)submit_time/1000.0 + " seconds");
    System.out.println("  waiting for results:  " + (double)wait_time/1000.0 + " seconds");
    System.out.println("Total time spent in doRendering():  " + (double)total_time/1000.0 + " seconds");
  }

  class SliderListener implements ChangeListener {
    public void stateChanged(ChangeEvent e) {
      JSlider source = (JSlider)e.getSource();
      if(!source.getValueIsAdjusting()) {
        int val = (int)source.getValue();
        System.out.println("val = " + val);
      }

      infoLabel.setText("Grid size: " + 
          gridSizeSlider.getValue() + " x " +
          gridSizeSlider.getValue() + "; Resolution: " + 
          xResolutionSlider.getValue() + " x " +
          yResolutionSlider.getValue() + "       ");
    }
  }

  public void actionPerformed(java.awt.event.ActionEvent event)
  {
    JButton source = (JButton) event.getSource();

    if(source == loadButton) {

      String newFile = getFilename(new PovFileFilter());

      if(newFile == null)
        return;

      povFileField.setText(newFile);
    }
    else if(source == quitButton) {
      System.exit(0);
    }
    else if(source == renderButton) {
      doRendering();
    }
    else if(source == saveButton) {
      String newFile;

      if(img == null) {
        JOptionPane.showMessageDialog(this, "No rendered image.");
        return;
      }

      newFile = getFilename(null);

      if(newFile == null)
        return;

      try {
        img.saveImage(newFile);
      } catch (Exception e) {
        JOptionPane.showMessageDialog(this, "Could not save File: " + e.getMessage());
      }
    }
  }
}

class StatusBar extends JLabel {
  /** Creates a new instance of StatusBar */
  public StatusBar() {
    super();
    super.setPreferredSize(new Dimension(100, 16));
    setMessage("Ready");
  }
  
  public void setMessage(String message) {
    setText(" "+message);        
  }        
}

