#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "GSpovrayService.h"
#include "grpc.h"

char *
get_c_string(JNIEnv *env, jstring s)
{
  char *str;
  int len;

  len = (*env)->GetStringLength(env, s);
  str = (char *)malloc(len+1);
  if(!str) {
    fprintf(stderr, "malloc failed\n");
    return NULL;
  }
  (*env)->GetStringUTFRegion(env, s, 0, len, str);
  str[len] = 0;

  return str;
}

/*
 * Class:     GSpovrayService
 * Method:    povray
 * Signature: (Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)I
 */
JNIEXPORT jint JNICALL Java_GSpovrayService_povray
  (JNIEnv *env, jobject obj, jstring cmd, jstring ini, jstring inc, jstring out)
{
  jint retval;
  char *cmdstr, *inistr, *incstr, *outstr;
  grpc_error_t rv;
  grpc_function_handle_t *handle;
  grpc_sessionid_t sessionID;

  rv = grpc_initialize(NULL);

  if((rv != GRPC_NO_ERROR) && (rv != GRPC_ALREADY_INITIALIZED)) {
    grpc_perror("grpc_initialize");
    retval = -1;
    return retval;
  }

  cmdstr = get_c_string(env, cmd);
  inistr = get_c_string(env, ini);
  incstr = get_c_string(env, inc);
  outstr = get_c_string(env, out);

  if(!cmdstr || !inistr || !incstr || !outstr) {
    fprintf(stderr, "could not initialize arguments\n");
    retval = -1;
    return retval;
  }

  handle = (grpc_function_handle_t *)malloc(sizeof(grpc_function_handle_t));

  if(!handle) {
    fprintf(stderr, "malloc failed\n");
    retval = -1;
    return retval;
  }

  if(grpc_function_handle_init(handle, "bind_servers_at_call_time", "povray_single_image") != GRPC_NO_ERROR) {
    fprintf(stderr,"Error creating function handle\n");
    retval = -1;
    return retval;
  }

  rv = grpc_call_async(handle, &sessionID, cmdstr, strlen(cmdstr)+1, inistr,
      incstr, outstr);

  if(rv != GRPC_NO_ERROR) {
    grpc_perror("grpc_call");
    retval = -1;
    return retval;
  }

  retval = sessionID;

  return retval;
}

/*
 * Class:     GSpovrayService
 * Method:    wait_for_any
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_GSpovrayService_wait_1for_1any
  (JNIEnv *env, jobject obj)
{
  jint retval;
  grpc_error_t status;
  int id;

  status = grpc_wait_any(&id);

  if(status != GRPC_NO_ERROR) {
    printf("Warning: call %d failed.\n", id);
    retval = -1;
    return retval;
  }

  retval = id;

  return retval;
}

/*
 * Class:     GSpovrayService
 * Method:    cancel_all
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_GSpovrayService_cancel_1all
  (JNIEnv *end, jobject obj)
{
  jint retval;
  grpc_error_t status;
  int id;

  status = grpc_cancel_all();

  if(status != GRPC_NO_ERROR) {
    printf("Warning: cancelling calls failed.\n");
    retval = -1;
    return retval;
  }

  retval = 0;

  return retval;
}
