/** 
 * @file 
 *
 * Header file for the agent.
 *
 */
/* $Id: agent.h,v 1.49 2007/04/11 16:47:33 seymour Exp $ */
/* $UTK_Copyright: $ */

#ifndef _AGENT_H
#define _AGENT_H

#include "portability.h"
#include "proxylib.h"
#include "problem.h"
#include "server.h"
#include "general.h"

#include "sqlite3.h"

#define GRIDSOLVE_DEFAULT_MYSQL_HOST "localhost"
#define GRIDSOLVE_DEFAULT_MYSQL_USER "gridsolve"
#define GRIDSOLVE_DEFAULT_MYSQL_PASSWD "GridSolve"
#define GRIDSOLVE_DEFAULT_MYSQL_DB "gridsolve"
#define GRIDSOLVE_DEFAULT_MYSQL_PORT 0
#define GRIDSOLVE_DEFAULT_MYSQL_UNIX_SOCKET "~/gs_mysql/mysql.sock"

#define DUMMY_TASKID "dummy id"

#define GS_SENSOR_MAX_LINE 1024
#define GS_AGENT_MAX_FD 256

#define GS_SCHED_LOCK_TIMEOUT 3

#ifndef timersub
# define timersub(a, b, result)                                               \
  do {                                                                        \
    (result)->tv_sec = (a)->tv_sec - (b)->tv_sec;                             \
    (result)->tv_usec = (a)->tv_usec - (b)->tv_usec;                          \
    if ((result)->tv_usec < 0) {                                              \
      --(result)->tv_sec;                                                     \
      (result)->tv_usec += 1000000;                                           \
    }                                                                         \
  } while (0)
#endif

#ifndef SECSPERMIN
#define SECSPERMIN      60
#endif

#ifndef MINSPERHOUR
#define MINSPERHOUR     60
#endif

#ifndef HOURSPERDAY
#define HOURSPERDAY     24
#endif

#ifndef SECSPERHOUR
#define SECSPERHOUR     (SECSPERMIN * MINSPERHOUR)
#endif

#ifndef SECSPERDAY
#define SECSPERDAY      ((long) SECSPERHOUR * HOURSPERDAY)
#endif

extern char GRIDSOLVE_SQLITE_DB[], GRIDSOLVE_SENSOR_USOCK[];

/** Frequency with which we expire servers from the database. */
#define GS_EXPIRE_FREQUENCY 20

/** Expire servers that haven't contacted us within this many seconds: */
#define GS_EXPIRE_TIMEOUT 130

#define GS_MAX_NAMELEN 128

/* Size of backlog in listen(socket, backlog) for the agent database
   manager process.  The agent forks processes, which then connect to
   the manager to query the database. */
#define GRIDSOLVE_AGENT_MAX_CONNECTIONS 64

/** The socket fd for comm w/ the sensor process */
extern int GS_SENSORFD;
extern FILE *GS_SENSORFILE;

typedef enum gs_scheduler {
  GS_DEFAULT_MCT,
  GS_HTM_ML,
  GS_HTM_MSF,
  GS_HTM_HMCT,
  GS_HTM_MP,
  GS_INVALID_SCHEDULER
} gs_agent_scheduler_t;

/** structure to hold information about the MySQL database **/
typedef struct gs_mysql_info_s {
  char *host;
  char *user;
  char *passwd;
  char *db_name;
  int port;
  char *unix_socket;
} gs_mysql_info_t;

/** Agent data structure */
typedef struct gs_agent_s {
  char *hostname;
  in_port_t port;
  int sock;

  void *authority;              /* who is allowed to command the agent? */

  void *servers;                /* list of servers, currently a hash table */
  void *problems;               /* list of known problems */
  void *mappings;               /* mapping of problems to to servers */

  void *agents;
  void *requests;

  int kflops;
  int ncpu;
  int availcpu;
  int status;

  int dsig;

  char *infodir;

  gs_mysql_info_t mysql;
} gs_agent_t;

/** Stripped-down agent structure for peer agents. */
typedef struct gs_peer_agent_s {
  char *hostname;
  in_port_t port;
} gs_peer_agent_t;

/* System statistics */
typedef struct {
  char hostname[GS_MAX_NAMELEN];
  in_port_t port;
  struct timeval start_time;
} gs_agent_stats_t;

/* To keep track of open descriptors */
typedef struct {
  int maxfd;
  int fd[GS_AGENT_MAX_FD+1];
} gs_agent_conn_t;

/* Function prototypes: */

gs_agent_t 
  *gs_agent_init();

int 
  gs_init_db(sqlite3 **),
  gs_agent_setup_fd_sets(gs_agent_conn_t *, int, int, fd_set *, int *),
  gs_agent_init_conns(gs_agent_conn_t *),
  gs_agent_add_conn(gs_agent_conn_t *, int),
  gs_agent_del_conn(gs_agent_conn_t *, int),
  gs_agent_finalize(),
  gs_lock_stats_file(char *, short, int),
  gs_unlock_stats_file(int),
  gs_init_stats_file(gs_agent_t *, char *),
  gs_agent_process_message(),
  gs_agent_scheduler(gs_problem_t *, int,  gs_server_t **);

double
  get_time_since_startup(),
  gs_agent_get_server_score(gs_problem_t *, gs_server_t *);

/* In gs_storage_sqlite_manager.c */
void 
  gs_agent_httpd(void **),
  gs_agent_manage_database(void **),
  gs_agent_manage_database_pre_fork(void **),
  gs_agent_manage_database_post_fork(void **),
  gs_agent_manage_database_exit(void **),
  gs_agent_sensor_run(void **args),
  gs_agent_sensor_pre(void **args),
  gs_agent_sensor_post(void **args),
  gs_agent_sensor_exit(void **args),
  gs_server_expiration(void **);

#endif                          /* _AGENT_H */
