/**
 * @file
 *
 * Header file for the GridSolve communication routines. 
 *
 */
/* $Id: comm_data.h,v 1.40 2008/10/17 15:03:49 tbrady Exp $ */
/* $UTK_Copyright: $ */

#ifndef GS_COMM_H
#define GS_COMM_H

#include <stdarg.h>
#include "problem.h"
#include "utility.h"
#include "comm_protocol.h"
#include "dsi.h"

#ifdef GS_SMART_GRIDSOLVE
#include "gs_smart_task_graph.h"
#include "gs_smart_mapping_graph.h"
#define GS_MAX_ARG_POLL 1000000
#endif

#define GS_CALL_FROM_C 1
#define GS_CALL_FROM_FORTRAN 2

#define FILE_XFER_BUFSZ 8192

#define GS_NO_CALL_SIG "-"

#define GS_MAX_PING 65536

typedef enum {
  GS_CLIENT_SIDE, GS_SERVER_SIDE
} gs_side_t;

/* Sending and receiving functions */
int 
  gs_send_tag(SOCKET, int),
  gs_recv_tag(SOCKET, int *),
  gs_send_int(SOCKET, int),
  gs_recv_int(SOCKET, int *),
  gs_send_string(SOCKET, char *),
  gs_recv_string(SOCKET, char **),
  gs_send_arg(SOCKET sock, gs_argument_t *arg, int my_dsig),
  gs_recv_arg(SOCKET sock, gs_argument_t *arg, int sender_major,
	int sender_dsig, int my_dsig, gs_side_t side),
  gs_wait_for_output(SOCKET),
  gs_send_arg_from_file(SOCKET, char *),
  gs_recv_arg_into_file(SOCKET, char *),
  gs_save_output_args_to_file(int, gs_problem_t *, int),
  gs_save_input_args_to_file(char *, gs_problem_t *, int, int, int),
  gs_restore_output_args_from_file(int, gs_problem_t *, int),
  gs_restore_input_args_from_file(char *, gs_problem_t *, int, int),
  gs_sender_compute_arg_sizes(gs_va_list *, void **, gs_problem_t *, int, int),
  gs_arglist_free_temps(gs_argument_t *),
  gs_send_input_args(gs_va_list *, void **, SOCKET, gs_problem_t *, int, int, int),
  gs_send_input_scalar_args(gs_va_list *, void **, SOCKET, gs_problem_t *, int, int, int),
  gs_send_input_nonscalar_args(SOCKET, gs_problem_t *, int),
  gs_recv_output_args(SOCKET, gs_problem_t *, int, int),
  gs_recv_input_scalar_args(SOCKET, gs_problem_t *, int, int, int *),
  gs_recv_input_args(SOCKET, gs_problem_t *, int, int),
  gs_send_output_args(SOCKET, gs_problem_t *, int),
  gs_recv_dsi_input_arg(SOCKET, gs_argument_t *, int),
  gs_send_dsi_input_arg(SOCKET, gs_argument_t *, int),
  gs_recv_dsi_output_arg(SOCKET, gs_argument_t *, int),
  gs_send_dsi_output_arg(SOCKET, gs_argument_t *, int),
  gs_transpose_matrix(gs_argument_t *, char *, gs_side_t),
  gs_convert_arg(gs_argument_t *, char *, int, int),
  gs_receiver_compute_arg_sizes(gs_problem_t *, enum inout),
  gs_construct_scalar_hashtable(icl_hash_t **, gs_problem_t *, enum inout),
  gs_setup_workspace(gs_problem_t *, icl_hash_t *),
#ifdef GS_SMART_GRIDSOLVE

  /*
   * Functions for sending and receiving structures of SmartGridSolve
   */ 

  gs_smart_send_tg(SOCKET sock, gs_smart_tg * task_graph),
  gs_smart_recv_tg(SOCKET sock, gs_smart_tg * task_graph),
  gs_smart_send_all_servers(SOCKET , gs_server_t **, int ),
  gs_smart_recv_all_servers(SOCKET , gs_server_t ***, int *),
  gs_smart_send_map_info(SOCKET , gs_problem_t * ),
  gs_smart_recv_map_info(SOCKET , gs_problem_t * ),

  /*
   * Functions for sending and receiving input output arguments
   */ 

  gs_smart_send_input_args(gs_va_list *, void **, SOCKET , gs_problem_t *, int , int , int ),
  gs_smart_send_input_args_remotely(SOCKET ,  gs_problem_t *, int ),
  gs_smart_recv_input_args(SOCKET sock, gs_server_t *, gs_problem_t *, int , int),
  gs_smart_send_input_nonscalar_args(SOCKET , gs_problem_t *, int ),
  gs_smart_send_output_args_remotely(SOCKET , gs_server_t *, gs_problem_t *, int ),
  gs_smart_send_output_args_to_client(SOCKET , gs_problem_t *, int ),
  gs_smart_recv_output_args_from_server(SOCKET , gs_problem_t *, gs_problem_t *,int , int ),
  gs_smart_recv_output_args(SOCKET , gs_problem_t *, int , int),

  /*
   * Functions for sending and receiving a single argument remotely
   */ 
  gs_smart_send_arg_remotely(SOCKET, gs_argument_t *, int, int ),
  gs_smart_send_arg_remotely_to_file(gs_server_t * , gs_argument_t *, int , int ),
  gs_smart_send_arg_remotely_to_s_mem(gs_server_t * , gs_argument_t *, int , int , int *),
  gs_smart_recv_arg_remotely(SOCKET, gs_argument_t *, int ),
  gs_smart_recv_arg_from_s_mem(gs_server_t *, gs_argument_t *, int, int, 
                                                              gs_side_t, int *),
  gs_smart_save_output_args_to_file(SOCKET ,  gs_server_t * , int , gs_problem_t *, int),
  gs_smart_restore_output_args_from_file(int, gs_problem_t *, int),

  /*
   * Functions for caching arguments
   */ 
  gs_smart_read_arg_from_file(gs_server_t * , char * , gs_argument_t * , int ,  int),
  gs_smart_write_arg_to_file(char * , gs_argument_t * ),
  gs_smart_get_obj_file_name(char ** , char *  ,int , int ),
  /*
   * Functions for reading and updating unique application
   * application number
   */ 
   
   gs_smart_read_update_app_no_file(char * , int * ),
   gs_smart_read_app_no_file(char * , int * ),

#endif
  gs_get_element_size(enum datatype, int);
  

void 
  sctrans_(void *,int *,int *,int *,int *,int *,int *),
  dctrans_(void *,int *,int *,int *,int *,int *,int *),
  dtrans_(void *,int *,int *,int *,int *,int *,int *),
  itrans_(void *,int *,int *,int *,int *,int *,int *),
  strans_(void *,int *,int *,int *,int *,int *,int *),
  ctrans_(void *,int *,int *,int *,int *,int *,int *);

#endif /* GS_COMM_H */
