/**
 * @file
 *
 * Header file containing GridSolve protocol tags. 
 *
 */
/* $Id: comm_protocol.h,v 1.28 2010/04/23 15:29:40 tbrady Exp $ */
/* $UTK_Copyright: $ */

#ifndef COMM_PROTOCOL_H
#define COMM_PROTOCOL_H

/**
 * The tag is going be transmitted in a single byte, so 
 * do not exeed that size (e.g. 0-255)  
 */

typedef enum {
  GS_PROT_OK = 1,
  GS_PROT_ERROR,
  GS_PROT_PROBLEM_SUBMIT,
  GS_PROT_SERVER_REGISTER,
  GS_PROT_PROBLEM_REGISTER,
  GS_PROT_WORKLOAD_REPORT,
  GS_PROT_KILL_AGENT,
  GS_PROT_KILL_SERVER,
  GS_PROT_SERVER_LIST,
  GS_PROT_SERVER_PING_LIST,
  GS_PROT_SERVER_PING_UPDATE,
  GS_PROT_PROBLEM_LIST,
  GS_PROT_PROBLEM_DESC,
  GS_PROT_PROBLEM_EXAMPLE,
  GS_PROT_PROBLEM_SOLVE,
  GS_PROT_PROBLEM_SOLVE_ASSIGNED,
  GS_PROT_BLOCKING,
  GS_PROT_NONBLOCKING,
  GS_PROT_KILL_JOB,
  GS_PROT_PROBE_REQUEST,
  GS_PROT_AWAIT_COMPLETION,
  GS_PROT_NOTIFY_SUBMIT,
  GS_PROT_NOTIFY_FAILURE,
  GS_PROT_NOTIFY_COMPLETE,
  GS_PROT_NOTIFY_CANCEL,
  GS_PROT_VERSION_MISMATCH,
  GS_PROT_UNKNOWN_SERVER,
  GS_PROT_MULTIPLE_SERVER,
  GS_PROT_AVAILABILITY_REQ,
  GS_PROT_PING,
  GS_PROT_STORE_FILE,
  GS_PROT_DATA_TRANSFER /* tag for processing inter-server data transfer */
} gs_message_tag_enum_t;

typedef enum {
  GS_SVC_NO_ERROR = 1,
  GS_SVC_UNUSED,
  GS_SVC_ERR_MALLOC,
  GS_SVC_ERR_MISSING_XML,
  GS_SVC_ERR_MISSING_SV_XML,
  GS_SVC_ERR_CREATE_XML,
  GS_SVC_ERR_OPEN_DATA_FILE,
  GS_SVC_ERR_CREAT_DATA_FILE,
  GS_SVC_ERR_COMPLETION_FILE,
  GS_SVC_ERR_PROBLEM_ENC,
  GS_SVC_ERR_GETCWD,
  GS_SVC_ERR_REQID,
  GS_SVC_ERR_MKDIR,
  GS_SVC_ERR_CHDIR,
  GS_SVC_ERR_SIGNALS,
  GS_SVC_ERR_FORK,
  GS_SVC_ERR_WAITPID,
  GS_SVC_ERR_ABNORMAL_EXIT,
  GS_SVC_ERR_IO,
  GS_SVC_ERR_EXEC,
  GS_SVC_ERR_READ_PROBLEM,
  GS_SVC_ERR_RESTORE_ARGS,
  GS_SVC_ERR_UNKNOWN_REQ,
  GS_SVC_ERR_REQ_DIR_GONE,
  GS_SVC_ERR_NOT_FINISHED,
  GS_SVC_ERR_INVALID_PID,
  GS_SVC_ERR_CANT_KILL,
  GS_SVC_ERR_BAD_NB_TAG,
  GS_SVC_ERR_RESTRICT_JOBS,
  GS_SVC_ERR_RESTRICT_TIME,
  GS_SVC_ERR_BAD_BATCH_MODE,
  GS_SVC_ERR_FILE_LOCK,
  GS_SVC_ERR_FILE_UNLOCK,
#ifdef GS_SMART_GRIDSOLVE
  GS_PROT_APP_PM_MODEL,
  GS_PROT_SMART_COMM,
  GS_SMART_GRAPH_OK,
  GS_SMART_GRAPH_FAIL,
  GS_SMART_STORE_ARG_TO_FILE,
  GS_SMART_STORE_ARG_TO_MEMORY,
  GS_SMART_DELETE_ARG_FROM_MEMORY,
  GS_SMART_DELETE_ARG_FROM_FILE,
  GS_SMART_RECV_ARG_FROM_MEMORY,
  GS_SMART_ARG_NOT_READY,
  GS_SMART_ARG_BUFF_FULL,
  GS_SMART_ARG_STORED_IN_FILE,
  GS_SMART_ARG_STORED_IN_S_MEM,
  GS_SMART_FAULT_UPDATE_PM,
#endif
  GS_SVC_ERR_UNSPECIFIED
} gs_service_error_enum_t;

#endif /* COMM_PROTOCOL_H */
