/**
 * @file
 *
 * This file contains error tags and strings for the GridRPC client.
 *
 */
/* $Id: grpc_error.h,v 1.7 2006/04/26 13:54:25 seymour Exp $ */
/* $UTK_Copyright: $ */

#ifndef __GRPC_ERROR__
#define  __GRPC_ERROR__

#include "comm_protocol.h"

typedef int grpc_error_t;

extern grpc_error_t grpc_errno;
extern gs_service_error_enum_t grpc_minor_errno;

/** Value for void sessionid **/
#define GRPC_SESSIONID_VOID           -1

/** Generic error tags for GridRPC */

#define GRPC_NO_ERROR                  0
#define GRPC_NOT_INITIALIZED           1
#define GRPC_ALREADY_INITIALIZED       2
#define GRPC_CONFIGFILE_NOT_FOUND      3
#define GRPC_CONFIGFILE_ERROR          4
#define GRPC_SERVER_NOT_FOUND          5
#define GRPC_FUNCTION_NOT_FOUND        6
#define GRPC_INVALID_FUNCTION_HANDLE   7
#define GRPC_INVALID_SESSION_ID        8
#define GRPC_RPC_REFUSED               9
#define GRPC_COMMUNICATION_FAILED     10 
#define GRPC_SESSION_FAILED           11
#define GRPC_NOT_COMPLETED            12
#define GRPC_NONE_COMPLETED           13
#define GRPC_OTHER_ERROR_CODE         14
#define GRPC_UNKNOWN_ERROR_CODE       15
/** dummy for boundary check */
#define GRPC_LAST_ERROR_CODE          16

/** Implementation specific error codes for GridSolve. 
 *  begin numbering at 256 since the service error codes
 *  from comm_protocol.h will not exceed 255.
 */

#define GRPC_NO_MINOR_ERROR           256
#define GRPC_NULL_FUNCTION_HANDLE     257
#define GRPC_NULL_FUNCTION_NAME       258
#define GRPC_NULL_ARGSTACK            259
#define GRPC_OUT_OF_MEMORY            260
#define GRPC_RESOLV_FAILED            261
#define GRPC_AGENT_NOT_SET            262
#define GRPC_SUBMIT_REQ_ENCODE        263
#define GRPC_SOLVE_REQ_ENCODE         264
#define GRPC_PROBLEM_DECODE           265
#define GRPC_PROBLEM_ENCODE           266
#define GRPC_SERVER_DECODE            267
#define GRPC_SERVER_ENCODE            268
#define GRPC_PARSE_HOST_INFO          269
#define GRPC_SERVER_CONNECTION        270
#define GRPC_KILL_FAILED              271
#define GRPC_INVALID_SERIALIZED_REQ   272
#define GRPC_DUP_PROBLEM_FAILED       273
#define GRPC_PROFILING_NOT_ENABLED    274
#define GRPC_INVALID_MAJOR            275
#define GRPC_INVALID_LANG             276
#define GRPC_FARM_SESSIONID_NOT_FOUND 277
#define GRPC_FARM_NULL_ITERATOR       278
#define GRPC_FARM_SETUP_ARGS_FAILED   279
#define GRPC_FARM_CREATE_STACK_FAILED 280
#define GRPC_VERSION_MISMATCH         281
#define GRPC_DSI_DISABLED             282
#define GRPC_DSI_MANAGE_ERROR         283
#define GRPC_DSI_UNKNOWN_FILE         284
#define GRPC_DSI_ALLOCATE_ERROR       285
#define GRPC_DSI_EACCES               286
#define GRPC_DSI_LOAD_ERROR           287
#define GRPC_DSI_STORE_ERROR          288
#define GRPC_DSI_INTERNAL_ERROR       289

/** dummy for boundary check */
#define GRPC_LAST_MINOR_ERROR_CODE    290

/** Text descriptions of the error tags defined above. */
extern char * GRPC_ERROR_MESSAGES[];
extern char * GRPC_MINOR_ERROR_MESSAGES[];

#endif
