/** 
 * @file 
 *
 * Header file for the http server.
 *
 */
/* $Id: gs_agent_httpd.h,v 1.12 2008/11/12 21:43:58 seymour Exp $ */
/* $UTK_Copyright: $ */

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/mman.h>
#include <sys/un.h>
#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>

#include "agent.h"
#include "comm_basics.h"
#include "comm_encode.h"
#include "utility.h"
#include "gs_storage.h"
#include "mfork.h"
#include "icl_list.h"
#include "gs_storage_sql_tables.h"

char 
  *gs_xmlize(const char *);

int 
  gs_httpd_connect_db(int),
  gs_send_task_list(int, char *),
  gs_send_server_array(int, gs_server_t **, int),
  gs_free_server_array(gs_server_t **, int),
  gs_send_problem_array(int, gs_problem_t **, int),
  gs_free_problem_array(gs_problem_t **, int),
  gs_httpd_preamble(int, int, const char *, int),
  gs_httpd_process_status(int),
  gs_httpd_process_task_list(int sockfd),
  gs_httpd_process_server_info(int, const char *),
  gs_httpd_process_problem_info(int, const char *),
  gs_httpd_reply(int, const char *),
  gs_httpd_get_file(int, const char *),
  gs_httpd_head_file(int, const char *),
  gs_agent_httpd_handle_connection(int),
  gs_read_until_crlf(int),
  gs_agent_httpd_listen_and_process_connections(int);


const char 
  *gs_get_content_type(const char *);

#define MAXLINE 1024

#define GS_DEFAULT_CONTENT_TYPE "text/html"

#define GS_AGENT_WWW_DIR "www-home"

#define GS_RESPONSE_DB_FAILURE "<html><head>\
<title>Failed to connect to database</title>\
</head><body>\
<h1>Failure</h1>\
<p>Could not connect to the database.</p>\
<hr>\
<address>GridSolve 0.1 beta -- \
<a href=\"mailto:netsolve@cs.utk.edu\">support</a></address>\
</body></html>"

#define GS_RESPONSE_INTERNAL_ERROR "<html><head>\
<title>Internal Error</title>\
</head><body>\
<h1>Failure</h1>\
<p>There was an internal error processing your request.  Bummer.</p>\
<hr>\
<address>GridSolve 0.1 beta -- \
<a href=\"mailto:netsolve@cs.utk.edu\">support</a></address>\
</body></html>"

#define GS_RESPONSE_BAD_SERVER_REQ "<html><head>\
<title>No such server</title>\
</head><body>\
<h1>Invalid Server</h1>\
<p>There was no server found matching the component ID.</p>\
<hr>\
<address>GridSolve 0.1 beta -- \
<a href=\"mailto:netsolve@cs.utk.edu\">support</a></address>\
</body></html>"

#define GS_RESPONSE_BAD_PROBLEM_REQ "<html><head>\
<title>No such problem</title>\
</head><body>\
<h1>Invalid Problem</h1>\
<p>There was no problem found matching that name.</p>\
<hr>\
<address>GridSolve 0.1 beta -- \
<a href=\"mailto:netsolve@cs.utk.edu\">support</a></address>\
</body></html>"

#define GS_RESPONSE_404 "<html><head>\
<title>404 Not Found</title>\
</head><body>\
<h1>Not Found</h1>\
<p>The requested URL was not found on this server.</p>\
<hr>\
<address>GridSolve 0.1 beta -- \
<a href=\"mailto:netsolve@cs.utk.edu\">support</a></address>\
</body></html>"

#define GS_XML_HEADER_SERVERLIST "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\
<?xml-stylesheet type=\"text/xsl\" href=\"serverlist.xsl\"?>"

#define GS_XML_HEADER_PROBLEMINFO "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\
<?xml-stylesheet type=\"text/xsl\" href=\"probleminfo.xsl\"?>"

#define GS_XML_HEADER_SERVERINFO "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\
<?xml-stylesheet type=\"text/xsl\" href=\"serverinfo.xsl\"?>"

#define GS_XML_HEADER_TASKLIST "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\
<?xml-stylesheet type=\"text/xsl\" href=\"tasklist.xsl\"?>"

#define GS_BEGIN_SYSTEM_STATUS "<system_status>\n"
#define GS_END_SYSTEM_STATUS "</system_status>\n"

#define GS_BEGIN_AGENT_STATS "<agent_stats>\n"
#define GS_END_AGENT_STATS "</agent_stats>\n"

#define GS_BEGIN_AGENT_NAME "<agent_name>\n"
#define GS_END_AGENT_NAME "</agent_name>\n"

#define GS_BEGIN_AGENT_PORT "<agent_port>\n"
#define GS_END_AGENT_PORT "</agent_port>\n"

#define GS_BEGIN_SERVER_INFO "<server_information>\n"
#define GS_END_SERVER_INFO "</server_information>\n"

#define GS_BEGIN_PROBLEM_INFO "<problem_information>\n"
#define GS_END_PROBLEM_INFO "</problem_information>\n"

#define GS_BEGIN_PROBLEM_LIST "<problem_list>\n"
#define GS_END_PROBLEM_LIST "</problem_list>\n"

#define GS_BEGIN_SERVER_LIST "<server_list>\n"
#define GS_END_SERVER_LIST "</server_list>\n"

#define GS_BEGIN_TASK_LIST "<task_list>\n"
#define GS_END_TASK_LIST "</task_list>\n"

#define GS_BEGIN_TASK_INFO "<task_info>\n"
#define GS_END_TASK_INFO "</task_info>\n"
