#ifndef GS_PM_MODEL_H
#define GS_PM_MODEL_H 

#undef DGELS

#define GS_PM_MAX_RUNS 1000

typedef struct {
  int nb_params;       /*size of the parameter vector*/
  int nb_categories;   /*size of the categorie vector*/
  int max_run;         /*max number of runs to be stored, i.e.
		       number of lines of param_mats and time_vec*/
  int run_index;       /*index of the next run to be stored*/
  int full;            /*flag that tells if the matrix is full*/
  double *params;      /* vector of one parameter run */
  double *categories;  /* vector of categories parameters,
			 which that imply different models such as 
			 transpose/no transpose*/
  double **param_mat;  /*matrix of runs of size max_run*(nb_params+nb_categories)*/
  double *time_vec;     /*vector of the timings of each run of size max_run*/
  double *coef_vec;      /*Current value of the model coefficient*/
  int model_ok;        /*flag that tells if the model is complete or not*/
} gs_pm_model_t;

/* Initialize the model*/
gs_pm_model_t *gs_pm_init_model(int nb_categories,int nb_params,int nb_lines);

/* Store a new timing */
void gs_pm_store_timing(double time,gs_pm_model_t *);

/* Update the model automomatically done after each store and load */
int gs_pm_compute_model(gs_pm_model_t *);

/* Return the compute time given by the model */ 
double gs_pm_guess_time(gs_pm_model_t *);

/* Free internals */
void gs_pm_free_model(gs_pm_model_t *);

/*display a model*/
void gs_pm_display_model(gs_pm_model_t *pb);

/* in :a model 
  out : table of categories and coefficients
  categories = NULL if no categories.
  Returns: number of lines of each table, 0 if no models*/
int gs_pm_all_models(gs_pm_model_t *md,double ***categories, double ***coefficients);

/* ASCII I/O : portable */ 
/* load a model from disk */
gs_pm_model_t * gs_pm_load_model(char *filename);

/* dump model to a FILE pointer */
int gs_pm_dump_model(gs_pm_model_t *md, FILE *pf);
int gs_pm_dump_model_terse(gs_pm_model_t *md, FILE *pf);

/* save a model to disk */
int gs_pm_save_model(gs_pm_model_t *md,char *filename);

/* binary I/O : not portable but fast and compact*/
/* save a model to the disk return -1 in case of failure, 0 otherwise*/
int gs_pm_save(gs_pm_model_t *md,int fd);

/* load a model from the disk return NULL in case of failure, a pointer to the model otherwise*/
gs_pm_model_t * gs_pm_load(int fd);

int nnls_(double *a, int *mda, int *m, int *n, double *b, double *x, double *rnorm, 
	  double *w, double *zz, int *index, int *mode);
int nnls_c(double* a, const int* mda, const int* m, const int* n, double* b, 
	 double* x, double* rnorm, double* w, double* zz, int* index, 
	 int* mode);
int dgels_(char *,int *,int *,int *,double *,int *,double *,int *,double *,int *,int *);

#endif
