/******************************************************************************/
/*                            gs_seq_data_handle.h                            */
/*  the header file for GridSolve request sequencing data handle processing   */
/*                            Yinan Li, 06/17/2008                            */
/******************************************************************************/


#ifndef GS_SEQ_DATA_HANDLE_H_
#define GS_SEQ_DATA_HANDLE_H_


#include "gs_dag.h"


/*****************************************************************************/
/* ---------- GridSolve Request Sequencing Data Handle Structures -----------*/
/*****************************************************************************/
typedef struct data_handle_type data_handle_t;
typedef struct data_handle_list data_handle_list_t;


struct data_handle_type {
    char *producer_func;
    char *consumer_func;
	int pseq_id;
	int cseq_id;
    int pindex;
	int cindex;
	void *pdptr;
	void *cdptr;
    char *data_handle;
	int used;
	int visited;
    struct data_handle_type *prev;
    struct data_handle_type *next;
};


struct data_handle_list {
    data_handle_t *head;
    data_handle_t *tail;
    int num_handles;
};


/*****************************************************************************/
/* -------------- GridSolve Request Sequencing Data Handle API --------------*/
/*****************************************************************************/

/* make a new data handle list */
int make_new_data_handle_list(GS_DAG_t *dag);

/* insert a data handle into a data handle list */
int insert_data_handle(
	char *producer_func, int seq_id, int pindex, void *dptr, char *data_handle);

/* find a specific data handle in a data handle list */
char *find_data_handle(char *consumer_func, int seq_id, int cindex, void *dptr);

/* free a data handle list */
void free_data_handle_list();


#endif

