/********************************************************************/
/*                            gs_seq_dsi.h                          */
/*      the header file for GridSolve request sequencing DSI API    */
/*                       Yinan Li, 06/17/2008                       */
/********************************************************************/


#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <stdarg.h>
#include <errno.h>
#include <time.h>

#include "portability.h"
#include "utility.h"
#include "comm_basics.h"
#include "comm_encode.h"
#include "comm_data.h"
#include "grpc.h"


/* the struct used to represent a local-filesystem-based dsi object */
typedef struct {
    int data_type;     /* the data type */
    int major;         /* the major (row or column) */
    int data_size;     /* the size of the data */
	int index;         /* the index of the argument in argument list */
	char *func_name;   /* the function name */
	char *source;      /* the server that produced the data */
    char *path;        /* the relative path to the root directory
                          of GridSolve; indicate where the data is
                          stored on the source server */
    char *file_name;   /* the name of the file that store the data */
} LFS_DSI_OBJECT;


/* the default path to put data file */
extern char *DATA_SEND_FILE_PATH;
extern char *DATA_RECV_FILE_PATH;


/*****************************************************************************/
/*------- GridSolve Request Sequencing Local-Filesystem-Based DSI API -------*/
/*****************************************************************************/

/* set the default LFS_DSI data storage path */
extern int gs_seq_set_lfs_dsi_data_storage_path();

/* encode a local-filesystem-based dsi object to a string */
extern char *gs_seq_encode_lfs_dsi_object(LFS_DSI_OBJECT *obj);

/* decode a message into a local-filesystem-based dsi object */
extern LFS_DSI_OBJECT *gs_seq_decode_lfs_dsi_object(char *data_handle);

/* save data to a local file */
extern char *gs_seq_create_unique_file_name(gs_argument_t *argptr);

/* create a new data handle */
extern LFS_DSI_OBJECT *gs_seq_create_data_handle(
gs_argument_t *argp, char *path, char *fname, int my_major);

/* save an output argument into a file */
extern int gs_seq_save_data_into_file(
char *fname, gs_argument_t *argptr, int my_major, int my_dsig);

/* restore an input argument from a file */
extern int gs_seq_restore_data_from_file(
char *fname, gs_argument_t *argptr, int my_dsig);














