/********************************************************************/
/*                           gs_sequence.h                          */
/*        the header file for GridSolve request sequencing API      */
/*                       Yinan Li, 06/17/2008                       */
/********************************************************************/


#ifndef GRPC_SEQUENCE_H_
#define GRPC_SEQUENCE_H_


#include "gs_dag.h"
#include "gs_seq_dsi.h"
#include "gs_seq_data_handle.h"


/* the current sequencing mode, could be either 
   OPTIMISTIC_MODE
   or
   RESTRICTIVE_MODE
   or
   CONSERVATIVE_MODE
   or
   OFF_MODE
   OFF_MODE can not be used as the initial mode 
*/
extern int grpc_sequence_mode;

/* the sequencing language, could be either
   CALL_FROM_C
   or
   CALL_FROM_FORTRAN 
*/
extern int grpc_sequence_language;

/* the sequencing major, could be either 'r' or 'c' */
extern int grpc_sequence_major;


/********************************************************************/
/* ------- Core GridSolve Request Sequencing Data Structures ------ */
/********************************************************************/
/* enumeration of GridSolve sequencing modes */
enum GS_SEQ_MODE {
    OPTIMISTIC_MODE,    /* ignore data dependency
                           among scalar arguments */
    RESTRICTIVE_MODE,   /* scalar arguments are 
                           passed by reference */
    CONSERVATIVE_MODE,  /* two successive calls
                           with one having input
                           scalar arguments and
                           the other one having
                           output scalar argument */
    OFF_MODE            /* sequencing is going 
                           to be finalized */
};


/********************************************************************/
/* ---------- GridSolve Request Sequencing API Functions ---------- */
/********************************************************************/

/* begins a new sequence with the specified mode */
extern void grpc_sequence_begin(int mode);

/* finalizes a sequence */
extern double grpc_sequence_end(int n, ...);

/* submit a sequencing grpc call with a variable list of arguments */
extern int grpc_submit(char *name, ...);

/* submit a sequencing grpc call with an argument stack */
extern int grpc_submit_arg_stack(char *name, grpc_arg_stack *arg_stack);

/* gets the current sequencing mode */
extern int grpc_get_sequence_mode();

/* gets the current sequence */
extern GS_DAG_t *grpc_get_sequence();


/* ------------------------- Not Decided -------------------------- */
/* change the sequencing mode */
extern void grpc_change_sequence_mode(int mode);
/* restore the previous sequencing mode */
extern void grpc_end_sequence_mode_change();

int if_passed_back(va_list, int, gs_argument_t *);

#endif
