/*
 * @file
 *
 * This file contains functions for all the implemented mapping heuristics.
 * The mapping heuristics generate solutions based on the
 * task graph of the group of tasks and the network 
 * performance model of the network.  These heuristics
 * generate the best mapping solution (best_mg) in
 * gs_smart_mapping_solution.h
 */

#include "config.h"
#ifdef GS_SMART_GRIDSOLVE
#ifndef GS_SMART_MAPPING_HEURISTIC
#define GS_SMART_MAPPING_HEURISTIC
#include "grpc.h"
#include "gs_smart_task_graph.h"
#include "gs_smart_netpm.h"
#include "gs_smart_mapping_graph.h"

/*
 * Mapping heuristic functions
 */

int gs_smart_exec_mapper(gs_smart_tg * , gs_smart_netpm * , gs_smart_mg ** , int );
int gs_smart_ex_map(gs_smart_tg *, gs_smart_netpm * , gs_smart_mg **);
int gs_smart_greedy_map(gs_smart_tg *, gs_smart_netpm * , gs_smart_mg **);
int gs_smart_basic_map(gs_smart_tg *, gs_smart_netpm * , gs_smart_mg **);
int gs_smart_random_map(gs_smart_tg *, gs_smart_netpm * , gs_smart_mg **);
int gs_smart_rwalk_map(gs_smart_tg * , gs_smart_netpm * , gs_smart_mg ** );

/*
 * Functions to set mapper_type and determine whether
 * the requested mapper_type is available.
 */

int gs_smart_check_if_mapper_exists(char * );
int gs_smart_get_nb_mappers();
char * gs_smart_get_mapping_name(int );
int gs_smart_get_mapper_type(char ** _mapper_type);
int gs_smart_set_mapper_type(char * _mapper_type);


/*
 * Functions to calculate the cost of communication and computation.
 */
int gs_smart_time_tg_task_node_with_c_comm(gs_smart_tg_remote_task_node * ,
                                           gs_server_t * , double *);
int gs_smart_time_tg_task_node(gs_smart_tg_remote_task_node * ,
                                           gs_server_t * , double *);
int gs_smart_time_mg_task_node(gs_smart_mg_task_node * , double * );

#endif 
#endif
