/* 
 * @file
 *
 * This file contains the initial sql query used to setup
 * an empty agent database.
 */
/* $Id: gs_storage_sql_tables.h,v 1.4 2008/12/15 21:01:52 seymour Exp $ */
/* $UTK_Copyright: $ */

#ifndef GS_STORAGE_SQL_TABLES_H
#define GS_STORAGE_SQL_TABLES_H

#define GS_SQL_SERVER_TABLE "CREATE TABLE servers (\
  hostname varchar(128) NOT NULL default '',\
  ipaddress varchar(128) NOT NULL default '0',\
  port int(16) NOT NULL default '0',\
  proxyip varchar(128) default '0',\
  proxyport int(16) default '0',\
  componentid varchar(128) NOT NULL default '',\
  arch varchar(64) NOT NULL default '',\
  data_format int(32) NOT NULL default '0',\
  kflops int(32) NOT NULL default '0',\
  workload int(16) NOT NULL default '0',\
  ncpu int(32) NOT NULL default '0',\
  status int(32) NOT NULL default '0',\
  availcpu float(16) NOT NULL default '0.0',\
  availmem float(16) NOT NULL default '0.0',\
  nproblems int(32) NOT NULL default '0',\
  agenthost varchar(128) NOT NULL default '',\
  agentport int(32) NOT NULL default '0',\
  smart int(32) NOT NULL default '0',\
  lastupdate bigint NOT NULL,\
  addedat bigint NOT NULL,\
  infolist text, \
  server_pings text, \
  PRIMARY KEY (componentid),\
  UNIQUE (ipaddress,port)\
);"

#define GS_SQL_PROBLEM_TABLE "CREATE TABLE problems (\
  problemname varchar(128) NOT NULL default '',\
  encoding text NOT NULL,\
  PRIMARY KEY (problemname)\
);"

#define GS_SQL_PROBLEM_SERVER_MAPPING_TABLE "CREATE TABLE problem_server (\
  problemname varchar(128) NOT NULL,\
  componentid varchar(128) NOT NULL,\
  perf_model_expr text NOT NULL, \
  UNIQUE (problemname,componentid)\
);"

#define GS_SQL_CRITERIA_TABLE "CREATE TABLE criteria (\
  critname varchar(128) NOT NULL,\
  description text NOT NULL,\
  PRIMARY KEY (critname)\
);"

#define GS_SQL_TASKS_TABLE "CREATE TABLE tasks (\
  t_componentid varchar(128) NOT NULL default '',\
  t_taskid varchar(128) NOT NULL default '',\
  t_agent_taskid int(16) NOT NULL default '-1',\
  t_start float(32) NOT NULL default '0.0',\
  t_duration float(32) NOT NULL default '0.0',\
  t_remaining float(32) NOT NULL default '0.0',\
  t_end float(32) NOT NULL default '0.0',\
  t_active int(16) NOT NULL default '0',\
  t_finished int(16) NOT NULL default '0',\
  PRIMARY KEY (t_agent_taskid)\
);"

#define GS_SQL_COMPLETED_TASKS_TABLE "CREATE TABLE completed_tasks (\
  c_componentid varchar(128) NOT NULL default '',\
  c_taskid varchar(128) NOT NULL default '',\
  c_agent_taskid int(16) NOT NULL default '-1',\
  c_start float(32) NOT NULL default '0.0',\
  c_duration float(32) NOT NULL default '0.0',\
  c_remaining float(32) NOT NULL default '0.0',\
  c_end float(32) NOT NULL default '0.0',\
  c_active int(16) NOT NULL default '0',\
  c_finished int(16) NOT NULL default '0',\
  PRIMARY KEY (c_agent_taskid)\
);"

#define GS_SQL_INIT_TASKS_TABLE "insert into tasks values ('','-1','','','','','','','');"

#define GS_SQL_INIT_COMPLETED_TASKS_TABLE "insert into completed_tasks values ('','-1','','','','','','','');"

#ifdef GS_USE_MYSQL
#define GS_SQL_EXTRA_OPTIONS ""
#else
#define GS_SQL_EXTRA_OPTIONS "PRAGMA default_synchronous = OFF;"
#endif

#endif
