/**
 * @file
 *
 * Portability routines
 *
 */
/* $Id: portability.h,v 1.11 2008/09/09 21:36:59 seymour Exp $ */
/* $UTK_Copyright: $ */


#ifndef _GS_PORTABILITY_H
#define _GS_PORTABILITY_H

/* ----------------------------------------------------------- */
#if defined (__INTERIX)
/* ----------------------------------------------------------- */

#include <string.h>
#include <strings.h>
#include <unistd.h>
#include <time.h>
#include <sys/ioctl.h>
#include <sys/param.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <sys/stat.h>		/* umask */
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/tcp.h>	/* TCP_NODELAY */
#include <arpa/inet.h>
#include <netdb.h>
#include <fcntl.h>
#include <pwd.h>
#include <glob.h>

/* Define ipaddr_t if it is not defined */
#ifndef	_IPADDR_T
#define	_IPADDR_T
typedef uint32_t ipaddr_t;
#endif /* _IPADDR_T */

#define GS_WNOHANG WNOHANG
#define GS_WBLOCK 0

#define GS_SIGKILL SIGKILL
#define GS_SIGPIPE SIGPIPE

#define WAIT_ABANDONED -1
#define WAIT_TIMEOUT 0

#define SOCKET int
#define INVALID_SOCKET -1
#define SOCKET_ERROR -1
#define read_socket(s,b,l) read(s,b,l)
#define write_socket(s,b,l) write(s,b,l)
#define close_socket close
#define errno_socket() errno
void initialize_sockets();
void cleanup_sockets();

/* In portability.c */
int gs_lock_fd(int fd, int lock_type);
int gs_lock_fd_nowait(int fd, int lock_type);
int gs_unlock_fd(int fd);

#define gs_struct_stat struct stat
#define gs_stat stat

#define getdomainname getpdomain



/* ----------------------------------------------------------- */
#elif defined (WIN32) 
/* ----------------------------------------------------------- */

#include <stdlib.h>
#include <stdio.h>
#include <winsock2.h>
#include <windows.h>
#include <signal.h>
#include <io.h>
#include <fcntl.h>
#include <direct.h>
#include <float.h>
#include <time.h>
#include <sys/stat.h>

typedef unsigned short int in_port_t;
typedef int socklen_t;

#define ECONNABORTED WSAECONNABORTED
#define ECONNREFUSED WSAECONNREFUSED
#define EINPROGRESS WSAEINPROGRESS
#define ETIMEDOUT WSAETIMEDOUT
#define ECONNRESET WSAECONNRESET
#define EWOULDBLOCK WSAEWOULDBLOCK
#define EADDRINUSE WSAEADDRINUSE

#define GS_WNOHANG 0		/* TODO: ayk */
#define GS_WBLOCK INFINITE	/* TODO: ayk */

#define GS_SIGKILL SIGABRT
#define GS_SIGPIPE SIGILL	/* TODO: ayk just a junk signal, should never happen */

#define srand48 srand
#define drand48() 1.0/RAND_MAX*rand()
#define srandom(x) srand(x)
#define random() rand()

#define mktemp _mktemp

#ifndef MAXDOUBLE
#define MAXDOUBLE DBL_MAX
#endif

#ifndef MINDOUBLE
#define MINDOUBLE DBL_MIN
#endif

typedef unsigned int uint32_t;

#ifndef ipaddr_t
typedef int ipaddr_t;
#endif

#define pid_t int
#define strcasecmp _stricmp
#define ssize_t int
#define sleep(n) Sleep(n*1000)

/* #define SOCKET SOCKET */
/* #define INVALID_SOCKET INVALID_SOCKET */
/* #define SOCKET_ERROR SOCKET_ERROR */
int read_socket(SOCKET sock, const char*buffer, int len);
/* #define read_socket(s,b,l) recv(s,b,l,0) */
int write_socket(SOCKET sock, const char*buffer, int len);
/* #define write_socket(s,b,l) send(s,b,l,0) */
int close_socket(SOCKET sock);
/* #define close_socket closesocket */
#define errno_socket() WSAGetLastError()
void initialize_sockets();
void cleanup_sockets();

#define uid_t unsigned int
/* TODO: In windows map the UID to a fixed number */
#define getuid() 999		

/* Defined in portability.c */
int gettimeofday(struct timeval* tp, void* tzp); 
int waitpid(int pid, int* status, int options);
int kill(pid_t pid, int sig); 
int gs_lock_fd(int fd, int lock_type);
int gs_lock_fd_nowait(int fd, int lock_type);
int gs_unlock_fd(int fd);
int getdomainname(char *name, size_t len);

#define gs_struct_stat struct _stat
#define gs_stat _stat

/* ----------------------------------------------------------- */
#else  /* Unix, cygwin or such */
/* ----------------------------------------------------------- */

#include <string.h>
#include <strings.h>
#include <unistd.h>
#include <time.h>
#include <sys/ioctl.h>
#include <sys/param.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <sys/stat.h>		/* umask */
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/tcp.h>	/* TCP_NODELAY */
#include <arpa/inet.h>
#include <netdb.h>
#include <net/if.h>
#include <fcntl.h>
#include <pwd.h>
#include <glob.h>

#ifdef HAVE_STDINT_H
#include <stdint.h>
#endif

/* Define ipaddr_t if it is not defined */
#ifndef	_IPADDR_T
#define	_IPADDR_T
#ifdef PLAT_T3E
typedef short ipaddr_t;
#else
typedef uint32_t ipaddr_t;
#endif /* PLATFORMS */
#endif /* _IPADDR_T */


#define GS_WNOHANG WNOHANG
#define GS_WBLOCK 0

#define GS_SIGKILL SIGKILL
#define GS_SIGPIPE SIGPIPE

#define WAIT_ABANDONED -1
#define WAIT_TIMEOUT 0

#define SOCKET int
#define INVALID_SOCKET -1
#define SOCKET_ERROR -1
#define read_socket(s,b,l) read(s,b,l)
#define write_socket(s,b,l) write(s,b,l)
#define close_socket close
#define errno_socket() errno
void initialize_sockets();
void cleanup_sockets();

/* In portability.c */
int gs_lock_fd(int fd, int lock_type);
int gs_lock_fd_nowait(int fd, int lock_type);
int gs_unlock_fd(int fd);

#define gs_struct_stat struct stat
#define gs_stat stat

#endif

#endif /* _GS_PORTABILITY_H */
