/**
 * @file
 *
 * This file contains structures and function prototypes for
 * the proxy server.
 *
 */
/* $Id: proxy_server.h,v 1.12 2005/04/27 22:44:10 seymour Exp $ */
/* $UTK_Copyright: $ */


#ifndef _PROXY_SERVER_H
#define _PROXY_SERVER_H

#include "proxy_utils.h"

/** the default port number the proxy should listen on.  This can be
   overwritten by using the environment variable PROXY_LISTEN_PORT. */
#define PROXY_LISTEN_PORT_DEFAULT 8888

/** size of the table of servers this proxy is handling */
#define PROXY_ACCEPT_TABLE_SIZE 211

/** amount of data transferred by the proxy for each write */
#define PROXY_XFER_CHUNKSIZE 4096

/** how often to free unused components **/
#define PROXY_PURGE_FREQ  30

#define PROXY_LOGFILE "proxy_server.log"

#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif

/** this structure defines the component ID, which is a globally unique
 *  identifier for every component.
 */
typedef struct {
  char id[CID_LEN];
} CID;

/** this structure defines the information stored about a component. */
typedef struct {
  CID id;
  in_port_t port;
  int sockfd;
  int to_be_freed;
  QUEUE *msg_queue;
} COMPONENT;

/*** Function Prototypes ***/

int 
  proxy_handle_control_connection(int),
  proxy_handle_connection_request(int),
  proxy_init_sock(int *,struct sockaddr_in *,char *, int, int),
  proxy_auth(int),
  proxy_delete_server_tag(char *, COMPONENT *),
  proxy_daemon_init(char *, char *);

void 
  proxy_print_componentID(FILE *, char *);

#ifdef KERBEROS5
int proxy_check_krb5_envvars();
#endif

#endif
