/**
 * @file
 *
 * This file contains structures and function prototypes for
 * the proxy utils.
 *
 */
/* $Id: proxy_utils.h,v 1.23 2007/07/24 20:13:00 yarkhan Exp $ */
/* $UTK_Copyright: $ */

#ifndef PROXY_BASE_H_
#define PROXY_BASE_H_

#include <sys/types.h>

#include "portability.h"

/** Maximum backlog on listening sockets: */
#define PROXY_MAX_CONNECTIONS 64

typedef unsigned char proxy_tag_t;

#define CID_LEN 8

/** Protocol tags for the proxy */

#define PROXY_CONNECT             100
#define PROXY_CONNECT_REQUEST     101
#define PROXY_CONNECTION_REFUSED  102
#define PROXY_CONNECTION_ACCEPTED 103
#define PROXY_CONTROL_CONNECTION  104
#define PROXY_CONNECT_REPLY       105
#define PROXY_KEEPALIVE           106
#define PROXY_KRB5_AUTH_REQUIRED  200
#define PROXY_AUTH_ACCEPTED       201
#define PROXY_AUTH_FAILED         202

#define PROXY_MSG_QUEUE_FREQ   3
#define PROXY_KEEPALIVE_FREQ  30

#define SKIP_LOOPBACK     1
#define INCLUDE_LOOPBACK  0
#define SKIP_PRIVATE      1
#define INCLUDE_PRIVATE   0

/** 
 * These control timeouts.  if PROXY_TIMEOUT_NONE is used,
 * proxy_read_timeout() will act as a normal read() call.
 * Otherwise, the caller may specify the maximum number of
 * seconds to wait for a call to proxy_read_timeout() to
 * complete.  Typically we use PROXY_TIMEOUT_DEFAULT in the
 * proxy.
 */

#define PROXY_TIMEOUT_NONE      0
#define PROXY_TIMEOUT_DEFAULT   8

/*** Function prototypes ***/

int proxy_tread(SOCKET, char *, int, int);
int proxy_readable_timeout(SOCKET, int);
int proxy_read_timeout(SOCKET, char *, int, int);
int proxy_is_socket_error(SOCKET);
int proxy_send_keepalive(SOCKET);
int proxy_is_something_on_socket(SOCKET);
int proxy_get_host_id(char *, int, int, int);

ipaddr_t proxy_get_my_ipaddr();

#endif /* PROXY_BASE_H_ */
