/**
 * @file
 *
 * This file contains structures and function prototypes for
 * the proxy communications library.
 *
 */
/* $Id: proxylib.h,v 1.13 2006/04/14 05:18:01 seymour Exp $ */
/* $UTK_Copyright: $ */

#ifndef PROXY_LIB_H_
#define PROXY_LIB_H_

#include <stdlib.h>
#include <stdio.h>

#if     !defined(TRUE) || ((TRUE) != 1)
#define TRUE    (1)
#endif
#if     !defined(FALSE) || ((FALSE) != 0)
#define FALSE   (0)
#endif

#define PROXY_CONN_TIMEOUT 10

#define PROXY_BIND_MAXFD 256

#include "portability.h"
#include "proxy_utils.h"

#ifdef KERBEROS5
typedef struct {
  krb5_principal server;
  krb5_keytab keytab;
  krb5_context context;
  char *users_file;
  int require_auth;         /* true if authentication required */
} PROXY_KRB5_INFO;
#endif /* KERBEROS5 */

/** This structure defines the information stored about a component */
typedef struct {
  char ID[CID_LEN];
  in_port_t port;
  ipaddr_t IP;
  ipaddr_t proxyIP;
  in_port_t proxyPort;
} PROXY_COMPONENTADDR;

/** An array of these structures is used to define the mapping
 *  between the file descriptors used by the proxy replacement
 *  functions and the control socket to the proxy and the local port.
 */
typedef struct {
  SOCKET sock;
  in_port_t port; 
  SOCKET control_sock;
} PROXY_BINDING;

/*** Function prototypes ***/

SOCKET proxy_socket(int, int, int);
int proxy_bind(SOCKET, const struct sockaddr *, int);
int proxy_listen(SOCKET, int);
int proxy_accept(SOCKET, struct sockaddr *, socklen_t *);
int proxy_connect(SOCKET, PROXY_COMPONENTADDR *);
int proxy_connect_nonb(SOCKET, const struct sockaddr *, socklen_t, int);
int proxy_getsockname(int, struct sockaddr *, socklen_t *);
int proxy_close(SOCKET);
int proxy_do_auth(ipaddr_t, SOCKET);


int proxy_set_cid_from_str(char *);
int proxy_str_to_cid(char *, char *);
int proxy_cid_to_str(char *, char *);
int proxy_ip_to_str(ipaddr_t, char *);
int proxy_str_to_ip(ipaddr_t *, char *);
int proxy_cidcmp(char *, char *);

PROXY_COMPONENTADDR proxy_get_local_addr();

ipaddr_t proxy_get_proxy_ip();

in_port_t proxy_get_proxy_port();

void proxy_init(char *);
void proxy_get_id(char *);
void proxy_print_componentID(FILE *, char *);
void proxy_print_local_info();

#endif /* PROXY_LIB_H_ */
