/**
 * @file
 *
 * This file contains various utility macros and #defines used
 * by GridSolve.
 *
 */
/* $Id: utility.h,v 1.47 2008/12/16 21:00:16 seymour Exp $ */
/* $UTK_Copyright: $ */

#ifndef UTILITY_H
#define UTILITY_H

#include <stdio.h>
#include <stdarg.h>

#include "log.h"
#include "icl_list.h"
#include "icl_hash.h"
#include "xnprintf.h"
#include "portability.h"

#include <signal.h>
typedef void Sigfunc(int);

#ifndef SUN_LEN
  /* This system is not POSIX.1g.         */
#define SUN_LEN(ptr) ((size_t) \
       (((struct sockaddr_un *)   0)->sun_path)  \
       + strlen ((ptr)->sun_path))
#endif

#if     !defined(MIN)
#define MIN(x,y) ((x)<(y)?(x):(y))
#endif
#if     !defined(MAX)
#define MAX(x,y) ((x)>(y)?(x):(y))
#endif

#if     !defined(TRUE) || ((TRUE) != 1)
#define TRUE    (1)
#endif
#if     !defined(FALSE) || ((FALSE) != 0)
#define FALSE   (0)
#endif

#define STRDUP(s) s==NULL ? NULL : strdup(s)

enum boolean { NO=0, YES };

#define CALLOC calloc
#define MALLOC malloc
#define REALLOC realloc
/* In FREE, I should also reset x to NULL ; x=NULL */
#define FREE(x) if(x) {free(x); x=NULL;}

struct floatsig {
  int length;
  unsigned char *bytes;
  char *name;
};

typedef struct info_s {
  char *type;
  char *value;
  struct info_s *next;
} gs_info_t;

typedef struct _gs_va_list_s {
  va_list args;
} gs_va_list;

extern struct floatsig thesigs[];

#define DSIG_SHORT_SIZE(dsig)   (1<<((dsig)&0x7))
#define DSIG_INT_SIZE(dsig)     (1<<(((dsig)&0xE0)>>5))
#define DSIG_LONG_SIZE(dsig)    (1<<(((dsig)&0x1C00)>>10))
#define DSIG_FLOAT_SIZE(dsig)   thesigs[((dsig)&0x78000)>>15].length
#define DSIG_DOUBLE_SIZE(dsig)  thesigs[((dsig)&0x1E00000)>>21].length

#define DSIG_SHORT_ORDER(dsig)  (((dsig)&0x18)>>3)
#define DSIG_INT_ORDER(dsig)    (((dsig)&0x300)>>8)
#define DSIG_LONG_ORDER(dsig)   (((dsig)&0x6000)>>13)
#define DSIG_FLOAT_ORDER(dsig)  (((dsig)&0x180000)>>19)
#define DSIG_DOUBLE_ORDER(dsig) (((dsig)&0x6000000)>>25)

char 
  *dstring_sprintf(const char *fmt, ...),
  *dstring_append(char *s1, char* s2),
  *dstring_append_free(char *s1, char* s2),
  *gs_read_line(FILE *, int *),
  *gs_get_line(FILE *),
  *gs_get_password(char *),
  *gs_get_login_name(),
  *gs_get_machine_name();

typedef int(*gs_int_swap_f)(int *);
typedef int(*gs_float_swap_f)(float *);
typedef int(*gs_double_swap_f)(double *);

int 
  gs_swap_int(int*),
  gs_swap_int_nop(int*),
  gs_swap_int_undef(int*),
  gs_swap_float(float*),
  gs_swap_float_nop(float*),
  gs_swap_float_undef(float*),
  gs_swap_double(double*),
  gs_swap_double_nop(double*),
  gs_swap_double_undef(double*),
  pvmgetdsig(),
  gs_get_workload(),
  expr_d(const char *, double *, icl_hash_t *),
  expr_i(const char *, int *, icl_hash_t *),
  gs_parse_config_file(char *, gs_info_t**),
  gs_daemon_init(char *, char *),
  gs_unlock_file(int),
  gs_open_locked_file(char *, short, int),
  gs_open_locked_file_timeout(char *, short, int, int),
  gs_get_contents_of_file(char *, char **),
  gs_versions_incompatible(const char *, const char *),
  gs_setup_signal_handlers(Sigfunc *);
  
long
  gs_seconds_since_modified(char *);

extern gs_int_swap_f gs_int_swap_func[4][4];
extern gs_float_swap_f gs_float_swap_func[4][4];
extern gs_double_swap_f gs_double_swap_func[4][4];

Sigfunc
  *gs_signal(int, Sigfunc *);

/* Return time between sucessive calls to wall time */
double walltime(void);
double usertime(void);

/* Check for an integer in an environment variable, returning the
   integer value or a provided default value */
int getenv_int(char* name, int defval);

#endif /* UTILITY_H */
