/* From the SuperLU distribution */
#include "slu_ddefs.h"

void dgssv_superlu(
  /* IN */ int nnz,
  /* IN */ int *asub/* [nnz], "Column indices [nnz]" */,
  /* IN */ int *xa/* [n+1], "Row pointers [cols+1]" */,
  /* IN */ double *a/* [m][n]{nnz, Ic, Jr}, "Sparse matrix [nnz]" */,
  /* IN */ int m/* , "Num of rows" */,
  /* IN */ int n/* , "Num of cols" */,
  /* IN */ double *rhs/* [m], "Right hand side vector [rows]" */,
  /* OUT */ double *x/* [n], "Solution vector [cols]" */
  )
{
  SuperMatrix A, L, U, B;
  int info;
  int *perm_r = NULL;
  int *perm_c = NULL;
  superlu_options_t options;
  SuperLUStat_t stat;

  /* double s, u, p, e, r, l; */
  /* int nrhs; */
  /* int i; */

  /* Create matrix A in the format expected by SuperLU. */
  dCreate_CompCol_Matrix(&A, m, n, nnz, a, asub, xa, SLU_NR, SLU_D, SLU_GE);
  /* dPrint_CompCol_Matrix("A", &A); */
  
  /* Copy rhs to x and setup B (output is also in B) */
  /* printf("copy rhs to x %d\n", m); fflush(NULL); */
  memcpy(x, rhs, m*sizeof(double));
  dCreate_Dense_Matrix(&B, m, 1, x, m, SLU_DN, SLU_D, SLU_GE);
  /* dPrint_Dense_Matrix("B", &B); */

  if ( !(perm_r = intMalloc(m)) ) ABORT("Malloc fails for perm_r[].");
  if ( !(perm_c = intMalloc(n)) ) ABORT("Malloc fails for perm_c[].");
  
  /* Set the default input options. */
  set_default_options(&options);
  options.PrintStat = YES;
  options.ColPerm = NATURAL;
  
  /* Initialize the statistics variables. */
  StatInit(&stat);
  
  /* printf("calling dgssv\n"); fflush(NULL); */
  dgssv(&options, &A, perm_c, perm_r, &L, &U, &B, &stat, &info);
  /* printf("done dgssv\n"); fflush(NULL); */
  
  /* printf("m = %d; n = %d\n", m, n); fflush(NULL); */
  /*   for (i=0; i<m; i++)  */
  /*     printf("x[%d] = %f\n", i, x[i]); fflush(NULL); */
  
  /* dPrint_CompCol_Matrix("A", &A); fflush(NULL); */
  /* dPrint_CompCol_Matrix("U", &U); */
  /* dPrint_SuperNode_Matrix("L", &L); */
  /* print_int_vec("\nperm_r", m, perm_r); */
  /* if ( options.PrintStat ) StatPrint(&stat); */
  
  /* De-allocate storage */
  SUPERLU_FREE (perm_r);
  SUPERLU_FREE (perm_c);
  Destroy_CompCol_Matrix(&A);
  Destroy_SuperMatrix_Store(&B);
  Destroy_SuperNode_Matrix(&L);
  Destroy_CompCol_Matrix(&U);
  StatFree(&stat);
  
  if (info == 0) {
    printf("superlu success\n");
  } else {
    printf("superlu failure\n");
    return;
  }
}



void sparse_direct_solve_double(
  /* IN */ int nnz,
  /* IN */ int *asub/* [nnz], "Column indices [nnz]" */,
  /* IN */ int *xa/* [n+1], "Row pointers [cols+1]" */,
  /* IN */ double *a/* [m][n]{nnz, Ic, Jr}, "Sparse matrix [nnz]" */,
  /* IN */ int m/* , "Num of rows" */,
  /* IN */ int n/* , "Num of cols" */,
  /* IN */ double *rhs/* [m], "Right hand side vector [rows]" */,
  /* OUT */ double *x/* [n], "Solution vector [cols]" */
  )
{
  dgssv_superlu(nnz, asub, xa, a, m, n, rhs, x);
}
