/** 
 * @file
 *
 * This is the definition for a lexical analyzer for the GridSolve
 * client criteria;
 *
 */
/* $Id: criteria_lexer.l,v 1.2 2008/10/27 20:32:26 seymour Exp $ */
/* $UTK_Copyright: $ */

%{
#include "criteria_parsing_defs.h"
#include <stdio.h>
#include <string.h>
#include "criteria_parser.h"
%}

D			[0-9]
L			[a-zA-Z_]
H			[a-fA-F0-9]
E			[Ee][+-]?{D}+
FS			(f|F|l|L)
IS			(u|U|l|L)*

white      [ \t\v\n\f]

%option nounput

%%
{white}+                /* ignore whitespace */;

"&&"                    { return(AND_OP); }
"||"                    { return(OR_OP); }
"<="                    { return(LE_OP); }
">="                    { return(GE_OP); }
"=="                    { return(EQ_OP); }
"!="                    { return(NE_OP); }
";"                     { return(';'); }
","                     { return(','); }
"<"                     { return('<'); }
">"                     { return('>'); }
"("                     { return('('); }
")"                     { return(')'); }
"+"                     { return('+'); }
"-"                     { return('-'); }

{L}({L}|{D})*           { crit_lval.strp = strdup(crit_text); return(IDENT); }

0[xX]{H}+{IS}?          { crit_lval.strp = strdup(crit_text); return(CONSTANT); }
0{D}+{IS}?              { crit_lval.strp = strdup(crit_text); return(CONSTANT); }
{D}+{IS}?               { crit_lval.strp = strdup(crit_text); return(CONSTANT); }
L?'(\\.|[^\\'])+'       { crit_lval.strp = strdup(crit_text); return(CONSTANT); }

{D}+{E}{FS}?            { crit_lval.strp = strdup(crit_text); return(CONSTANT); }
{D}*"."{D}+({E})?{FS}?  { crit_lval.strp = strdup(crit_text); return(CONSTANT); }
{D}+"."{D}*({E})?{FS}?  { crit_lval.strp = strdup(crit_text); return(CONSTANT); }

L?\'(\\.|[^\\"])*\'     { crit_lval.strp = strdup(crit_text); return(STRING_LITERAL); }

.                       /* ignore */

%%

int yywrap(void) {
  return 1;
}
