/* 
 * @file
 * 
 * This file contains the basic sql queries needed to setup
 * the agent's sql database.  This should contain the same
 * query as ns_storage_tables.h, but formated for sqlite and 
 * not for C.
 */
/* $Id: tables.sql,v 1.17 2006/06/10 17:13:45 seymour Exp $ */
/* $UTK_Copyright: $ */

CREATE TABLE servers (
  hostname varchar(128) NOT NULL default '',
  ipaddress varchar(128) NOT NULL default '0',
  port int(16) NOT NULL default '0',
  proxyip varchar(128) default '0',
  proxyport int(16) default '0',
  componentid varchar(128) NOT NULL default '',
  arch varchar(64) NOT NULL default '',
  workload int(16) NOT NULL default '0',
  lastupdate timestamp(14) NOT NULL,
  addedat timestamp(14) NOT NULL,
  infolist text,
  PRIMARY KEY ('componentid'),
  UNIQUE ('ipaddress','port')
);

/* When a server is deleted, delete its problems too. */
CREATE TRIGGER delete_server_problems DELETE ON servers
  BEGIN
    DELETE FROM problem_server WHERE componentid = old.componentid;
  END;

CREATE TABLE problems (
  problemname varchar(128) NOT NULL default '',
  arglist text NOT NULL,
  description text NOT NULL,
  PRIMARY KEY ('problemname'),
  UNIQUE ('problemname','arglist')
);

CREATE TABLE problem_server (
  problemname varchar(128) NOT NULL,
  componentid varchar(128) NOT NULL,
  perf_model_expr text NOT NULL,
  UNIQUE ('problemname','componentid')
);

/* When the last instance of a problem is deleted, delete the problem */
CREATE TRIGGER delete_empty_problem AFTER DELETE ON problem_server
  BEGIN
    DELETE FROM problems WHERE problemname = old.problemname AND (SELECT COUNT(x) < 1 FROM ( SELECT 1 AS x FROM problem_server WHERE problemname = old.problemname ));
  END;

CREATE TABLE criteria (
  critname varchar(128) NOT NULL,
  description text NOT NULL,
  PRIMARY KEY ('critname')
);

CREATE TABLE tasks (
  componentid varchar(128) NOT NULL default '',
  taskid varchar(128) NOT NULL default '',
  agent_taskid int(16) NOT NULL default '-1',
  start float(8) NOT NULL default '0.0',
  duration float(8) NOT NULL default '0.0',
  remaining float(8) NOT NULL default '0.0',
  end float(8) NOT NULL default '0.0',
  active int(16) NOT NULL default '0',
  finished int(16) NOT NULL default '0',
  PRIMARY KEY ('taskid')
);

CREATE TABLE completed_tasks (
  componentid varchar(128) NOT NULL default '',
  taskid varchar(128) NOT NULL default '',
  agent_taskid int(16) NOT NULL default '-1',
  start float(8) NOT NULL default '0.0',
  duration float(8) NOT NULL default '0.0',
  remaining float(8) NOT NULL default '0.0',
  end float(8) NOT NULL default '0.0',
  active int(16) NOT NULL default '0',
  finished int(16) NOT NULL default '0',
  PRIMARY KEY ('taskid')
);

CREATE VIEW taskviewjoin AS
  SELECT
    tasks.componentid as componentid,
    tasks.taskid as taskid,
    tasks.agent_taskid as agent_taskid,
    COALESCE(MIN(completed_tasks.start, tasks.start), tasks.start)
       as calculated_start,
    COALESCE(ABS( MAX( completed_tasks.end, tasks.end ) -
       MIN( completed_tasks.start, tasks.start ) ), tasks.duration)
       AS calculated_duration,
    COALESCE(completed_tasks.remaining, tasks.remaining)
       as calculated_remaining,
    COALESCE(MAX(completed_tasks.end, tasks.end), tasks.end)
       as calculated_end,
    tasks.active as active,
    COALESCE(completed_tasks.finished, tasks.finished)
       as calculated_finish
  FROM tasks
  LEFT JOIN completed_tasks ON tasks.taskid=completed_tasks.taskid;
