
#ifdef HAVE_CONFIG_H
#include "config.h"
#include "gridsolve-config.h"
#endif /* HAVE_CONFIG_H */

#include "nsdsi.h"
#include "grpc.h"

int 
ns_dsi_close(DSI_FILE *dsi_file)
{
  if(grpc_dsi_close(dsi_file) != GRPC_NO_ERROR) {
    if(grpc_minor_errno == GRPC_DSI_MANAGE_ERROR)
      ns_errno = NetSolveIBPManageError;
    else if(grpc_minor_errno == GRPC_DSI_DISABLED)
      ns_errno = NetSolveDsiDisabled;

    return -1;
  }

  return 1;
}

DSI_FILE *
ns_dsi_open(char *host_name, int flag, int permissions, int size, 
  dsi_type storage_system)
{
  DSI_FILE *dfile;

  if(grpc_dsi_open(&dfile, host_name, flag, permissions, size, storage_system)
     != GRPC_NO_ERROR) 
  {
    if(grpc_minor_errno == GRPC_DSI_UNKNOWN_FILE)
      ns_errno = NetSolveUnknownDsiFile;
    else if(grpc_minor_errno == GRPC_DSI_DISABLED)
      ns_errno = NetSolveDsiDisabled;
    else if(grpc_minor_errno == GRPC_DSI_ALLOCATE_ERROR)
      ns_errno = NetSolveIBPAllocateError;

    return NULL;
  }

  return dfile;
}

int 
ns_dsi_read_vector(DSI_OBJECT *dsi_obj, void *data, 
  int count, enum datatype data_type)
{
  int bytes_read = 0;

  if(grpc_dsi_read_vector(dsi_obj, data, count, data_type, &bytes_read) 
      != GRPC_NO_ERROR) 
  {
    if(grpc_minor_errno == GRPC_DSI_LOAD_ERROR)
      ns_errno = NetSolveIBPLoadError;
    else if(grpc_minor_errno == GRPC_DSI_DISABLED)
      ns_errno = NetSolveDsiDisabled;
    else if(grpc_minor_errno == GRPC_DSI_EACCES)
      ns_errno = NetSolveDsiEACCESS;

    return -1;
  }

  return bytes_read;
}

int 
ns_dsi_read_matrix(DSI_OBJECT *dsi_obj, void *data, 
   int rows, int cols, enum datatype data_type)
{
  return ns_dsi_read_vector(dsi_obj, data, rows*cols, data_type);
}

DSI_OBJECT *
ns_dsi_write_vector(DSI_FILE *dsi_file, void *data, int count, 
  enum datatype data_type)
{
  DSI_OBJECT *rv = NULL;

  if(grpc_dsi_write_vector(&rv, dsi_file, data, count, data_type)
      != GRPC_NO_ERROR)
  {
    if((grpc_minor_errno == GRPC_DSI_STORE_ERROR) ||
       (grpc_minor_errno == GRPC_DSI_INTERNAL_ERROR))
      ns_errno = NetSolveIBPStoreError;
    else if(grpc_minor_errno == GRPC_DSI_DISABLED)
      ns_errno = NetSolveDsiDisabled;
    else if(grpc_minor_errno == GRPC_DSI_EACCES)
      ns_errno = NetSolveDsiEACCESS;

    return NULL;
  }

  return rv;
}

DSI_OBJECT *
ns_dsi_write_matrix(DSI_FILE *dsi_file, void *data, int rows, 
  int cols, enum datatype data_type)
{
  return ns_dsi_write_vector(dsi_file, data, rows*cols, data_type);
}
