/**
 * @file
 *
 * Routines to keep track of which input args are DSI objects.
 *
 */
/* $Id: dsi_storage.c,v 1.4 2007/07/24 19:24:32 yarkhan Exp $ */
/* $UTK_Copyright: $ */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include "dsi.h"
#include "problem.h"

/* global for keeping track of DSI objects */

icl_list_t *dsi_objects = NULL;

/**
 * Comparison function for DSI objects.
 *
 * @param key -- arg pointer to be compared
 * @param d -- arg pointer to be compared
 *
 * @returns 0 if equal, non-zero otherwise.
 */

int 
gs_comp_dsi_object(void *key, void *d)
{
  return (key != d);
}

/**
 * Frees a DSI object.
 *
 * @param d -- pointer to object to be freed.
 */

void 
gs_free_dsi_object(void *d)
{
  free(d);
}

/**
 * Inserts the object into the list of DSI objects.
 *
 * @param dsi_obj -- object to be inserted.
 *
 * @returns 0 on success, -1 on failure.
 */

int 
gs_insert_dsi_object(DSI_OBJECT *dsi_obj)
{
  if(!dsi_objects) {
    dsi_objects = icl_list_new();
    if(!dsi_objects)
      return -1;
  }

  if(icl_list_append(dsi_objects, dsi_obj) == NULL)
    return -1;

  return 0;
}

/**
 * Removes the specified object from the DSI object list.
 *
 * @param dsi_obj -- the object to be removed
 *
 * @returns 0 on success, -1 on failure.
 */

int 
gs_remove_dsi_object(DSI_OBJECT *dsi_obj)
{
  icl_list_t *tmp;

  if(!dsi_objects)
    return 0;

  tmp = icl_list_search(dsi_objects, dsi_obj, gs_comp_dsi_object);

  if(!tmp)
    return -1;

  if(icl_list_delete(dsi_objects, tmp, gs_free_dsi_object) < 0)
    return -1;

  return 0;
}

/**
 * Determines whether the specified pointer is a DSI object or just
 * a regular argument.
 *
 * @param obj -- pointer to the argument to check
 *
 * @returns 1 if it is a DSI object, 0 otherwise.
 */

int 
gs_is_dsi_object(void *obj)
{
  icl_list_t *tmp;

  if(!dsi_objects)
    return 0;

  tmp = icl_list_search(dsi_objects, obj, gs_comp_dsi_object);

  return (tmp != NULL);
}
