/**
 * Entry module of GridSolve interface for IDL
 *
 * This is part of the IDL Client for GridSolve.
 **/
/* $Id: gridsolve_module.c,v 1.4 2007/03/09 19:35:19 seymour Exp $ */
/* $UTK_Copyright: $ */

#include <stdio.h>
#include "idl_export.h"
#include "idl_request.h"

/* IDL Function prototypes */
IDL_VPTR 
  gs_call(int argc, IDL_VPTR* argv),
  gs_call_async(int argc, IDL_VPTR* argv),
  gs_probe(int argc, IDL_VPTR* argv),
  gs_cancel(int argc, IDL_VPTR* argv),
  gs_wait(int argc, IDL_VPTR* argv),
  gs_error_string(int argc, IDL_VPTR* argv),
  gs_get_last_error(int argc, IDL_VPTR* argv);

/* IDL Procedure prototypes */
void
  gridsolve(int argc, IDL_VPTR* argv),
  gs_info(int argc, IDL_VPTR* argv),
  gs_requests(int argc, IDL_VPTR* argv);

/**
 * Loads all the GridSolve functions into the IDL environment.
 *
 * @returns TRUE on success, FALSE on failure.
 */

int
IDL_Load(void)
{
  /* Definitions of GridSolve functions */
  static IDL_SYSFUN_DEF2 function_addr[] = {
    {{gs_call},            "GS_CALL",             1, IDL_MAXPARAMS, 0, 0},
    {{gs_call_async},      "GS_CALL_ASYNC",       1, IDL_MAXPARAMS, 0, 0},
    {{gs_probe},           "GS_PROBE",            1,             1, 0, 0},
    {{gs_cancel},          "GS_CANCEL",           1,             1, 0, 0},
    {{gs_wait},            "GS_WAIT",             1,             1, 0, 0},
    {{gs_error_string},    "GS_ERROR_STRING",     1,             1, 0, 0},
    {{gs_get_last_error},  "GS_GET_LAST_ERROR",   0,             0, 0, 0}
  };

  /* Definitions of GridSolve procedures */
  static IDL_SYSFUN_DEF2 procedure_addr[] = {
    {{(IDL_SYSRTN_GENERIC)gridsolve},   "GRIDSOLVE",   0, 0, 0, 0},
    {{(IDL_SYSRTN_GENERIC)gs_info},     "GS_INFO",     1, 1, 0, 0},
    {{(IDL_SYSRTN_GENERIC)gs_requests}, "GS_REQUESTS", 0, 1, 0, 0}
  };

  /* Register all GridSolve functions and procedures */
  if(!IDL_SysRtnAdd(function_addr, TRUE, IDL_CARRAY_ELTS(function_addr)) ||
     !IDL_SysRtnAdd(procedure_addr, FALSE, IDL_CARRAY_ELTS(procedure_addr)))
  {
    IDL_Message(IDL_M_GENERIC, IDL_MSG_RET, "Error adding GridSolve functions!");
    return FALSE;
  }

  /* Initialize the request structures. */
  gs_init_req();

  return TRUE;
}
