/**
 * The implementation of the gs_cancel() call.  This cancels a
 * previously submitted non-blocking request.
 *
 * This is part of the IDL Client for GridSolve.
 **/
/* $Id: gs_cancel.c,v 1.4 2007/03/01 22:52:31 seymour Exp $ */
/* $UTK_Copyright: $ */


#include <stdio.h>
#include "idl_export.h"
#include "idl_request.h"
#include "grpc.h"

/**
 * GridRPC cancel call
 *
 * @param argc -- the number of arguments
 * @param argv_idl -- the array of IDL arguments passed in
 *   this should contain one argument: the request ID of
 *   the job to cancel.
 *
 * @returns an IDL variable containing the GridRPC exit status
 *    of the call.  See the GridRPC documentation for details.
 */

IDL_VPTR
gs_cancel(int argc, IDL_VPTR* argv_idl) {
  grpc_error_t ret;
  int req_id;

  IDL_ENSURE_SCALAR(argv_idl[0]);

  req_id = IDL_LongScalar(argv_idl[0]);

  /* Initialize GridRPC system */
  ret = grpc_initialize(NULL);
  if((ret != GRPC_NO_ERROR) && (ret != GRPC_ALREADY_INITIALIZED)) {
    IDL_Message(IDL_M_GENERIC, IDL_MSG_INFO, "Error: cannot initialize!");
    return IDL_GettmpLong(ret);
  }

  IDL_TimerBlock(IDL_TRUE);
  ret = (int) grpc_cancel((grpc_sessionid_t) req_id);
  IDL_TimerBlock(IDL_FALSE);

  if(ret == GRPC_NO_ERROR)
    del_req(req_id);    

  return IDL_GettmpLong(ret);
}
