/**
 * @file
 * 
 * Header file for the matlab client.
 *
 */
/* $Id: matlab_gs.h,v 1.15 2007/08/08 18:41:07 yarkhan Exp $ */
/* $UTK_Copyright: $ */

#ifndef _MATLAB_GS_H_
#define _MATLAB_GS_H_

#include <matrix.h>
#include <mex.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include "portability.h"
#include "grpc.h"

/* Define various types that may be missing and are used in the
 * matlab_client in order to take matlab native data widths and
 * transfer them back/forth to GridSolve */
#ifdef WIN32
#ifndef __int8_t_defined
#define __int8_t_defined
typedef char int8_t;
#endif
#ifndef __int16_t_defined
#define __int16_t_defined
typedef short int16_t;
#endif
#ifndef __int32_t_defined
#define __int32_t_defined
typedef int int32_t;
#endif
#ifndef __uint8_t_defined
#define __uint8_t_defined
typedef unsigned char uint8_t;
#endif
#ifndef __uint16_t_defined
#define __uint16_t_defined
typedef unsigned short uint16_t;
#endif
#ifndef __uint32_t_defined
#define __uint32_t_defined
typedef unsigned int uint32_t;
#endif
#endif /* WIN32 */

/* #undef DBGPRINTF */
/* #define DBGPRINTF(...) */
/* #define DBGPRINTF mexPrintf("%s:%d: ", __FILE__ , __LINE__ ); mexPrintf */

enum matlab_gs_case { GS_CALL=1, GS_CALL_ASYNC=2, GS_WAIT=3, GS_ERROR_STRING=4, GS_GET_LAST_ERROR=5, GS_PROBE=6, GS_CANCEL=7, GS_INFO=8, GS_PUTENV=9};

int matlab_gs_error_num;
int matlab_gs_minor_errno;

#define DO_ERROR_MACRO(errnum) do { matlab_gs_error_num=errnum; matlab_gs_minor_errno=grpc_minor_errno; goto error; } while (0);

void *matlab_gs_calloc(size_t N, size_t S);
void *matlab_gs_free(void *ptr);

int matlab_gs_setup_args(gs_problem_t* problem, int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]);
int matlab_gs_get_output(gs_problem_t* problem, int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]);
int matlab_gs_free_input_args(gs_problem_t* problem, int nlhs, mxArray *plhs[],int nrhs, const mxArray *prhs[]);
int matlab_gs_free_args(gs_problem_t* problem, int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]);

void matlab_gs_call( int nlhs, mxArray *plhs[],int nrhs, const mxArray *prhs[] );
void matlab_gs_call_async( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] );
void matlab_gs_error_string(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]);
void matlab_gs_get_last_error(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]);
void matlab_gs_wait( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] );
void matlab_gs_probe( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] );
void matlab_gs_cancel( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] );
void matlab_gs_info(int nlhs, mxArray *plhs[],int nrhs, const mxArray *prhs[] );
void matlab_gs_putenv(int nlhs, mxArray *plhs[],int nrhs, const mxArray *prhs[] );
void matlab_gs_probe_or( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] );
int matlab_gs_arg_is_sparse_index_or_pointer(gs_argument_t *argp, gs_problem_t *problem);

#endif /* _MATLAB_GS_H_ */

