/**
 * @file
 * 
 * Matlab client call
 *
 */
/* $Id: matlab_gs_get_last_error.c,v 1.5 2006/05/09 20:19:44 yarkhan Exp $ */
/* $UTK_Copyright: $ */

#include <stdio.h>
#include <string.h>

#include <matrix.h>
#include <mex.h>

#include "grpc.h"
#include "matlab_gs.h"


/**
 * Return the error_code for the last error in the right hand side by
 * calling grpc_get_last_error.
 * 
 * @param nlhs - Number of left hand side arguments from Matlab
 * @param plhs - Pointer to left hand side arguments from Matlab
 * @param rlhs - Number of right hand side arguments from Matlab
 * @param plhs - Pointer to right hand side arguments from Matlab
 */
void 
matlab_gs_get_last_error(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
  int i;

  if ((nlhs != 1)||(nrhs != 1)) {
    mexErrMsgTxt("Usage: num = gridsolve_errno\n");
    goto error;
  }

  plhs[0] = mxCreateDoubleMatrix(1,1,mxREAL);

  /* Use Matlab global error number */
  mxGetPr(plhs[0])[0] = (double)matlab_gs_error_num;

  return; 

 error:
  for (i=0;i<nlhs;i++) plhs[i] = mxCreateDoubleMatrix(0,0,mxREAL); 
  mexPrintf("Warning: GridSolve call failed: %d:%d: %s; %s\n", grpc_errno, grpc_minor_errno, grpc_error_string(grpc_errno), grpc_minor_error_string(grpc_minor_errno));  
  return;

}
