/**
 * @file
 * 
 * Matlab client call
 *
 */
/* $Id: matlab_gs_probe.c,v 1.5 2006/05/09 20:19:44 yarkhan Exp $ */
/* $UTK_Copyright: $ */

#include <stdio.h>
#include <string.h>

#include <matrix.h>
#include <mex.h>

#include "grpc.h"
#include "matlab_gs.h"


/**
 * Return the status of a previous asynchronous request in the prhs
 * array using grpc_probe.
 * 
 * @param nlhs - Number of left hand side arguments from Matlab
 * @param plhs - Pointer to left hand side arguments from Matlab
 * @param rlhs - Number of right hand side arguments from Matlab
 * @param plhs - Pointer to right hand side arguments from Matlab
 */
void 
matlab_gs_probe(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
  int i;
  grpc_sessionid_t sessionId;
  grpc_error_t retval;

  if (nrhs != 2 || nlhs != 1) {
    mexErrMsgTxt("Usage: status = gs_probe(sessionId)\n");
    for (i=0;i<nlhs;i++) plhs[i] = mxCreateDoubleMatrix(0,0,mxREAL); 
    return;
  }

  /* reset global error setting */
  matlab_gs_error_num = 0;	
  matlab_gs_minor_errno = 256;   

  /* Init GRPC */
  retval = grpc_initialize(NULL);
  if ((retval != GRPC_NO_ERROR) && (retval != GRPC_ALREADY_INITIALIZED)) 
    DO_ERROR_MACRO(retval);


  /* Get sessionid from rhs */
  sessionId = (int)mxGetPr(prhs[1])[0];
  if (sessionId < 0) DO_ERROR_MACRO(GRPC_OTHER_ERROR_CODE);

  retval = grpc_probe_ft(sessionId);
  if ((retval != GRPC_NO_ERROR) & (retval != GRPC_NOT_COMPLETED)) 
    DO_ERROR_MACRO(retval);

  plhs[0] = mxCreateDoubleMatrix(1,1,mxREAL);
  mxGetPr(plhs[0])[0] = (double)retval;

  return; 


 error:
  plhs[0] = mxCreateDoubleMatrix(1,1,mxREAL);
  mxGetPr(plhs[0])[0] = (double)-1;
  mexPrintf("Warning: GridSolve call failed: %d:%d: %s; %s\n", grpc_errno, grpc_minor_errno, grpc_error_string(grpc_errno), grpc_minor_error_string(grpc_minor_errno));  
  return;

}
