/******************************************************************************/
/*                                                                            */
/*    gs_get_problems.cpp                                                     */
/*    source file that implements the function of                             */
/*    retrieving the list of supported services                               */
/*                                                                            */
/*    Copyright (C)                                                           */
/*    Innovative Computing Laboratory (ICL)                                   */
/*    Univesity of Tennessee, Knoxville                                       */
/*                                                                            */
/*    Feburary 22, 2007                                                       */
/******************************************************************************/


#include <oct.h>
#include <stdio.h>
#include <stdlib.h>
#include "grpc.h"
#include "gs_oct.h"


DEFUN_DLD (gs_problems, args, nargout, "Octave client for GridSolve") {
    octave_value_list bad_retval;   //bad return value
	grpc_error_t status;            //GridRPC status/error code
    char *info[1];


    //meaningless return value for error condition
    //or those calls that do not make grpc function call
    for (int i = 0; i < nargout; i++) {
        bad_retval(i) = Matrix(0, 0);
    }

    //initialize the GridSolve environment
    status = grpc_initialize(NULL);
    if (status != GRPC_NO_ERROR && status != GRPC_ALREADY_INITIALIZED) {
        fprintf(stderr, "%s\n", grpc_error_string(status));
        return bad_retval;
    }

    //number of input arguments
    int nargin = args.length();

    //display the list of available GridSolve problems
    if ((status = grpc_get_problems(info)) != GRPC_NO_ERROR) {
        fprintf(stderr, "%s\n", grpc_error_string(status));
    } else { //print out the list
        printf("GridSolve - List of available problems:\n");
        printf("%s\n", info[0]);
    }

    //finalize and clean up
    grpc_finalize();
    return bad_retval;
}
