#include <oct.h>

double *oct_GetPr(octave_value arg){
    double *pr;
    int i, j;

    if (arg.is_real_type()) {
        pr = (double *) malloc(arg.rows() * arg.columns() * sizeof(double));
        for ( j = 0; j < arg.columns(); j++)
            for (i = 0; i < arg.rows(); i++)
                *(pr + i + (j * arg.rows())) = (arg.matrix_value())(i, j);
    } else {
        pr = (double *) malloc(arg.rows() * arg.columns() * sizeof(double));
        for (j = 0; j<arg.columns(); j++)
            for (i = 0; i < arg.rows(); i++)
                *(pr + i + (j * arg.rows())) =
                real((arg.complex_matrix_value())(i, j));
    }

    return pr;
}

double *oct_GetPi(octave_value arg) {
    double *pi;
    int i, j;
 
    if (arg.is_real_type()) {
        pi = (double *) malloc(arg.rows() * arg.columns() * sizeof(double));
        for (j = 0; j < arg.columns(); j++)
            for (i = 0; i < arg.rows(); i++)
                *(pi + i + (j * arg.rows())) = 0;
    } else {
        pi = (double *) malloc(arg.rows() * arg.columns() * sizeof(double));
        for (j = 0; j < arg.columns(); j++)
            for (i = 0; i < arg.rows(); i++)
                *(pi + i + (j * arg.rows())) =
                imag((arg.complex_matrix_value())(i, j));
    }
 
    return pi;
}

double *oct_GetSparsePr(octave_value arg) {
    double *pr;
    int i, j;
    int count;
    int index;

    if (arg.is_real_type()) {
        count = 0;
        for (j = 0; j < arg.columns(); j++)
            for (i = 0; i < arg.rows(); i++)
                if ((arg.matrix_value())(i, j) != 0) count++;
    
        pr = (double *) malloc(count * sizeof(double));
        index = 0;
        for (j = 0; j < arg.columns(); j++)
            for(i = 0; i < arg.rows(); i++)
                if( (arg.matrix_value())(i,j) != 0 ) {
                    *(pr + index) = (arg.matrix_value())(i, j);
                    index++;
                }
    } else {
        count = 0;
        for (j = 0; j < arg.columns(); j++)
            for (i = 0; i < arg.rows(); i++)
                if ((real((arg.complex_matrix_value())(i, j)) != 0) ||
                    (imag((arg.complex_matrix_value())(i, j)) != 0) )
                    count++;
    
        pr = (double *) malloc(count * sizeof(double));
        index = 0;
        for (j = 0; j < arg.columns(); j++)
            for (i = 0; i < arg.rows(); i++)
                if ((real((arg.complex_matrix_value())(i, j)) != 0) ||
                    (imag((arg.complex_matrix_value())(i, j)) != 0)) {
                    *(pr + index) = real((arg.complex_matrix_value())(i, j));
                    index++;
                }

    }

    return pr;
}

double *oct_GetSparsePi(octave_value arg) {
    double *pi;
    int    i, j;
    int    count;
    int    index;
  
    if (arg.is_real_type()) {
        count = 0;
        for (j = 0; j < arg.columns(); j++)
            for (i = 0; i < arg.rows(); i++)
                if ((arg.matrix_value())(i, j) != 0) count++;
               
        pi = (double *) malloc(count * sizeof(double));
        index = 0;
        for (j = 0; j < arg.columns(); j++)
            for (i = 0; i<arg.rows(); i++)
                if ((arg.matrix_value())(i, j) != 0) {
                    *(pi + index) = 0;
                    index++;
                }
    } else {
        count = 0;
        for (j = 0; j < arg.columns(); j++)
            for (i = 0; i < arg.rows(); i++)
                if ((real((arg.complex_matrix_value())(i, j)) != 0) ||
                    (imag((arg.complex_matrix_value())(i, j)) != 0))
                    count++;
  
        pi = (double *) malloc(count*sizeof(double));
        index = 0;
        for (j = 0; j < arg.columns(); j++)
            for (i = 0; i < arg.rows(); i++)
                if ((real((arg.complex_matrix_value())(i, j)) != 0) ||
                    (imag((arg.complex_matrix_value())(i, j)) != 0)) {
                    *(pi + index) = imag((arg.complex_matrix_value())(i, j));
                    index++;
                }
      
    }

    return pi;
}

int *oct_GetSparseIr(octave_value arg) {
    int    *Ir;
    int    i, j;
    int    count;
    int    index;

    if (arg.is_real_type()) {
        count = 0;
        for (j = 0; j < arg.columns(); j++)
            for (i = 0; i < arg.rows(); i++)
                if ((arg.matrix_value())(i, j) != 0) count++;
    
        Ir = (int *) malloc(count * sizeof(int));
        index = 0;
        for (j = 0; j < arg.columns(); j++)
            for (i = 0; i < arg.rows(); i++)
                if ((arg.matrix_value())(i, j) != 0) {
                    *(Ir + index) = i;
                    index++;
                }
    } else {
        count = 0;
        for (j = 0; j < arg.columns(); j++)
            for (i = 0; i < arg.rows(); i++)
                if ((real((arg.complex_matrix_value())(i, j)) != 0) ||
                    (imag((arg.complex_matrix_value())(i, j)) != 0) )
                    count++;
        
        Ir = (int *) malloc(count * sizeof(int));
        index = 0;
        for (j = 0; j < arg.columns(); j++)
            for (i = 0; i < arg.rows(); i++)
                if ((real((arg.complex_matrix_value())(i, j)) != 0) ||
                    (imag((arg.complex_matrix_value())(i, j)) != 0)) {
                    *(Ir + index) = i;
                    index++;
                }
              
    }

    return Ir;
}

int *oct_GetSparseJc(octave_value arg) {
    int    *Jc;
    int    i, j;
    int    count;

    Jc = (int *) malloc(arg.columns() * sizeof(int) + 1);

    if (arg.is_real_type()) {
        count = 0;
        for (j = 0; j < arg.columns(); j++)
          for (i = 0; i < arg.rows(); i++)
             if ((arg.matrix_value())(i, j) != 0 ) count++;
        *(Jc + arg.columns()) = count;

        count = 0;
        for (j = 0; j < arg.columns(); j++) {
            *(Jc + j) = count;
            for (i = 0; i < arg.rows(); i++)
                if ((arg.matrix_value())(i, j) != 0) count++;
        }
    } else {
        count = 0;
        for (j = 0;j<arg.columns();j++)
            for (i = 0; i < arg.rows(); i++)
                if ((real((arg.complex_matrix_value())(i,j)) != 0) || 
                    (imag((arg.complex_matrix_value())(i,j)) != 0) )
                    count++;
        *(Jc + arg.columns()) = count;

        count = 0;
        for (j = 0; j < arg.columns(); j++)
        {
          *(Jc + j) = count;
          for (i = 0; i< arg.rows(); i++)
             if ((real((arg.complex_matrix_value())(i, j)) != 0) ||
                (imag((arg.complex_matrix_value())(i, j)) != 0)) count++;
        }
    }

    return Jc;
}
