/** 
 * @file
 *
 * This is the definition for a lexical analyzer for the GridSolve
 *  Interface Definition Language.
 *
 */
/* $Id: problem_idl_lexer.l,v 1.28 2008/10/27 19:30:29 seymour Exp $ */
/* $UTK_Copyright: $ */

 
%{
#include <stdio.h>
#include <string.h>

#include "idl_parsing_defs.h"

#include "problem.h"
#include "utility.h"
#include "problem_idl_parser.h" 

int lineno = 1;

%}

D                       [0-9]
L                       [a-zA-Z_]
H                       [a-fA-F0-9]
E                       [Ee][+-]?{D}+
FS                      (f|F|l|L)
IS                      (u|U|l|L)*

identifier              {L}({L}|{D})*

white      [ \t\v\f]
newline    [\n]

%option nounput

%%

%{
  /* this code segment allows setting the first token returned
   * to notify the parser whether we are parsing a gsIDL file or
   * evaluating an expression.
   */
  extern int first_tok;
  if (first_tok)
  {
    int holdtok = first_tok;
    first_tok = 0;
    return holdtok;
  }
%}

{white}+      /* ignore whitespace */;
{newline}     lineno++;
"#"[^\n]*     /* ignore comments */;
FUNCTION      {  return FUNCTION;   }
SUBROUTINE    {  return SUBROUTINE; }
IN            {  return IN_TOK;     }
OUT           {  return OUT_TOK;    }
INOUT         {  return INOUT_TOK;  }
VAROUT        {  return VAROUT;     }
WORKSPACE     {  return WORKSPACE;  }
int           {  return INT_TOK;    }
float         {  return FLOAT_TOK;  }
double        {  return DOUBLE_TOK; }
short         {  return SHORT_TOK;  }
long          {  return LONG_TOK;   }
void          {  return VOID_TOK;   }
scomplex      {  return SCOMPLEX;   }
dcomplex      {  return DCOMPLEX;   }
char          {  return CHAR_TOK;   }
file          {  return FILE_TOK;   }
enum          {  return ENUM_TOK;   }

L?\"(\\.|[^\\"])*\"     {   idl_lval.strp = idl_text+1; 
                            idl_lval.strp[strlen(idl_lval.strp)-1] = '\0';
                            return STRING_LITERAL;  }

LANGUAGE           {  idl_lval.strp = idl_text; 
                      return LANGUAGE;  }

MAJOR              {  idl_lval.strp = idl_text; 
                      return MAJOR;  }

LIBS               {  idl_lval.strp = idl_text; 
                      return LIBS;  }

INCLUDES           {  idl_lval.strp = idl_text; 
                      return INCLUDES;  }

COMPLEXITY         {  idl_lval.strp = idl_text; 
                      return COMPLEXITY;  }

PARALLEL           {  idl_lval.strp = idl_text; 
                      return PARALLEL;  }

CODE               {  idl_lval.strp = idl_text; 
                      return CODE;  }

">>"                    { return(RIGHT_OP); }
"<<"                    { return(LEFT_OP); }
"&&"                    { return(AND_OP); }
"||"                    { return(OR_OP); }
"<="                    { return(LE_OP); }
">="                    { return(GE_OP); }
"=="                    { return(EQ_OP); }
"!="                    { return(NE_OP); }
":"                     { return(':'); }
"("                     { return('('); }
")"                     { return(')'); }
"{"                     { return('{'); }
"}"                     { return('}'); }
("["|"<:")              { return('['); }
("]"|":>")              { return(']'); }
"&"                     { return('&'); }
"!"                     { return('!'); }
"~"                     { return('~'); }
"-"                     { return('-'); }
"+"                     { return('+'); }
"*"                     { return('*'); }
"/"                     { return('/'); }
"%"                     { return('%'); }
"<"                     { return('<'); }
">"                     { return('>'); }
"^"                     { return('^'); }
"|"                     { return('|'); }
"?"                     { return('?'); }
","                     { return(','); }
"'"                     { return('\''); }
"="                     { return('='); }

0[xX]{H}+{IS}?          {  idl_lval.strp = idl_text; 
                           return CONSTANT;  }

0{D}+{IS}?              {  idl_lval.strp = idl_text; 
                           return CONSTANT;  }

{D}+{IS}?               {  idl_lval.strp = idl_text; 
                           return CONSTANT;  }

L?'(\\.|[^\\'])+'       {  idl_lval.strp = idl_text; 
                           return CONSTANT;  }

{D}+{E}{FS}?            {  idl_lval.strp = idl_text; 
                           return CONSTANT;  }

{D}*"."{D}+({E})?{FS}?  {  idl_lval.strp = idl_text; 
                           return CONSTANT;  }

{D}+"."{D}*({E})?{FS}?  {  idl_lval.strp = idl_text; 
                           return CONSTANT;  }

{identifier}            {  idl_lval.strp = idl_text; 
                           return IDENTIFIER;  }
%%

int yywrap(void) {
  return 1;
}

void
idl_lexer_free_memory()
{
  idl__delete_buffer(YY_CURRENT_BUFFER);
}
