; Tests passing filenames and packed filenames (arrays of filenames)
;
; $Id: filetest.pro,v 1.2 2007/03/05 18:10:23 seymour Exp $
; $UTK_Copyright: $
;

pro create_gs_test_file,fname,text
  openw, lun, fname, /get_lun
  printf,lun,text
  free_lun,lun
end

function check_file_results, fname, corr
  openr, lun, fname, /get_lun
  str=''
  readf, lun, str
  free_lun, lun
  if(str ne corr) then begin
    print, 'results incorrect!'
    return, 1
  endif else begin
    print, 'results ok.'
    return, 0
  endelse
end

pro filetest
  on_error, 2
  nfail=0

  numfiles=3

  infile = 'filetest.in'
  inoutfile = 'filetest.inout'
  outfile = 'filetest.out'

  packed_infile = make_array(numfiles, /string)
  packed_inoutfile = make_array(numfiles, /string)
  packed_outfile = make_array(numfiles, /string)

  for i=0l, numfiles-1 do begin
    packed_infile(i)='packedfile_'+strtrim(string(i),2)+'.in'
    packed_inoutfile(i)='packedfile_'+strtrim(string(i),2)+'.inout'
    packed_outfile(i)='packedfile_'+strtrim(string(i),2)+'.out'
  endfor

  create_gs_test_file, infile, "this is the input file."
  create_gs_test_file, inoutfile, "this is the input/output file."

  for i=0l, numfiles-1 do begin
    msg_in = 'this is packed input file number '+strtrim(string(i),2)
    msg_inout = 'this is packed input/output file number '+strtrim(string(i),2)
    create_gs_test_file, packed_infile(i), msg_in
    create_gs_test_file, packed_inoutfile(i), msg_inout
  endfor

  status = gs_call('filetest', infile, inoutfile, outfile, $
     packed_infile, packed_inoutfile, packed_outfile, numfiles)

  inout_corr = 'This is the input/output file.'
  out_corr = 'uijt jt uif joqvu gjmf.'

  print, 'Checking inout file'
  nfail += check_file_results(inoutfile, inout_corr)
  print, 'Checking out file'
  nfail += check_file_results(outfile, out_corr)

  for i=0l, numfiles-1 do begin
    inout_corr = 'This is packed input/output file number '+strtrim(string(i),2)
    print, 'Checking inout file number ', i
    nfail += check_file_results(packed_inoutfile(i), inout_corr)
    out_corr = 'uijt jt qbdlfe joqvu gjmf ovncfs '+strtrim(string(i),2)
    print, 'Checking out file number ', i
    nfail += check_file_results(packed_outfile(i), out_corr)
  endfor

  print, '----------------------'
  if(nfail NE 0) then begin
     print, nfail, " arguments had incorrect results!"
  endif else begin
     print, "All results were ok."
  endelse

end
