; General test of the GridSolve IDL client.  Runs various services
; in blocking and non-blocking mode.
;
; $Id: idltest.pro,v 1.2 2007/03/09 19:35:37 seymour Exp $ 
; $UTK_Copyright: $ 
;

pro idltest 
  on_error, 2
  fail_num = 0

  print, "Test 1: Sending a blocking request for iqsort()"
  size   = 4
  sorted = [4l, 3l, 2l, 1l]
  tmp    = sorted
  status = gs_call('iqsort()', sorted, size)
  if (status NE 0) then print, "GridRPC Error: ", gs_error_string(status)
  if (array_equal(tmp(sort(tmp)), sorted)) $
    then print, "Test 1 SUCCESS" else begin
      fail_num++
      print, "Test 1 FAILURE"
  endelse

  print, "Test 2: Sending a non-blocking request for iqsort()"
  size   = 4
  sorted = [4l, 3l, 2l, 1l]
  tmp    = sorted
  req    = gs_call_async('iqsort()', sorted, size)
  if (req LT 0) then print, "GridRPC Error: ", gs_error_string(req) $
  else begin
    status = gs_probe(req)
    print, "Probing: ", gs_error_string(status)
    print, "Waiting for completion"
    status = gs_wait(req)
    if (status NE 0) then print, "GridRPC Error: ", gs_error_string(status)
  endelse
  if (array_equal(tmp(sort(tmp)), sorted)) $
    then print, "Test 2 SUCCESS" else begin
      fail_num++
      print, "Test 2 FAILURE"
  endelse  

  print, "Test 3: Sending a blocking request for dqsort()"
  size   = 4
  sorted = [4d, 3d, 2d, 1d]
  tmp    = sorted
  status = gs_call('dqsort()', sorted, size)
  if (status NE 0) then print, "GridRPC Error: ", gs_error_string(status)
  if (array_equal(tmp(sort(tmp)), sorted)) $
    then print, "Test 3 SUCCESS" else begin
      fail_num++
      print, "Test 3 FAILURE"
  endelse

  print, "Test 4: Sending a non-blocking request for dqsort()"
  sorted = [4d, 3d, 2d, 1d]
  tmp    = sorted
  req    = gs_call_async('dqsort()', sorted, size)
  if (req LT 0) then print, "GridRPC Error: ", gs_error_string(req) $
  else begin
    status = gs_probe(req)
    print, "Probing: ", gs_error_string(status)
    print, "Waiting for completion"
    status = gs_wait(req)
    if (status NE 0) then print, "GridRPC Error: ", gs_error_string(status)
  endelse
  if (array_equal(tmp(sort(tmp)), sorted)) $
    then print, "Test 4 SUCCESS" else begin
      fail_num++
      print, "Test 4 FAILURE"
  endelse  

  print, "Test 5: Sending a blocking request for dgemm()"
  size   = 10
  A      = dindgen(size, size)
  B      = dindgen(size, size)
  C      = dindgen(size, size)
  status = gs_call('dgemm()', 'N', 'N', size, size, size, $
                    1, A, size, B, size, 0, C, size)
  if (status NE 0) then print, "GridRPC Error: ", gs_error_string(status)
  if (array_equal(A##B,C)) then print, "Test 5 SUCCESS" $
  else begin
      fail_num++
      print, "Test 5 Failure"
  endelse

  print, "Test 6: Sending a nonblocking request for dgemm()"
  size   = 10
  A      = dindgen(size, size)
  B      = dindgen(size, size)
  C      = dindgen(size, size)
  req    = gs_call_async('dgemm()', 'N', 'N', size, size, size, $
                       1, A, size, B, size, 0, C, size)
  if (req LT 0) then print, "GridRPC Error: ", gs_error_string(req) $
  else begin
    print, "Waiting for completion"
    status = gs_wait(req)
    if (status NE 0) then print, "GridRPC Error: ", gs_error_string(status)
  endelse
  if (array_equal(A##B,C)) then print, "Test 6 SUCCESS" $
  else begin
      fail_num++
      print, "Test 6 Failure"
  endelse

  print, "Test 7: Sending a blocking request for ddot()"
  seed   = 2
  size   = 10
  x      = double(randomn(seed, size))
  y      = double(randomn(seed, size))
  y2     = y
  result = 0d
  status = gs_call('ddot()', size, x, 1, y, 1, result)
  if (status NE 0) then print, "GridRPC Error: ", gs_error_string(status)
  if (total(x*y2) EQ result) then print, "Test 7 SUCCESS" $
  else begin
      fail_num++
      print, "Test 7 Failure"
  endelse


  print, "Test 8: Sending a nonblocking request for ddot()"
  seed   = 2
  size   = 10
  x      = double(randomn(seed, size))
  y      = double(randomn(seed, size))
  y2     = y
  result = 0d
  req    = gs_call_async('ddot()', size, x, 1, y, 1, result)
  if (req LT 0) then print, "GridRPC Error: ", gs_error_string(req) $
  else begin
    print, "Waiting for completion"
    status = gs_wait(req)
    if (status NE 0) then print, "GridRPC Error: ", gs_error_string(status)
  endelse
  if (total(x*y2) EQ result) then print, "Test 8 SUCCESS" $
  else begin
      fail_num++
      print, "Test 8 Failure"
  endelse


  print, "Test 9: Sending a blocking request for dgesv()"
  seed   = 1
  size   = 3
  ;A      = double(randomn(seed, size, size))
  A      = double([[1, 0, 0],[0, 1, 0],[0, 0, 1]])
  Acopy  = A
  ;b      = double(randomn(seed, size))
  b      = double([3, 4, 5]) 
  bcopy  = b
  ipiv   = lonarr(size)
  info   = -1l
  status = gs_call('dgesv()', size, 1, A, size, ipiv, b, size, info)
  if (status NE 0) then print, "GridRPC Error: ", gs_error_string(status)
  ;print, 'A', Acopy
  ;print, 'b', bcopy
  ;print, 'x', b
  ;print, 'Ax=', Acopy##b
  if (norm(Acopy##b - bcopy) LE 1.0e-9 ) then print, "Test 9 SUCCESS" $
  else begin
      fail_num++
      print, "Test 9 Failure"
  endelse

  print, "Test 10: Sending a non-blocking request for dgesv()"
  seed   = 1
  size   = 3
  ;A      = double(randomn(seed, size, size))
  A      = double([[1, 0, 0],[0, 1, 0],[0, 0, 1]])
  Acopy  = A
  ;b      = double(randomn(seed, size))
  b      = double([3, 4, 5]) 
  bcopy  = b
  ipiv   = lonarr(size)
  info   = -1l
  req    = gs_call_async('dgesv()', size, 1, A, size, ipiv, b, size, info)
  if (req LT 0) then print, "GridRPC Error: ", gs_error_string(req) $
  else begin
    print, "Waiting for completion"
    status = gs_wait(req)
    if (status NE 0) then print, "GridRPC Error: ", gs_error_string(status)
  endelse
  if (norm(Acopy##b - bcopy) LE 1.0e-9 ) then print, "Test 10 SUCCESS" $
  else begin
      fail_num++
      print, "Test 10 Failure"
  endelse

  print, "Test 11 : Sending a blocking request for daxpy()"
  size   = 10
  x      = double(randomn(seed, size))
  y      = double(randomn(seed, size))
  y2     = y
  status = gs_call('daxpy()', size, 0.5, x, 1, y, 1)
  if (status NE 0) then print, "GridRPC Error: ", gs_error_string(status)
  if (array_equal(0.5*x+y2, y)) then print, "Test 11 SUCCESS" $
  else begin
      fail_num++
      print, "Test 11 Failure"
  endelse

  print, "Test 12 : Sending a non-blocking request for daxpy()"
  size   = 10
  x      = double(randomn(seed, size))
  y      = double(randomn(seed, size))
  y2     = y
  req = gs_call_async('daxpy()', size, 0.5, x, 1, y, 1)
  if (req LT 0) then print, "GridRPC Error: ", gs_error_string(req) $
  else begin
    print, "Waiting for completion"
    status = gs_wait(req)
    if (status NE 0) then print, "GridRPC Error: ", gs_error_string(status)
  endelse
  if (array_equal(0.5*x+y2, y)) then print, "Test 12 SUCCESS" $
  else begin
      fail_num++
      print, "Test 12 Failure"
  endelse

  print, "Test 13 : Sending a blocking request for ddot()"
  size   = 10
  x      = double(randomn(seed, size))
  y      = double(randomn(seed, size))
  y2     = y
  result = 0d
  status = gs_call('ddot()', size, x, 1, y, 1, result)
  if (status NE 0) then print, "GridRPC Error: ", gs_error_string(status)
  if (total(x*y2) EQ result) then print, "Test 13 SUCCESS" $
  else begin
      fail_num++
      print, "Test 13 Failure"
  endelse

  print, "Test 14 : Sending a non-blocking request for ddot()"
  size   = 10
  x      = double(randomn(seed, size))
  y      = double(randomn(seed, size))
  y2     = y
  result = 0d
  req = gs_call_async('ddot()', size, x, 1, y, 1, result)
  if (req LT 0) then print, "GridRPC Error: ", gs_error_string(req) $
  else begin
    print, "Waiting for completion"
    status = gs_wait(req)
    if (status NE 0) then print, "GridRPC Error: ", gs_error_string(status)
  endelse
  if (total(x*y2) EQ result) then print, "Test 14 SUCCESS" $
  else begin
      fail_num++
      print, "Test 14 Failure"
  endelse

  print, "Test 15 : Sending a blocking request for dposv()"
  size   = 5
  A      = double(randomn(seed, size, size))
  A      = transpose(A)##A
  Acopy  = A
  b      = double(randomn(seed, size))
  bcopy  = b
  status = gs_call('dposv()', 'U', size, 1, A, size, b, size, info)
  if (status NE 0) then print, "GridRPC Error: ", gs_error_string(status)
  if (norm(Acopy##b - bcopy) LE 1.0e-9 ) then print, "Test 15 SUCCESS" $
  else begin
      fail_num++
      print, "Test 15 Failure"
  endelse

  print, "Test 16 : Sending a non-blocking request for dposv()"
  size   = 5
  A      = double(randomn(seed, size, size))
  A      = transpose(A)##A
  Acopy  = A
  b      = double(randomn(seed, size))
  bcopy  = b
  req = gs_call_async('dposv()', 'U', size, 1, A, size, b, size, info)
  if (req LT 0) then print, "GridRPC Error: ", gs_error_string(req) $
  else begin
    print, "Waiting for completion"
    status = gs_wait(req)
    if (status NE 0) then print, "GridRPC Error: ", gs_error_string(status)
  endelse
  if (norm(Acopy##b - bcopy) LE 1.0e-9 ) then print, "Test 16 SUCCESS" $
  else begin
      fail_num++
      print, "Test 16 Failure"
  endelse

  print, "Test 17 : Sending a blocking request for zdotc()"
  size   = 10
  x      = dcomplex(randomn(seed, size))
  y      = dcomplex(randomn(seed, size))
  y2     = y
  result = 0d
  status = gs_call('zdotc()', size, x, 1, y, 1, result)
  if (status NE 0) then print, "GridRPC Error: ", gs_error_string(status)
  if (total(x*y2) EQ result) then print, "Test 17 SUCCESS" $
  else begin
      fail_num++
      print, "Test 17 Failure"
  endelse

  print, "Test 18 : Sending a non-blocking request for zdotc()"
  size   = 10
  x      = dcomplex(randomn(seed, size))
  y      = dcomplex(randomn(seed, size))
  y2     = y
  result = 0d
  req = gs_call_async('zdotc()', size, x, 1, y, 1, result)
  if (req LT 0) then print, "GridRPC Error: ", gs_error_string(req) $
  else begin
    print, "Waiting for completion"
    status = gs_wait(req)
    if (status NE 0) then print, "GridRPC Error: ", gs_error_string(status)
  endelse
  if (total(x*y2) EQ result) then print, "Test 18 SUCCESS" $
  else begin
      fail_num++
      print, "Test 18 Failure"
  endelse

  ; SUMMARY INFO
  if (fail_num EQ 0) then print, "All 18 tests passed!" else $
  print, fail_num, " out of 18 tests failed!"

end
