; Invokes the GridSolve 'totaltest' service.  This tests most of the
; combinations of object types (scalar, vector, etc) and data types
; (int, float, etc).  It does not test sparse matrices, files, packed
; files, or varout types.  There are specific tests for those.
;
; $Id: totaltest.pro,v 1.1 2007/03/05 17:45:45 seymour Exp $ 
; $UTK_Copyright: $ 
;

pro totaltest
  on_error, 2
  nfail=0

  ivec_size = 5l
  fvec_size = 5l
  dvec_size = 5l
  scmp_vec_size = 5l
  dcmp_vec_size = 5l
  cvec_size = 6l
  mat_m = 2l
  mat_n = 3l

  int_in = 8080l
  int_inout = int_in + 100l
  int_o = 0l

  float_in = 1.32
  float_inout = float_in + 100.0
  float_o = 0.0

  double_in = 6.66d
  double_inout = double_in + 100.0d
  double_o = 0.0d

  scmp_in = complex(1.32, 2.0)
  scmp_inout = scmp_in + complex(100.0, 10.0)
  scmp_o = complex(0.0, 0.0)

  dcmp_in = dcomplex(6.66d, 2.0d)
  dcmp_inout = dcmp_in + dcomplex(100.0d, 10.0d)
  dcmp_o = dcomplex(0.0, 0.0)

  char_in = 'a'
  char_inout = 'i'
  char_o = 'x'

  ivec_in = make_array(ivec_size, /long, /index)
  ivec_inout = ivec_in + 100
  ivec_o = make_array(ivec_size, /long)

  fvec_in = make_array(fvec_size, /float, /index)
  fvec_inout = fvec_in + 100.0
  fvec_o = make_array(fvec_size, /float)

  dvec_in = make_array(dvec_size, /double, /index)
  dvec_inout = dvec_in + 100.0
  dvec_o = make_array(dvec_size, /double)

  scmp_vec_in = make_array(scmp_vec_size, /complex, /index)
  scmp_vec_in = scmp_vec_in + complex(0.0, 10.0)
  scmp_vec_inout = scmp_vec_in + complex(100.0,10.0)
  scmp_vec_o = make_array(scmp_vec_size, /complex)

  dcmp_vec_in = make_array(dcmp_vec_size, /dcomplex, /index)
  dcmp_vec_in = dcmp_vec_in + dcomplex(0.0, 10.0)
  dcmp_vec_inout = dcmp_vec_in + dcomplex(100.0,10.0)
  dcmp_vec_o = make_array(dcmp_vec_size, /dcomplex)

  cvec_in = 'abcdef'
  cvec_inout = 'ijklmn'
  cvec_io_corr = 'jklmno'
  cvec_o = 'xxxxxx'
  cvec_o_corr = 'bcdefg'

  imat_in = make_array(mat_n, mat_m, /long, /index)
  imat_inout = imat_in + 100
  imat_o = make_array(mat_n, mat_m, /long)

  fmat_in = make_array(mat_n, mat_m, /float, /index)
  fmat_inout = fmat_in + 100.0
  fmat_o = make_array(mat_n, mat_m, /float)

  dmat_in = make_array(mat_n, mat_m, /double, /index)
  dmat_inout = dmat_in + 100.0
  dmat_o = make_array(mat_n, mat_m, /double)

  scmp_mat_in = make_array(mat_n, mat_m, /complex, /index)
  scmp_mat_in = scmp_mat_in + complex(0.0, 10.0)
  scmp_mat_inout = scmp_mat_in + complex(100.0, 10.0)
  scmp_mat_o = make_array(mat_n, mat_m, /complex)

  dcmp_mat_in = make_array(mat_n, mat_m, /dcomplex, /index)
  dcmp_mat_in = dcmp_mat_in + dcomplex(0.0, 10.0)
  dcmp_mat_inout = dcmp_mat_in + dcomplex(100.0, 10.0)
  dcmp_mat_o = make_array(mat_n, mat_m, /dcomplex)

  cmat_in = ['abc','def']
  cmat_inout = ['jkl','mno']
  cmat_o = ['opq','rst']

  status = gs_call('ctotal', $
     int_in, int_inout, int_o, $
     float_in, float_inout, float_o, $
     double_in, double_inout, double_o, $
     scmp_in, scmp_inout, scmp_o, $
     dcmp_in, dcmp_inout, dcmp_o, $
     char_in, char_inout, char_o, $
     ivec_in, ivec_inout, ivec_o, ivec_size, $
     fvec_in, fvec_inout, fvec_o, fvec_size, $
     dvec_in, dvec_inout, dvec_o, dvec_size, $
     scmp_vec_in, scmp_vec_inout, scmp_vec_o, scmp_vec_size, $
     dcmp_vec_in, dcmp_vec_inout, dcmp_vec_o, dcmp_vec_size, $
     cvec_in, cvec_inout, cvec_o, cvec_size, $
     imat_in, imat_inout, imat_o, mat_m, mat_n, $
     fmat_in, fmat_inout, fmat_o, mat_m, mat_n, $
     dmat_in, dmat_inout, dmat_o, mat_m, mat_n, $
     scmp_mat_in, scmp_mat_inout, scmp_mat_o, mat_m, mat_n, $
     dcmp_mat_in, dcmp_mat_inout, dcmp_mat_o, mat_m, mat_n, $
     cmat_in, cmat_inout, cmat_o, mat_m, mat_n)

  if(status NE 0) then print, "GridRPC Error: ", gs_error_string(status)

  if(int_inout ne (int_in + 101l)) then begin
    print, "int_inout results incorrect!"
    nfail++
  endif else begin
    print, "int_inout results ok"
  endelse

  if(int_o ne (int_in + 1l)) then begin
    print, "int_o results incorrect"
    nfail++
  endif else begin
    print, "int_o results ok"
  endelse

  if(gs_eq(float_inout, float_in + 101.0) ne 1) then begin
    print, "float_inout results incorrect"
    nfail++
  endif else begin
    print, "float_inout results ok"
  endelse
  
  if(gs_eq(float_o, float_in + 1.0) ne 1) then begin
    print, "float_o results incorrect"
    nfail++
  endif else begin
    print, "float_o results ok"
  endelse 

  if(gs_eq(double_inout, double_in + 101.0d) ne 1) then begin
    print, "double_inout results incorrect"
    nfail++
  endif else begin
    print, "double_inout results ok"
  endelse

  if(gs_eq(double_o, double_in + 1.0d) ne 1) then begin
    print, "double_o results incorrect"
    nfail++
  endif else begin
    print, "double_o results ok"
  endelse

  if(gs_eq(scmp_inout, scmp_in + complex(101.0, 11.0)) ne 1) then begin
    print, "scmp_inout results incorrect"
    nfail++ 
  endif else begin
    print, "scmp_inout results ok"
  endelse

  if(gs_eq(scmp_o, scmp_in + complex(1.0, 1.0)) ne 1) then begin
    print, "scmp_o results incorrect"
    nfail++
  endif else begin
    print, "scmp_o results ok"
  endelse
  
  if(gs_eq(dcmp_inout, dcmp_in + dcomplex(101.0, 11.0)) ne 1) then begin
    print, "dcmp_inout results incorrect"
    nfail++
  endif else begin
    print, "dcmp_inout results ok"
  endelse

  if(gs_eq(dcmp_o, dcmp_in + dcomplex(1.0, 1.0)) ne 1) then begin
    print, "dcmp_o results incorrect"
    nfail++
  endif else begin
    print, "dcmp_o results ok"
  endelse

  if(char_inout ne 'j') then begin
    print, "char_inout results incorrect"
    nfail++
  endif else begin
    print, "char_inout results ok"
  endelse

  if(char_o ne 'b') then begin
    print, "char_o results incorrect"
    nfail++
  endif else begin
    print, "char_o results ok"
  endelse

  if(gs_vec_eq(ivec_inout, (ivec_in + 101), ivec_size) ne 1) then begin
    print, "ivec_inout results incorrect"
    nfail++
  endif else begin
    print, "ivec_inout results ok"
  endelse

  if(gs_vec_eq(ivec_o, (ivec_in + 1), ivec_size) ne 1) then begin
    print, "ivec_o results incorrect"
    nfail++
  endif else begin
    print, "ivec_o results ok"
  endelse

  if(gs_vec_eq(fvec_inout, (fvec_in + 101.0), fvec_size) ne 1) then begin
    print, "fvec_inout results incorrect"
    nfail++
  endif else begin
    print, "fvec_inout results ok"
  endelse

  if(gs_vec_eq(fvec_o, (fvec_in + 1.0), fvec_size) ne 1) then begin
    print, "fvec_o results incorrect"
    nfail++
  endif else begin
    print, "fvec_o results ok"
  endelse

  if(gs_vec_eq(dvec_inout, (dvec_in + 101.0d), dvec_size) ne 1) then begin
    print, "dvec_inout results incorrect"
    nfail++
  endif else begin
    print, "dvec_inout results ok"
  endelse

  if(gs_vec_eq(dvec_o, (dvec_in + 1.0d), dvec_size) ne 1) then begin
    print, "dvec_o results incorrect"
    nfail++
  endif else begin
    print, "dvec_o results ok"
  endelse

  if(gs_vec_eq(scmp_vec_inout, (scmp_vec_in + complex(101.0,11.0)), scmp_vec_size) ne 1) then begin
    print, "scmp_vec_inout results incorrect"
    nfail++
  endif else begin
    print, "scmp_vec_inout results ok"
  endelse

  if(gs_vec_eq(scmp_vec_o, (scmp_vec_in + complex(1.0,1.0)), scmp_vec_size) ne 1) then begin
    print, "scmp_vec_out results incorrect"
    nfail++
  endif else begin
    print, "scmp_vec_out results ok"
  endelse

  if(gs_vec_eq(dcmp_vec_inout, (dcmp_vec_in + dcomplex(101.0,11.0)), dcmp_vec_size) ne 1) then begin
    print, "dcmp_vec_inout results incorrect"
    nfail++
  endif else begin
    print, "dcmp_vec_inout results ok"
  endelse

  if(gs_vec_eq(dcmp_vec_o, (dcmp_vec_in + dcomplex(1.0,1.0)), dcmp_vec_size) ne 1) then begin
    print, "dcmp_vec_out results incorrect"
    nfail++
  endif else begin
    print, "dcmp_vec_out results ok"
  endelse

  if(cvec_inout ne cvec_io_corr) then begin
    print, "cvec_inout results incorrect"
    nfail++
  endif else begin
    print, "cvec_inout results ok"
  endelse

  if(cvec_o ne cvec_o_corr) then begin
    print, "cvec_o results incorrect"
    nfail++
  endif else begin
    print, "cvec_o results ok"
  endelse

  if(gs_mat_eq(imat_inout, (imat_in + 101), mat_m, mat_n) ne 1) then begin
    print, "imat_inout results incorrect"
    nfail++
  endif else begin
    print, "imat_inout results ok"
  endelse

  if(gs_mat_eq(imat_o, (imat_in + 1), mat_m, mat_n) ne 1) then begin
    print, "imat_o results incorrect"
    nfail++
  endif else begin
    print, "imat_o results ok"
  endelse

  if(gs_mat_eq(fmat_inout, (fmat_in + 101.0), mat_m, mat_n) ne 1) then begin
    print, "fmat_inout results incorrect"
    nfail++
  endif else begin
    print, "fmat_inout results ok"
  endelse

  if(gs_mat_eq(fmat_o, (fmat_in + 1.0), mat_m, mat_n) ne 1) then begin
    print, "fmat_o results incorrect"
    nfail++
  endif else begin
    print, "fmat_o results ok"
  endelse

  if(gs_mat_eq(dmat_inout, (dmat_in + 101.0d), mat_m, mat_n) ne 1) then begin
    print, "dmat_inout results incorrect"
    nfail++
  endif else begin
    print, "dmat_inout results ok"
  endelse

  if(gs_mat_eq(dmat_o, (dmat_in + 1.0d), mat_m, mat_n) ne 1) then begin
    print, "dmat_o results incorrect"
    nfail++
  endif else begin
    print, "dmat_o results ok"
  endelse

  if(gs_mat_eq(scmp_mat_inout, (scmp_mat_in + complex(101.0,11.0)), mat_m, mat_n) ne 1) then begin
    print, "scmp_mat_inout results incorrect"
    nfail++
  endif else begin
    print, "scmp_mat_inout results ok"
  endelse

  if(gs_mat_eq(scmp_mat_o, (scmp_mat_in + complex(1.0,1.0)), mat_m, mat_n) ne 1) then begin
    print, "scmp_mat_out results incorrect"
    nfail++
  endif else begin
    print, "scmp_mat_out results ok"
  endelse

  if(gs_mat_eq(dcmp_mat_inout, (dcmp_mat_in + dcomplex(101.0,11.0)), mat_m, mat_n) ne 1) then begin
    print, "dcmp_mat_inout results incorrect"
    nfail++
  endif else begin
    print, "dcmp_mat_inout results ok"
  endelse

  if(gs_mat_eq(dcmp_mat_o, (dcmp_mat_in + dcomplex(1.0,1.0)), mat_m, mat_n) ne 1) then begin
    print, "dcmp_mat_o results incorrect"
    nfail++
  endif else begin
    print, "dcmp_mat_o results ok"
  endelse

  cmat_io_corr_cat = ['klmnop']
  cmat_io_cat = cmat_inout(0)+cmat_inout(1)

  if(cmat_io_cat ne cmat_io_corr_cat) then begin
    print, "cmat_io results incorrect"
    nfail++
  endif else begin
    print, "cmat_io results ok"
  endelse

  cmat_o_corr_cat = ['bcdefg']
  cmat_o_cat = cmat_o(0)+cmat_o(1)

  if(cmat_o_cat ne cmat_o_corr_cat) then begin
    print, "cmat_o results incorrect"
    nfail++
  endif else begin
    print, "cmat_o results ok"
  endelse

  print, '----------------------'
  if(nfail NE 0) then begin
     print, nfail, " arguments had incorrect results!"
  endif else begin
     print, "All results were ok."
  endelse

end
