/**
 * @file
 *
 * This call tests whether failure on the server side is
 * dealt with correctly when making calls.
 *
 */
/* $Id: abort_test.c,v 1.3 2005/12/14 18:39:17 seymour Exp $ */
/* $UTK_Copyright: $ */

#include <stdio.h>
#include <stdlib.h>

#include "grpc.h"
#include "totaltest.h"

grpc_sessionid_t 
do_call(grpc_function_handle_t *handle, int *val) {
  grpc_sessionid_t sessionID;
  grpc_error_t status;

  status = grpc_call_async(handle, &sessionID, val);

  if(status != GRPC_NO_ERROR) {
    grpc_perror("grpc_call");
    die(sessionID);
  }

  printf("Waiting for result.  Ignore any error messages printed here.\n");
  
  return sessionID;
}

int
do_blocking_call(grpc_function_handle_t *handle, int *val) {
  grpc_error_t status;

  status = grpc_call(handle, val);

  if(status != GRPC_NO_ERROR) {
    grpc_perror("grpc_call");
    return -1;
  }

  printf("Call returned successfully\n");

  return 0;
}

int
check_wait_status(grpc_error_t status) {
  if(status == GRPC_NO_ERROR) {
    printf("\nThis is not good.  grpc_wait should not return successfully.\n");
    die(EXIT_SUCCESS);
  }

  return 0;
}

int
main(int argc, char *argv[])
{
  grpc_function_handle_t handle;
  int val, sessionIDs[1], completed_id, exit_status;

  if(grpc_initialize(NULL) != GRPC_NO_ERROR) {
    grpc_perror("grpc_initialize");
    die(EXIT_FAILURE);
  }

  if(grpc_function_handle_default(&handle, "gs_abort") != GRPC_NO_ERROR) {
    fprintf(stderr,"Error creating function handle\n");
    die(EXIT_FAILURE);
  }

  exit_status = EXIT_FAILURE;
  val = -1;

  if(!strstr(argv[0], "async")) {
    exit_status = do_blocking_call(&handle, &val) == -1 ? EXIT_SUCCESS : EXIT_FAILURE;
  }
  else {
    printf("\nTesting grpc_wait_any()\n");
    do_call(&handle, &val);
    check_wait_status(grpc_wait_any(&(sessionIDs[0])));

    printf("\nTesting grpc_wait_or()\n");
    sessionIDs[0] = do_call(&handle, &val);
    check_wait_status(grpc_wait_or(sessionIDs, 1, &completed_id));

    printf("\nTesting grpc_wait_and()\n");
    sessionIDs[0] = do_call(&handle, &val);
    check_wait_status(grpc_wait_and(sessionIDs, 1));

    printf("\nTesting grpc_wait_all()\n");
    sessionIDs[0] = do_call(&handle, &val);
    check_wait_status(grpc_wait_all());

    printf("\nTesting grpc_wait()\n");
    sessionIDs[0] = do_call(&handle, &val);
    check_wait_status(grpc_wait(sessionIDs[0]));

    printf("\ngrpc_wait successfully detected abnormal service termination.\n");

    exit_status = EXIT_SUCCESS;
  }

  grpc_function_handle_destruct(&handle);

  printf("Finalizing...\n");

  grpc_finalize();

  printf("Done.\n");

  die(EXIT_SUCCESS);
}
