/**
 * @file
 *
 */
/* $Id: agent_latency_test.c,v 1.1 2007/10/23 20:02:50 seymour Exp $ */
/* $UTK_Copyright: $ */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include <stdio.h>
#include <stdlib.h>

#ifdef HAVE_VALUES_H
#include<values.h>
#else  /* HAVE_VALUES_H */
#ifdef HAVE_LIMITS_H
#include <limits.h>
#endif /* HAVE_LIMITS_H */
#ifdef HAVE_FLOAT_H
#include <float.h>
#define MAXDOUBLE DBL_MAX
#endif /* HAVE_FLOAT_H */
#endif /* HAVE_VALUES_H */

#ifndef MAXDOUBLE
#define MAXDOUBLE 1.7976931348623157e+308
#endif

#include "portability.h"	/* for <sys/time.h> */
#include "grpc.h"
#include "totaltest.h"

double elapsed_time(const struct timeval*, const struct timeval*);

int
main(int argc, char *argv[])
{
  int i, j, *x, size, rep, status;
  grpc_function_handle_t handle;
  grpc_profile_t gsprof;
  double time_sum;

  if(argc != 3) {
    fprintf(stderr,"\nUsage: %s <data_size> <reps>\n", argv[0]);
    fprintf(stderr,"\n  Transmits array of size <data_size> and measures\n");
    fprintf(stderr,"  agent latency.  The number of repetitions is\n");
    fprintf(stderr,"  specified by <reps>.\n");
    die(EXIT_FAILURE);
  }

  size = atoi(argv[1]);
  rep = atoi(argv[2]);

  x = (int *)malloc(size*sizeof(x[0]));

  if(!x) {
    grpc_perror("malloc");
    die(EXIT_FAILURE);
  }

  if(grpc_initialize(NULL) != GRPC_NO_ERROR) {
    grpc_perror("grpc_initialize");
    die(EXIT_FAILURE);
  }

  for(i=0;i<size;i++)
    x[i] = i%64;

  time_sum = 0.0;

  for(j=0;j<rep;j++) {
    if(grpc_function_handle_init(&handle, "bind_servers_at_call_time", 
        "vpass_int") != GRPC_NO_ERROR)
    {
      fprintf(stderr,"Error creating function handle\n");
      die(EXIT_FAILURE);
    }

    if(grpc_profile(&gsprof) != GRPC_NO_ERROR) {
      fprintf(stderr,"Error: profiling not enabled in this build.\n");
      die(EXIT_FAILURE);
    }

    status = grpc_call(&handle, x, i);

    if(status != GRPC_NO_ERROR) {
      printf("GRPC error status  = %d\n", status);
      grpc_perror("grpc_call");
      die(status);
    }

    time_sum += gsprof.agent_comm;

    grpc_function_handle_destruct(&handle);
  }

  printf("%d: %g\n",(int)(i*sizeof(x[0])), time_sum/(double)rep);

  printf("Finalizing...\n");

  grpc_finalize();

  printf("Done.\n");

  die(EXIT_SUCCESS);
}

/**
 * Computes the elapsed wall time between the two given 
 * times (timeval structs).
 *
 * @param t0 -- starting time
 * @param t1 -- ending time
 *
 * @returns the elapsed time between the two times
 **/

double
elapsed_time(const struct timeval* t0, const struct timeval* t1)
{
  double ret = t1->tv_sec - t0->tv_sec;
  double uret = t1->tv_usec - t0->tv_usec;
  return ret + uret / 1E6;
}
