/**
 * @file
 *
 * This program tests the GridRPC argument stack calling sequence using a 
 * call.  The program is invoked as follows for blocking calls:
 *   argstack_totaltest [ctotal | ftotal]
 * and for non-blocking calls:
 *   argstack_totaltest_async [ctotal | ftotal]
 *
 * The command line argument represents which server-side routine to call.
 * "ctotal" is implemented in C and "ftotal" is implemented in Fortran.
 * Calling the Fortran routine allows verifying that matrices are transposed
 * properly.
 */
/* $Id: argstack_totaltest.c,v 1.10 2005/12/14 18:39:17 seymour Exp $ */
/* $UTK_Copyright: $ */

#include <stdio.h>
#include <stdlib.h>

#include "grpc.h"
#include "totaltest.h"

int
main(int argc, char *argv[])
{
  grpc_function_handle_t handle;
  char *test_name;
  grpc_error_t status;
#include "totaltest_decl.c"

  test_name = parse_totaltest_cmdline(argc, argv);

#include "totaltest_init.c"

  if(grpc_initialize(NULL) != GRPC_NO_ERROR) {
    grpc_perror("grpc_initialize");
    die(EXIT_FAILURE);
  }

  if(grpc_function_handle_default(&handle, test_name) != GRPC_NO_ERROR) {
    fprintf(stderr,"Error creating function handle\n");
    die(EXIT_FAILURE);
  }

  if(!strstr(argv[0], "async")) {
    status = grpc_call_arg_stack(&handle, stack);

    if(status != GRPC_NO_ERROR) {
      printf("GRPC error status  = %d\n", status);
      grpc_perror("grpc_call");
      die(status);
    }
  }
  else {
    grpc_sessionid_t sessionID; 

    status = grpc_call_arg_stack_async(&handle, &sessionID, stack);

    if(status != GRPC_NO_ERROR) {
      printf("GRPC error status  = %d\n", status);
      grpc_perror("grpc_call_async");
      die(EXIT_FAILURE);
    }

    printf("Waiting for result.\n");
  
    status = grpc_wait(sessionID);
  
    if(status != GRPC_NO_ERROR) {
      grpc_perror("grpc_wait");
      die(EXIT_FAILURE);
    }
  }

#include "totaltest_results.c"

  grpc_function_handle_destruct(&handle);
  grpc_arg_stack_destruct(stack);

  printf("Finalizing...\n");

  grpc_finalize();

  printf("Done.\n");

  die(EXIT_SUCCESS);
}
