/**
 * @file
 *
 * This program tests whether grpc_cancel is working.
 */
/* $Id: cancel_test.c,v 1.1 2005/12/15 20:53:02 seymour Exp $ */
/* $UTK_Copyright: $ */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "grpc.h"
#include "totaltest.h"

int
main(int argc, char *argv[])
{
  grpc_function_handle_t handle;
  grpc_sessionid_t sessionID;
  grpc_error_t status;
  int len, wait;

  if(argc != 3) {
    len = 30;
    wait = 5;
  }
  else {
    len = atoi(argv[1]);
    wait = atoi(argv[2]);
  }

  if(grpc_initialize(NULL) != GRPC_NO_ERROR) {
    grpc_perror("grpc_initialize");
    die(EXIT_FAILURE);
  }

  if(grpc_function_handle_default(&handle, "sleeptest") != GRPC_NO_ERROR) {
    fprintf(stderr,"Error creating function handle\n");
    die(EXIT_FAILURE);
  }

  printf("Calling sleeptest...\n");

  status = grpc_call_async(&handle, &sessionID, len);

  if(status != GRPC_NO_ERROR) {
    grpc_perror("grpc_call");
    die(sessionID);
  }

  sleep(wait);

  printf("Going to cancel.\n");
  status = grpc_cancel(sessionID);

  if(status == GRPC_NO_ERROR)
    printf("Successfully cancelled.\n");
  else {
    fprintf(stderr, "Could not cancel.\n");
    die(EXIT_FAILURE);
  }
 
  if(grpc_function_handle_destruct(&handle) != GRPC_NO_ERROR) {
    fprintf(stderr,"Warning: failed to destroy function handle\n");
    die(EXIT_FAILURE);
  }

  printf("Finalizing...\n");

  grpc_finalize();

  printf("Done.\n");

  die(EXIT_SUCCESS);
}
