/**
 * @file
 *
 * This will be used to test whether the communication cost
 * estimation is working.
 *
 */
/* $Id: comm_test.c,v 1.6 2007/06/15 03:29:58 anonymous Exp $ */
/* $UTK_Copyright: $ */

#include <stdio.h>
#include <stdlib.h>

#include "grpc.h"
#include "totaltest.h"

double
elapsed_time(const struct timeval*, const struct timeval*);

int main(int argc, char *argv[])
{
  grpc_function_handle_t *handles;
  grpc_sessionid_t *sessionIDs;
  grpc_profile_t *profiles;
  grpc_error_t status;
  struct timeval t1, t2;
  int i, j, n, tasks, prof_enabled;
  double *x, et;

  if(argc != 3) {
    fprintf(stderr, "Usage: %s <num elements> <num tasks>\n", argv[0]);
    die(EXIT_FAILURE);
  }

  n = atoi(argv[1]);
  tasks = atoi(argv[2]);
  prof_enabled = 0;

  x = (double *) malloc(tasks * n * sizeof(*x));

  if(!x) {
    fprintf(stderr, "out of memory\n");
    die(EXIT_FAILURE);
  }

  for(i=0;i<tasks;i++)
    for(j=0;j<n;j++)
      x[i*n + j] = ((double)rand()/32767.0);

  sessionIDs = (grpc_sessionid_t *) malloc(tasks * sizeof(grpc_sessionid_t));

  if(!sessionIDs) {
    fprintf(stderr, "out of memory\n");
    die(EXIT_FAILURE);
  }

  handles = (grpc_function_handle_t *) malloc(tasks * sizeof(grpc_function_handle_t));

  if(!handles) {
    fprintf(stderr, "out of memory\n");
    die(EXIT_FAILURE);
  }

  profiles = (grpc_profile_t *) malloc(tasks * sizeof(grpc_profile_t));

  if(!profiles) {
    fprintf(stderr, "out of memory\n");
    die(EXIT_FAILURE);
  }

  memset(profiles, 0, tasks * sizeof(grpc_profile_t));

  handles = (grpc_function_handle_t *) malloc(tasks * sizeof(grpc_function_handle_t));

  if(grpc_initialize("comm_test.cfg") != GRPC_NO_ERROR) {
    grpc_perror("grpc_initialize");
    die(EXIT_FAILURE);
  }

  gettimeofday(&t1, 0);

  for(i=0;i<tasks;i++) {
    prof_enabled = grpc_profile(&(profiles[i])) == GRPC_NO_ERROR;

    if(grpc_function_handle_init(&handles[i], "bind_servers_at_call_time", "dqsort") != GRPC_NO_ERROR) {
      fprintf(stderr,"Error creating function handle\n");
      die(EXIT_FAILURE);
    }

    status = grpc_call_async(&handles[i], &sessionIDs[i], x + (i*n), n);

    if(status != GRPC_NO_ERROR) {
      printf("GRPC error status  = %d\n", status);
      grpc_perror("grpc_call iqsort");
      die(status);
    }
  }

  status = grpc_wait_all();

  if(status != GRPC_NO_ERROR) {
    grpc_perror("grpc_wait_all");
    die(status);
  }

  gettimeofday(&t2, 0);

  et = elapsed_time(&t1,&t2);

  for(i=0;i<tasks;i++) {
/**
    if(prof_enabled)
      printf("proxy_start=%g object_init=%g agent_comm=%g send_input=%g job_complete=%g recv_output=%g measure_comm=%g\n",
        profiles[i].proxy_start, 
        profiles[i].object_init, 
        profiles[i].agent_comm,
        profiles[i].send_input, 
        profiles[i].job_complete, 
        profiles[i].recv_output, 
        profiles[i].measure_comm);
**/

    grpc_function_handle_destruct(&handles[i]);
  }

  for(i=0;i<tasks;i++)
    for(j=0;j<n-1;j++)
      if(x[i*n + j] > x[i*n + j + 1]) {
        fprintf(stderr, "Bad results in list\n");
        die(EXIT_FAILURE);
      }
    
  printf("%d: %g\n", n, et);
/*  printf("GridRPC calls completed successfully.\n"); */
  grpc_finalize();
  die(EXIT_SUCCESS);
}

/**
 * Computes the elapsed wall time between the two given
 * times (timeval structs).
 *
 * @param t0 -- starting time
 * @param t1 -- ending time
 *
 * @returns the elapsed time between the two times
 **/

double
elapsed_time(const struct timeval* t0, const struct timeval* t1)
{
  double ret = t1->tv_sec - t0->tv_sec;
  double uret = t1->tv_usec - t0->tv_usec;
  return ret + uret / 1E6;
}
