/**
 * @file
 *
 * This program tests the probe, wait, and cancel features of the GridRPC API.
 */
/* $Id: control_example.c,v 1.11 2006/01/30 18:00:37 yarkhan Exp $ */
/* $UTK_Copyright: $ */

#include <stdio.h>
#include <stdlib.h>

#include "portability.h"
#include "grpc.h"
#include "totaltest.h"

/* Make wait time less than sleep time so that the job will
 * get cancelled.
 */
#define SLEEP_INTERVAL 1
#define SLEEP_AMT 10
#define SLEEP_LEN (SLEEP_AMT*SLEEP_INTERVAL)
#define WAIT_TIME (SLEEP_LEN/2)

int
main(int argc, char *argv[])
{
  grpc_function_handle_t handle;
  grpc_sessionid_t sessionID;
  grpc_error_t status;
  int len, c;

  len = SLEEP_LEN;

  if(grpc_initialize(NULL) != GRPC_NO_ERROR) {
    grpc_perror("grpc_initialize");
    die(EXIT_FAILURE);
  }

  if(grpc_function_handle_default(&handle, "sleeptest") != GRPC_NO_ERROR) {
    fprintf(stderr,"Error creating function handle\n");
    die(EXIT_FAILURE);
  }

  printf("Calling sleeptest...\n");

  status = grpc_call_async(&handle, &sessionID, len);

  if(status != GRPC_NO_ERROR) {
    grpc_perror("grpc_call");
    die(sessionID);
  }

  c = WAIT_TIME;
  
  for(;;) {
    printf("Probing...\n");
    status = grpc_probe(sessionID);

    if(status == GRPC_NO_ERROR) {
      printf("The GridRPC call completed.\n");
      if(grpc_wait(sessionID) != GRPC_NO_ERROR) {
        fprintf(stderr,"grpc_wait failed\n");
        die(EXIT_FAILURE);
      }
      break;
    }
    else if(status != GRPC_NOT_COMPLETED) {
      fprintf(stderr,"grpc_probe failed\n");
      die(EXIT_FAILURE);
    }

    sleep(SLEEP_INTERVAL);
    if(c-- <= 0) {
      printf("Tired of waiting.. going to cancel.\n");
      status = grpc_cancel(sessionID);

      if(status == GRPC_NO_ERROR)
        printf("Successfully cancelled.\n");
      else {
        fprintf(stderr, "Could not cancel.\n");
        die(EXIT_FAILURE);
      }
 
      break;
    }
  }

  printf("Submitting several requests (for testing grpc_cancel_all())\n");
  
  for(c=0;c<3;c++) {
    status = grpc_call_async(&handle, &sessionID, len);
    
    if(status != GRPC_NO_ERROR) {
      grpc_perror("grpc_call");
      die(sessionID);
    }
  }

  printf("requests submitted -- going to cancel all with grpc_cancel_all()\n");

  if(grpc_cancel_all() != GRPC_NO_ERROR) {
    fprintf(stderr,"Error: grpc_cancel_all() failed\n");
    die(EXIT_FAILURE);
  }

  if(grpc_function_handle_destruct(&handle) != GRPC_NO_ERROR) {
    fprintf(stderr,"Warning: failed to destroy function handle\n");
    die(EXIT_FAILURE);
  }

  printf("Finalizing...\n");

  grpc_finalize();

  printf("Done.\n");

  die(EXIT_SUCCESS);
}
