/**
 * @file
 *
 * This program makes a GridRPC call to calculate the dot
 * product of two vectors.
 *
 */
/* $Id: ddot_test.c,v 1.12 2005/12/14 18:39:17 seymour Exp $ */
/* $UTK_Copyright: $ */

#include <stdio.h>
#include <stdlib.h>

#include "grpc.h"
#include "totaltest.h"

int
main(int argc, char *argv[])
{
  int n, incx, incy, status;
  double correct_result = 13000.0, gs_result = 0.0;
  double dx[] = {10.0, 20.0, 30.0, 40.0, 50.0};
  double dy[] = {60.0, 70.0, 80.0, 90.0, 100.0};
  grpc_function_handle_t handle;
  char answer[100];

  n = 5;

  incx = incy = 1;

  if(grpc_initialize(NULL) != GRPC_NO_ERROR) {
    grpc_perror("grpc_initialize");
    die(EXIT_FAILURE);
  }

  if(grpc_function_handle_default(&handle, "ddot") != GRPC_NO_ERROR) {
    fprintf(stderr,"Error creating function handle\n");
    die(EXIT_FAILURE);
  }

  if(!strstr(argv[0], "async")) {
    status = grpc_call(&handle, n, dx, incx, dy, incy, &gs_result);

    if(status != GRPC_NO_ERROR) {
      printf("GRPC error status  = %d\n", status);
      grpc_perror("grpc_call");
      die(status);
    }
  }
  else {
    grpc_sessionid_t sessionID; 

    status = grpc_call_async(&handle, &sessionID, n, dx, incx, dy, incy, &gs_result);

    if(status != GRPC_NO_ERROR) {
      printf("GRPC error status  = %d\n", status);
      grpc_perror("grpc_call_async");
      die(EXIT_FAILURE);
    }

    printf("Waiting for result.\n");
  
    status = grpc_wait(sessionID);
  
    if(status != GRPC_NO_ERROR) {
      grpc_perror("grpc_wait");
      die(EXIT_FAILURE);
    }
  }

  if(GS_NE(gs_result, correct_result)) {
    printf("\nresult from GRPC = %g (correct result should be %g)\n", 
      gs_result, correct_result);
    die(EXIT_FAILURE);
  }
  else
    printf("\nresult is correct.\n");

  grpc_function_handle_destruct(&handle);

  printf("\n\nNow testing whether assigned-server feature works.\n");

  printf("\nEnter the hostname of a specific GridSolve server\n");
  printf("or hit enter to skip this test: ");

  if(fgets(answer,100,stdin) != NULL) {
    answer[strlen(answer)-1] = 0;

    if(strcmp(answer,"") != 0) {
      grpc_function_handle_init(&handle, answer, "ddot");

      printf("Ok, going to try to use server '%s'..\n", answer);

      if(!strstr(argv[0], "async")) {

        status = grpc_call(&handle, n, dx, incx, dy, incy, &gs_result);

        if(status != GRPC_NO_ERROR) {
          printf("GRPC error status  = %d\n", status);
          grpc_perror("grpc_call");
          die(status);
        }
      }
      else {
        grpc_sessionid_t sessionID; 

        status = grpc_call_async(&handle, &sessionID, n, dx, incx, dy, incy, 
          &gs_result);

        if(status != GRPC_NO_ERROR) {
          printf("GRPC error status  = %d\n", status);
          grpc_perror("grpc_call_async");
          die(EXIT_FAILURE);
        }

        printf("Waiting for result.\n");
  
        status = grpc_wait(sessionID);
  
        if(status != GRPC_NO_ERROR) {
          grpc_perror("grpc_wait");
          die(EXIT_FAILURE);
        }
      }

      if(GS_NE(gs_result, correct_result)) {
        printf("\nresult from GRPC = %g (correct result should be %g)\n",
          gs_result, correct_result);
        die(EXIT_FAILURE);
      }
      else
        printf("\nresult is correct.\n");
    }
  }

  printf("Finalizing...\n");

  grpc_finalize();

  printf("Done.\n");

  die(EXIT_SUCCESS);
}
