/**
 * @file
 *
 * This program makes a GridRPC call to the "dgeev" routine.
 */
/* $Id: dgeev_test.c,v 1.1 2006/02/02 17:40:46 yarkhan Exp $ */
/* $UTK_Copyright: $ */

#include <stdio.h>
#include <stdlib.h>

#include "grpc.h"
#include "totaltest.h"

#define MATSIZE 2

int
main(int argc, char *argv[])
{
  grpc_function_handle_t handle;
  int status, i, n, nrhs, lda, info,lwork;
  double *a, *wr, *wi, *vl, *vr, *work;
  char charn, charv;

  n = MATSIZE;
  nrhs = n;
  lda = n;
  lwork = 4*n;
  charn = 'N';
  charv = 'V';

  a = (double*)malloc(n*n*sizeof(*a));
  wr = (double*)malloc(n*sizeof(*a));
  wi = (double*)malloc(n*sizeof(*a));
  vl = (double*)malloc(n*n*sizeof(*a));
  vr = (double*)malloc(n*n*sizeof(*a));
  work = (double*)malloc(lwork*sizeof(*a));

  for(i=0;i<n*n;i++) {
    a[i] = (double)i;
  }

  if(grpc_initialize(NULL) != GRPC_NO_ERROR) {
    grpc_perror("grpc_initialize");
    die(EXIT_FAILURE);
  }

  if(grpc_function_handle_default(&handle, "dgeev") != GRPC_NO_ERROR) {
    fprintf(stderr,"Error creating function handle\n");
    die(EXIT_FAILURE);
  }

  printf("matrix A before call:\n");
  for(i=0;i<n*n;i++)
    printf("%g\n",a[i]);

  if(!strstr(argv[0], "async")) {
    /* status = grpc_call(&handle, n, nrhs, a, lda, ipiv, b, ldb, &info); */
    status = grpc_call(&handle, 'N', 'V', n, a, lda, wr, wi, vl, n, vr, n, lwork, &info);

    if(status != GRPC_NO_ERROR) {
      printf("GRPC error status  = %d\n", status);
      grpc_perror("grpc_call");
      die(status);
    }
  }
  else {
    grpc_sessionid_t sessionID; 

    /* status = grpc_call_async(&handle, &sessionID, n, nrhs, a, lda, ipiv, b, ldb, &info); */
    status = grpc_call(&handle, &sessionID, 'N', 'V', n, a, lda, wr, wi, vl, n, vr, n, lwork, &info);

    if(status != GRPC_NO_ERROR) {
      printf("GRPC error status  = %d\n", status);
      grpc_perror("grpc_call_async");
      die(EXIT_FAILURE);
    }

    printf("Waiting for result.\n");
  
    status = grpc_wait(sessionID);
  
    if(status != GRPC_NO_ERROR) {
      grpc_perror("grpc_wait");
      die(EXIT_FAILURE);
    }
  }

  printf("matrix A after call:\n");
  for(i=0;i<n*n;i++)
    printf("%g\n",a[i]);
  printf("info = %d\n", info);

  printf("Finalizing...\n");

  grpc_finalize();

  printf("Done.\n");

  die(EXIT_SUCCESS);
}
