/**
 * Test of dgemm (matrix multiply).
 */

#include <stdio.h>
#include <stdlib.h>

#include "grpc.h"
#include "totaltest.h"

int main(int argc, char *argv[])
{
  grpc_function_handle_t handle;
  grpc_error_t status;
  char transa, transb;
  int i, j, m, n, k, lda, ldb, ldc, saw_failure;
  double alpha, beta, *c;
  double a[] = {
    0.2028,   0.1988,   0.9318,   0.5252,
    0.1987,   0.0153,   0.4660,   0.2026,
    0.6038,   0.7468,   0.4186,   0.6721,
    0.2722,   0.4451,   0.8462,   0.8381
  };
  double b[] = {
   -3.5798,   6.2382,   0.7424,   0.1393,
    6.2193,  -7.5001,   2.0660,  -3.7404,
    4.0175,  -1.9960,   0.3524,  -2.3173,
   -6.1966,   3.9724,  -1.6941,   5.4740
  };

  saw_failure = 0;
  m = n = k = lda = ldb = ldc = 4;
  transa = transb = 'N';
  alpha = beta = 1.0;

  c = (double *)malloc(m*n*sizeof(*c));

  if(grpc_initialize(NULL) != GRPC_NO_ERROR) {
    grpc_perror("grpc_initialize");
    die(EXIT_FAILURE);
  }

  if(grpc_function_handle_default(&handle, "dgemm") != GRPC_NO_ERROR) {
    fprintf(stderr,"Error creating function handle\n");
    die(EXIT_FAILURE);
  }

  if(!strstr(argv[0], "async")) {
    status = grpc_call(&handle,transa,transb,m,n,k,alpha,a,lda,b,ldb,beta,c,ldc);

    if(status != GRPC_NO_ERROR) {
      printf("GRPC error status  = %d\n", status);
      grpc_perror("grpc_call");
      die(status);
    }
  }
  else {
    grpc_sessionid_t sessionID; 

    status = grpc_call_async(&handle, &sessionID, transa,transb,m,n,k,alpha,a,lda,b,ldb,beta,c,ldc);

    if(status != GRPC_NO_ERROR) {
      printf("GRPC error status  = %d\n", status);
      grpc_perror("grpc_call_async");
      die(EXIT_FAILURE);
    }

    printf("Waiting for result.\n");
  
    status = grpc_wait(sessionID);
  
    if(status != GRPC_NO_ERROR) {
      grpc_perror("grpc_wait");
      die(EXIT_FAILURE);
    }
  }

  for(i=0;i<m;i++) {
    for(j=0;j<n;j++) {
      if(i==j) {
        if(GS_NE(c[i*ldc+j], 1.0)) {
          printf("Error: bad value at %d,%d\n", i, j);
          saw_failure = 1;
        }
      }
      else {
        if(GS_NE(c[i*ldc+j], 0.0)) {
          printf("Error: bad value at %d,%d\n", i, j);
          saw_failure = 1;
        }
      }
    }
  }

  printf("Done.\n");
  grpc_finalize();
  die(saw_failure ? EXIT_FAILURE : EXIT_SUCCESS);
}
