/**
 * @file
 *
 * This program makes a GridRPC call to the "dgesv" routine.
 */
/* $Id: dgesv_test.c,v 1.9 2005/12/14 18:39:17 seymour Exp $ */
/* $UTK_Copyright: $ */

#include <stdio.h>
#include <stdlib.h>

#include "grpc.h"
#include "totaltest.h"

#define MATSIZE 2

int
main(int argc, char *argv[])
{
  grpc_function_handle_t handle;
  int status, i, n, nrhs, lda, *ipiv, ldb, info;
  double *a, *b;

  n = MATSIZE;
  nrhs = n;
  lda = n;
  ldb = n;

  a = (double*)malloc(n*n*sizeof(*a));
  b = (double*)malloc(n*n*sizeof(*a));
  ipiv = (int*)malloc(n*sizeof(*ipiv));

  for(i=0;i<n*n;i++) {
    a[i] = (double)i;
    b[i] = (double)(n+i);
  }

  for(i=0;i<n;i++)
    ipiv[i] = n-i;
  
  if(grpc_initialize(NULL) != GRPC_NO_ERROR) {
    grpc_perror("grpc_initialize");
    die(EXIT_FAILURE);
  }

  if(grpc_function_handle_default(&handle, "dgesv") != GRPC_NO_ERROR) {
    fprintf(stderr,"Error creating function handle\n");
    die(EXIT_FAILURE);
  }

  printf("matrix A before call:\n");
  for(i=0;i<n*n;i++)
    printf("%g\n",a[i]);

  if(!strstr(argv[0], "async")) {
    status = grpc_call(&handle, n, nrhs, a, lda, ipiv, b, ldb, &info);

    if(status != GRPC_NO_ERROR) {
      printf("GRPC error status  = %d\n", status);
      grpc_perror("grpc_call");
      die(status);
    }
  }
  else {
    grpc_sessionid_t sessionID; 

    status = grpc_call_async(&handle, &sessionID, n, nrhs, a, lda, ipiv, b, ldb, &info);

    if(status != GRPC_NO_ERROR) {
      printf("GRPC error status  = %d\n", status);
      grpc_perror("grpc_call_async");
      die(EXIT_FAILURE);
    }

    printf("Waiting for result.\n");
  
    status = grpc_wait(sessionID);
  
    if(status != GRPC_NO_ERROR) {
      grpc_perror("grpc_wait");
      die(EXIT_FAILURE);
    }
  }

  printf("matrix A after call:\n");
  for(i=0;i<n*n;i++)
    printf("%g\n",a[i]);
  printf("info = %d\n", info);

  printf("Finalizing...\n");

  grpc_finalize();

  printf("Done.\n");

  die(EXIT_SUCCESS);
}
