/**
 * @file
 *
 * This program tests the assigned server feature of GridRPC by calling
 * the "dgesv" procedure on the server specified on the command line.
 * The program is invoked as follows:
 *   dgesv_test_assign <server>
 *
 * The server should be specified in the following format:
 *    serverhost/serverport%proxyhost/proxyport
 * where 'serverhost' is required, but the rest is optional.
 */
/* $Id: dgesv_test_assign.c,v 1.6 2005/05/15 00:08:14 seymour Exp $ */
/* $UTK_Copyright: $ */

#include <stdio.h>
#include <stdlib.h>

#include "grpc.h"
#include "totaltest.h"

#define MATSIZE 2

int
main(int argc, char *argv[])
{
  grpc_function_handle_t handle;
  int i, status, n, nrhs, lda, *ipiv, ldb, info;
  double *a, *b;

  if(argc != 2) {
    fprintf(stderr,"Usage: %s <str>\n", argv[0]);
    die(EXIT_FAILURE);
  }

  n = MATSIZE;
  nrhs = n;
  lda = n;
  ldb = n;

  a = (double*)calloc(n*n,sizeof(*a));
  b = (double*)calloc(n*n,sizeof(*a));
  ipiv = (int*)calloc(n,sizeof(*ipiv));

  for(i=0;i<n*n;i++) {
    a[i] = (double)i;
    b[i] = (double)(n+i);
  }

  for(i=0;i<n;i++)
    ipiv[i] = n-i;

  if(grpc_initialize(NULL) != GRPC_NO_ERROR) {
    grpc_perror("grpc_initialize");
    die(EXIT_FAILURE);
  }

  if(grpc_function_handle_init(&handle, argv[1], "dgesv") != GRPC_NO_ERROR) {
    fprintf(stderr,"Error creating function handle\n");
    die(EXIT_FAILURE);
  }

  printf("matrix A before call:\n");
  for(i=0;i<n*n;i++)
    printf("%g\n",a[i]);

  status = grpc_call(&handle, n, nrhs, a, lda, ipiv, b, ldb, &info);

  if(status != GRPC_NO_ERROR) {
    printf("GRPC error status  = %d\n", status);
    grpc_perror("grpc_call");
    die(status);
  }

  printf("matrix A after call:\n");
  for(i=0;i<n*n;i++)
    printf("%g\n",a[i]);
  printf("info = %d\n", info);

  printf("Finalizing...\n");

  grpc_finalize();

  printf("Done.\n");

  die(EXIT_SUCCESS);
}
