/**
 * @file
 *
 * Tests IBP interface.
 *
 */
/* $Id: ibp_test.c,v 1.2 2005/12/02 03:48:57 seymour Exp $ */
/* $UTK_Copyright: $ */

#include <stdio.h>
#include <stdlib.h>

#include "grpc.h"
#include "totaltest.h"

int
main(int argc, char *argv[])
{
  int int_vec_in[] = {93, 120, 84, 57, 147, 138, 66, 12, 88, 2};
  int *int_vec_out, i, n;
  grpc_function_handle_t handle;
  grpc_error_t status;
  DSI_OBJECT *int_vec;
  DSI_FILE *dsi_file;
  char *host;

  if(argc == 2)
    host = argv[1];
  else
    host = "localhost";

  n = sizeof(int_vec_in) / sizeof(*int_vec_in);

  int_vec_out = (int *)malloc(n * sizeof(int));

  if(!int_vec_out) {
    fprintf(stderr, "malloc failed\n");
    die(EXIT_FAILURE);
  }

  if(grpc_initialize(NULL) != GRPC_NO_ERROR) {
    grpc_perror("grpc_initialize");
    die(EXIT_FAILURE);
  }

  if(grpc_dsi_open(&dsi_file, host, O_CREAT|O_RDWR, 644, 30000, GS_DSI_IBP)
       != GRPC_NO_ERROR) 
  {
    fprintf(stderr, "Error opening DSI file.\n");
    die(EXIT_FAILURE);
  }

  if(grpc_dsi_write_vector(&int_vec, dsi_file, int_vec_in, n, GS_INT)
       != GRPC_NO_ERROR)
  {
    fprintf(stderr, "Error writing in_vec to DSI file.\n");
    die(EXIT_FAILURE);
  }

  if(grpc_function_handle_default(&handle, "int_vector_add5") != GRPC_NO_ERROR) {
    fprintf(stderr,"Error creating function handle\n");
    die(EXIT_FAILURE);
  }

  if(!strstr(argv[0], "async")) {
    status = grpc_call(&handle, n, int_vec, int_vec_out);

    if(status != GRPC_NO_ERROR) {
      printf("GRPC error status  = %d\n", status);
      grpc_perror("grpc_call");
      die(status);
    }
  }
  else {
    grpc_sessionid_t sessionID; 

    status = grpc_call_async(&handle, &sessionID, n, int_vec, int_vec_out);

    if(status != GRPC_NO_ERROR) {
      printf("GRPC error status  = %d\n", status);
      grpc_perror("grpc_call_async");
      die(EXIT_FAILURE);
    }

    printf("Waiting for result.\n");
  
    status = grpc_wait(sessionID);
  
    if(status != GRPC_NO_ERROR) {
      grpc_perror("grpc_wait");
      die(EXIT_FAILURE);
    }
  }

  for(i=0; i < n; i++) {
    if(int_vec_in[i] != int_vec_out[i] - 5) {
      fprintf(stderr, "Bad results in integer list\n");
      die(EXIT_FAILURE);
    }
  }

  grpc_dsi_close(dsi_file);
    
  printf("GridRPC calls completed successfully.\n");
  grpc_finalize();
  die(EXIT_SUCCESS);
}
