/**
 * @file
 *
 * This program tests the GridRPC probe feature, which is used to
 * determine whether a previously submitted job has completed.
 *
 */
/* $Id: probe_test.c,v 1.9 2006/01/30 18:00:05 yarkhan Exp $ */
/* $UTK_Copyright: $ */

#include <stdio.h>
#include <stdlib.h>

#include "portability.h"	/* for <unistd.h> */
#include "grpc.h"
#include "totaltest.h"

#define NUM_REQ 5

int probe_or_test(grpc_function_handle_t *);

int
main(int argc, char *argv[])
{
  grpc_function_handle_t handle;
  grpc_sessionid_t sessionID;
  grpc_error_t status;

  if(grpc_initialize(NULL) != GRPC_NO_ERROR) {
    grpc_perror("grpc_initialize");
    die(EXIT_FAILURE);
  }

  if(grpc_function_handle_default(&handle, "sleeptest") != GRPC_NO_ERROR) {
    fprintf(stderr,"Error creating function handle\n");
    die(EXIT_FAILURE);
  }

  status = grpc_call_async(&handle, &sessionID, 10);

  if(status != GRPC_NO_ERROR) {
    grpc_perror("grpc_call");
    die(sessionID);
  }

  printf("Probing for result.\n");
  
  while(1) {
    status = grpc_probe(sessionID);
 
    if(status == GRPC_NO_ERROR) {
      printf("Job complete.\n");
      break;
    }
    else if(status != GRPC_NOT_COMPLETED) {
      fprintf(stderr,"grpc_probe failed\n");
      die(EXIT_FAILURE);
    }

    printf("Not finished, sleeping....\n");
    sleep(1);
  }

  if(status == GRPC_NO_ERROR) {
    status = grpc_wait(sessionID);
 
    if(status != GRPC_NO_ERROR) {
      fprintf(stderr,"grpc_wait failed\n");
      die(EXIT_FAILURE);
    }
  }

  if(probe_or_test(&handle) < 0) {
    fprintf(stderr,"Test of grpc_probe_or() failed\n");
    die(EXIT_FAILURE);
  }

  grpc_function_handle_destruct(&handle);

  printf("Finalizing...\n");

  grpc_finalize();

  printf("Done.\n");

  die(EXIT_SUCCESS);
}


int
probe_or_test(grpc_function_handle_t *handle)
{
  int ss1 = NUM_REQ/2, ss2 = NUM_REQ - (NUM_REQ/2), saw_failure;
  grpc_sessionid_t sessionID[NUM_REQ];
  int i, len[] = {15,8,5,10,20};
  grpc_error_t status;

  saw_failure = 0;

  for(i=0;i<NUM_REQ;i++) {
    status = grpc_call_async(handle, &(sessionID[i]), len[i]);
    if(status != GRPC_NO_ERROR) {
      grpc_perror("grpc_call");
      die(sessionID[i]);
    }

    printf("sleeptest session ID = %d\n", sessionID[i]);
  }

  for(i=0;i<ss1;i++) {
    int id;

    printf("Probing for anything from the first set of calls.\n");

    for(;;) {
      status = grpc_probe_or(sessionID, ss1, &id);

      if(status == GRPC_NO_ERROR) {
        printf("%d completed\n", id);
        break;
      }
      else if (status != GRPC_NONE_COMPLETED) {
        printf("Warning: one or more calls in the first set failed.\n");
        saw_failure = 1;
        break;
      }

      sleep(1);
    }
    
    if(saw_failure) break;

    if(grpc_wait(id) != GRPC_NO_ERROR) {
      printf("Warning: failed to wait for request %d.\n", id);
    }
  }

  for(i=0;i<ss2;i++) {
    int id;

    printf("Probing for anything from the second set of calls.\n");

    for(;;) {
      status = grpc_probe_or(sessionID+ss1, ss2, &id);

      if(status == GRPC_NO_ERROR) {
        printf("%d completed\n", id);
        break;
      }
      else if (status != GRPC_NONE_COMPLETED) {
        printf("Warning: one or more calls in the first set failed.\n");
        saw_failure = 1;
        break;
      }

      sleep(1);
    }

    if(saw_failure) break;

    if(grpc_wait(id) != GRPC_NO_ERROR) {
      printf("Warning: failed to wait for request %d.\n", id);
    }
  }

  return saw_failure ? -1 : 0;
}
