/**
 * @file
 *
 * This program tests retrieval of the results of a previously submitted
 * request that was serialized to disk.  Therefore, you should first run
 * submit_test to submit a request and serialize it.
 *
 * This test is run as follows:
 *   submit_test foo.xml
 *   retrieve_test foo.xml
 *
 * Of course, the same filename specified to submit_test should be used for
 * retrieve_test.
 */
/* $Id: retrieve_test.c,v 1.10 2006/02/08 21:53:27 seymour Exp $ */
/* $UTK_Copyright: $ */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "portability.h"	/* for <unistd.h> */
#include "grpc.h"
#include "totaltest.h"

int
main(int argc, char *argv[])
{
  struct stat stbuf;
  int fd, status, filelen, n;
  char *test_name, *request_str;
#include "totaltest_decl.c"

  test_name = "ctotal";

  if(argc != 2) {
    fprintf(stderr,"Usage: %s <request file>\n", argv[0]);
    die(EXIT_FAILURE);
  }

  if(stat(argv[1], &stbuf) < 0) {
    fprintf(stderr,"Could not stat request file '%s'\n", argv[1]);
    die(EXIT_FAILURE);
  }

  fd = open(argv[1], O_RDONLY);
  if(fd < 0) {
    fprintf(stderr,"Could not open request file '%s'\n", argv[1]);
    die(EXIT_FAILURE);
  }

  filelen = (int)stbuf.st_size;
  request_str = (char *)CALLOC(filelen+1, sizeof(char));
  if((n=read(fd, request_str, filelen)) != filelen) {
    fprintf(stderr,"Error reading request file '%s'\n", argv[1]);
    fprintf(stderr,"n = %d, filelen = %d\n", n, filelen);
    close(fd);
    perror("read");
    die(EXIT_FAILURE);
  }
  close(fd);
  request_str[filelen] = '\0';
  
#include "totaltest_init.c"

  if(grpc_initialize(NULL) != GRPC_NO_ERROR) {
    grpc_perror("grpc_initialize");
    die(EXIT_FAILURE);
  }

  status = grpc_retrieve(request_str, TOTALTEST_ARGLIST);

  if(status != GRPC_NO_ERROR) {
    grpc_perror("grpc_wait");
    die(EXIT_FAILURE);
  }

#include "totaltest_results.c"

  free(request_str);
  grpc_arg_stack_destruct(stack);

  printf("Finalizing...\n");

  grpc_finalize();

  printf("Done.\n");

  die(EXIT_SUCCESS);
}
