/**
 * @file
 *
 * This is a simple test case for debugging whether the
 * output to stdout/stderr on the service is working
 * correctly.
 *
 */
/* $Id: service_output_test.c,v 1.1 2007/10/23 17:37:27 seymour Exp $ */
/* $UTK_Copyright: $ */

#include <stdio.h>
#include <stdlib.h>

#include "portability.h"	/* for <unistd.h> */
#include "grpc.h"
#include "totaltest.h"

#define WAIT_TIME 10

int
main(int argc, char *argv[])
{
  grpc_function_handle_t handle;
  grpc_sessionid_t sessionID;
  grpc_error_t status;

  if(grpc_initialize(NULL) != GRPC_NO_ERROR) {
    grpc_perror("grpc_initialize");
    die(EXIT_FAILURE);
  }

  if(grpc_function_handle_default(&handle, "sleeptest") != GRPC_NO_ERROR) {
    fprintf(stderr,"Error creating function handle\n");
    die(EXIT_FAILURE);
  }

  status = grpc_call_async(&handle, &sessionID, WAIT_TIME);

  if(status != GRPC_NO_ERROR) {
    grpc_perror("grpc_call");
    die(sessionID);
  }

  printf("Waiting for result.\n");
  
  status = grpc_wait(sessionID);
 
  if(status != GRPC_NO_ERROR) {
    fprintf(stderr,"grpc_wait failed\n");
    die(EXIT_FAILURE);
  }

  printf("Non-blocking Success, now going to do blocking.\n\n");

  status = grpc_call(&handle, WAIT_TIME);
  if(status != GRPC_NO_ERROR) {
    grpc_perror("grpc_call");
    die(status);
  }

  printf("Blocking Success.\n\n");

  grpc_function_handle_destruct(&handle);

  printf("Finalizing...\n");

  grpc_finalize();

  printf("Done.\n");

  die(EXIT_SUCCESS);
}
