/**
 * @file
 *
 * Simple test of DSI interface.
 *
 */
/* $Id: simple_dsi_test.c,v 1.4 2006/01/30 17:58:43 yarkhan Exp $ */
/* $UTK_Copyright: $ */

#include <stdio.h>
#include <stdlib.h>

#include "portability.h"
#include "grpc.h"
#include "totaltest.h"

int
main(int argc, char *argv[])
{
  double *in_vec, *in_vec1, *out_vec;
  DSI_OBJECT *dvec, *dvec1;
  DSI_FILE *dsi_file;
  int i, count;
  char *host;

  if(argc == 2)
    host = argv[1];
  else
    host = "localhost";

  if(grpc_initialize(NULL) != GRPC_NO_ERROR) {
    grpc_perror("grpc_initialize");
    die(EXIT_FAILURE);
  }

  if(grpc_dsi_open(&dsi_file, host, O_CREAT|O_RDWR, 644, 30000, GS_DSI_IBP)
       != GRPC_NO_ERROR) 
  {
    fprintf(stderr, "Error opening DSI file.\n");
    die(EXIT_FAILURE);
  }

  in_vec = (double*)malloc(sizeof(double)*10);
  in_vec1 = (double*)malloc(sizeof(double)*10);
  out_vec = (double*)malloc(sizeof(double)*10);

  for(i=0; i<10; i++){
    in_vec[i] = (double)i;
    in_vec1[i] = (double)(2*i);
    out_vec[i] = 0.0;
  }

  if(grpc_dsi_write_vector(&dvec, dsi_file, in_vec, 10, GS_DOUBLE)
       != GRPC_NO_ERROR)
  {
    fprintf(stderr, "Error writing in_vec to DSI file.\n");
    die(EXIT_FAILURE);
  }

  if(grpc_dsi_write_vector(&dvec1, dsi_file, in_vec1, 10, GS_DOUBLE)
       != GRPC_NO_ERROR)
  {
    fprintf(stderr, "Error writing in_vec1 to DSI file.\n");
    die(EXIT_FAILURE);
  }

  if(grpc_dsi_read_vector(dvec1, out_vec, 10, GS_DOUBLE, &count) 
      != GRPC_NO_ERROR)
  {
    fprintf(stderr, "Error reading dvec1 from DSI file.\n");
    die(EXIT_FAILURE);
  }
  else{
    printf("count: %d\n", count);
    for(i=0; i<10; i++)
      printf("%f ", out_vec[i]);
    printf("\n");
  }

  if(grpc_dsi_read_vector(dvec, out_vec, 10, GS_DOUBLE, &count)
      != GRPC_NO_ERROR)
  {
    fprintf(stderr, "Error reading dvec from DSI file.\n");
    die(EXIT_FAILURE);
  }
  else{
    printf("count: %d\n", count);
    for(i=0; i<10; i++)
      printf("%f ", out_vec[i]);
    printf("\n");
  }

  grpc_dsi_close(dsi_file);

  printf("Finalizing...\n");

  grpc_finalize();

  printf("Done.\n");

  die(EXIT_SUCCESS);
}
