/**
 * @file
 *
 * Tests integer and double qsort routines.
 *
 */
/* $Id: sort_test.c,v 1.4 2008/03/24 19:53:11 seymour Exp $ */
/* $UTK_Copyright: $ */

#include <stdio.h>
#include <stdlib.h>

#include "grpc.h"
#include "totaltest.h"

int do_batch_async_testing(char *, int, int);
int do_batch_cancel_test(char *, int);
double elapsed_time(const struct timeval* t0, const struct timeval* t1);

int main(int argc, char *argv[])
{
  double y[] = {58.0, 73.0, 13.0, 85.0, 40.0, 94.0, 32.0, 8.0, 58.0, 33.0};
  int x[] = {93, 120, 84, 57, 147, 138, 66, 12, 88, 2};
  grpc_function_handle_t handle;
  int i, n, batch_mode, delay;
  struct timeval i_start, i_end, d_start, d_end;
  char *ifname, *dfname;
  grpc_error_t status;

  if(strstr(argv[0], "batch")) {
    if(argc >= 2)
      delay = atoi(argv[1]);
    else
      delay = 5;

    batch_mode = 1;
    ifname = "batch_test_int";
    dfname = "batch_test_double";
  }
  else {
    batch_mode = 0;
    delay = 0;
    ifname = "iqsort";
    dfname = "dqsort";
  }
  
  n = sizeof(x) / sizeof(*x);

  if(grpc_initialize(NULL) != GRPC_NO_ERROR) {
    grpc_perror("grpc_initialize");
    die(EXIT_FAILURE);
  }

  if(grpc_function_handle_default(&handle, ifname) != GRPC_NO_ERROR) {
    fprintf(stderr,"Error creating function handle\n");
    die(EXIT_FAILURE);
  }

  gettimeofday(&i_start, NULL);
  if(batch_mode)
    status = grpc_call(&handle, x, n, delay);
  else
    status = grpc_call(&handle, x, n);
  gettimeofday(&i_end, NULL);

  if(status != GRPC_NO_ERROR) {
    printf("GRPC error status  = %d\n", status);
    grpc_perror("grpc_call iqsort");
    die(status);
  }

  grpc_function_handle_destruct(&handle);

  if(grpc_function_handle_default(&handle, dfname) != GRPC_NO_ERROR) {
    fprintf(stderr,"Error creating function handle\n");
    die(EXIT_FAILURE);
  }

  gettimeofday(&d_start, NULL);
  if(batch_mode)
    status = grpc_call(&handle, y, n, delay);
  else
    status = grpc_call(&handle, y, n);
  gettimeofday(&d_end, NULL);

  if(status != GRPC_NO_ERROR) {
    printf("GRPC error status  = %d\n", status);
    grpc_perror("grpc_call dqsort");
    die(status);
  }

  grpc_function_handle_destruct(&handle);

  for(i=0; i < n-1; i++) {
    if(x[i] > x[i+1]) {
      fprintf(stderr, "Bad results in integer list\n");
      die(EXIT_FAILURE);
    }

    if(y[i] > y[i+1]) {
      fprintf(stderr, "Bad results in double precision list\n");
      die(EXIT_FAILURE);
    }
  }
    
  if(batch_mode) {
    printf("results from blocking calls correct.\n");
    printf("\ntesting async/probe now.\n");

    if(do_batch_async_testing(ifname, delay, 1) < 0)
      die(EXIT_FAILURE);

    printf("\ntesting async/wait now.\n");
    if(do_batch_async_testing(ifname, delay, 0) < 0)
      die(EXIT_FAILURE);

    printf("\ntesting async/cancel now.\n");
    if(do_batch_cancel_test(ifname, delay) < 0)
      die(EXIT_FAILURE);
  }

  printf("GridRPC calls completed successfully (int rtt = %lf, double rtt = %lf).\n",
    elapsed_time(&i_start, &i_end), elapsed_time(&d_start, &d_end));
  grpc_finalize();
  die(EXIT_SUCCESS);
}

int
do_batch_cancel_test(char *ifname, int delay)
{
  int x[] = {93, 120, 84, 57, 147, 138, 66, 12, 88, 2};
  grpc_function_handle_t handle;
  grpc_sessionid_t sessionID;
  grpc_error_t status;
  int n;

  n = sizeof(x) / sizeof(*x);

  if(grpc_function_handle_default(&handle, ifname) != GRPC_NO_ERROR) {
    fprintf(stderr,"Error creating function handle\n");
    return -1;
  }

  status = grpc_call_async(&handle, &sessionID, x, n, delay);

  if(status != GRPC_NO_ERROR) {
    grpc_perror("grpc_call");
    return -1;
  }

  sleep(delay/2);

  printf("Going to cancel.\n");
  status = grpc_cancel(sessionID);

  if(status == GRPC_NO_ERROR)
    printf("Successfully cancelled.\n");
  else {
    fprintf(stderr, "Could not cancel.\n");
    return -1;
  }

  grpc_function_handle_destruct(&handle);

  return 0;
}

int
do_batch_async_testing(char *ifname, int delay, int probe)
{
  int x[] = {93, 120, 84, 57, 147, 138, 66, 12, 88, 2};
  grpc_function_handle_t handle;
  grpc_sessionid_t sessionID;
  grpc_error_t status;
  int i, n;

  n = sizeof(x) / sizeof(*x);

  if(grpc_function_handle_default(&handle, ifname) != GRPC_NO_ERROR) {
    fprintf(stderr,"Error creating function handle\n");
    return -1;
  }

  status = grpc_call_async(&handle, &sessionID, x, n, delay);

  if(status != GRPC_NO_ERROR) {
    grpc_perror("grpc_call");
    return -1;
  }

  if(probe) {
    printf("Probing for result.\n");
  
    while(1) {
      status = grpc_probe(sessionID);
 
      if(status == GRPC_NO_ERROR) {
        printf("Job complete.\n");
        break;
      }
      else if(status != GRPC_NOT_COMPLETED) {
        fprintf(stderr,"grpc_probe failed\n");
        return -1;
      }

      printf("Not finished, sleeping....\n");
      sleep(1);
    }
  }

  if(status == GRPC_NO_ERROR) {
    status = grpc_wait(sessionID);
 
    if(status != GRPC_NO_ERROR) {
      fprintf(stderr,"grpc_wait failed\n");
      return -1;
    }
  }

  for(i=0; i < n-1; i++) {
    if(x[i] > x[i+1]) {
      fprintf(stderr, "Bad results in integer list\n");
      return -1;
    }
  }

  grpc_function_handle_destruct(&handle);

  return 0;
}

/**
 * Computes the elapsed wall time between the two given 
 * times (timeval structs).
 *
 * @param t0 -- starting time
 * @param t1 -- ending time
 *
 * @returns the elapsed time between the two times
 **/

double
elapsed_time(const struct timeval* t0, const struct timeval* t1)
{
  double ret = t1->tv_sec - t0->tv_sec;
  double uret = t1->tv_usec - t0->tv_usec;
  return ret + uret / 1E6;
}
