/**
 * @file
 *
 * This program tests serialization of a previously submitted non-blocking
 * request.  Serializing the request allows terminating the program and
 * retrieving the results later.
 *
 * This test is run as follows:
 *   submit_test foo.xml
 *   retrieve_test foo.xml
 *
 * Of course, the same filename specified to submit_test should be used for
 * retrieve_test.
 *
 */
/* $Id: submit_test.c,v 1.7 2005/05/15 00:08:16 seymour Exp $ */
/* $UTK_Copyright: $ */

#include <stdio.h>
#include <stdlib.h>

#include "grpc.h"
#include "totaltest.h"

int
main(int argc, char *argv[])
{
  grpc_function_handle_t handle;
  grpc_sessionid_t sessionID;
  char *test_name, *foo;
  grpc_error_t status;
#include "totaltest_decl.c"

  FILE *outfile;

  test_name = "ctotal";

  if(argc != 2) {
    fprintf(stderr,"Usage: %s <output filename>\n", argv[0]);
    die(EXIT_FAILURE);
  }

  outfile = fopen(argv[1], "w");
  if(!outfile) {
    fprintf(stderr,"Could not open output file '%s'\n", argv[1]);
    die(EXIT_FAILURE);
  }

#include "totaltest_init.c"

  if(grpc_initialize(NULL) != GRPC_NO_ERROR) {
    grpc_perror("grpc_initialize");
    die(EXIT_FAILURE);
  }

  if(grpc_function_handle_default(&handle, test_name) != GRPC_NO_ERROR) {
    fprintf(stderr,"Error creating function handle\n");
    die(EXIT_FAILURE);
  }

  status = grpc_call_async(&handle, &sessionID, TOTALTEST_ARGLIST);

  if(status != GRPC_NO_ERROR) {
    grpc_perror("grpc_call_async");
    die(EXIT_FAILURE);
  }

  if(grpc_serialize_request(sessionID, &foo) != GRPC_NO_ERROR) {
    fprintf(stderr,"Error serializing the request.\n");
    die(EXIT_FAILURE);
  }

  fprintf(outfile, "%s",foo);

  if(foo) free(foo);
  fclose(outfile);

  printf("Finalizing...\n");

  grpc_finalize();

  printf("Done.\n");

  die(EXIT_SUCCESS);
}
