/**
 * @file
 *
 * This program makes a blocking GridRPC call to the "varlen_return" 
 * routine to test whether we can receive a variable-length output arg.
 */
/* $Id: varlen_test.c,v 1.6 2007/03/04 22:16:51 seymour Exp $ */
/* $UTK_Copyright: $ */

#include <stdio.h>
#include <stdlib.h>

#include "grpc.h"
#include "totaltest.h"

int
main(int argc, char *argv[])
{
  grpc_function_handle_t handle;
  int status, slen, ilen, flen, dlen, sclen, dclen;
  char *str;
  int *ivec;
  float *fvec;
  double *dvec;
  gs_scomplex *scvec;
  gs_dcomplex *dcvec;

  slen = ilen = flen = dlen = sclen = dclen = 0;
  
  if(grpc_initialize(NULL) != GRPC_NO_ERROR) {
    grpc_perror("grpc_initialize");
    die(EXIT_FAILURE);
  }

  if(grpc_function_handle_default(&handle, "varlen_return") != GRPC_NO_ERROR) {
    fprintf(stderr,"Error creating function handle\n");
    die(EXIT_FAILURE);
  }

  if(!strstr(argv[0], "async")) {
    status = grpc_call(&handle, &slen, &str, &ilen, &ivec, &flen, &fvec,
       &dlen, &dvec, &sclen, &scvec, &dclen, &dcvec);

    if(status != GRPC_NO_ERROR) {
      printf("GRPC error status  = %d\n", status);
      grpc_perror("grpc_call");
      die(status);
    }
  }
  else {
    grpc_sessionid_t sessionID; 

    status = grpc_call_async(&handle, &sessionID, &slen, &str, &ilen, &ivec,
       &flen, &fvec, &dlen, &dvec, &sclen, &scvec, &dclen, &dcvec);

    if(status != GRPC_NO_ERROR) {
      printf("GRPC error status  = %d\n", status);
      grpc_perror("grpc_call_async");
      die(EXIT_FAILURE);
    }

    printf("Waiting for result.\n");
  
    status = grpc_wait(sessionID);
  
    if(status != GRPC_NO_ERROR) {
      grpc_perror("grpc_wait");
      die(EXIT_FAILURE);
    }
  }

  printf("string result = '%s'\n", str);
  printf("returned length = %d, actual length = %ld\n\n", slen, 
    (long)strlen(str)+1);

  printf("lengths: %d,%d,%d,%d,%d\n\n", ilen, flen, dlen, sclen, dclen);
  print_ivec("ivec:", ivec, ilen);
  print_fvec("fvec:", fvec, flen);
  print_dvec("dvec:", dvec, dlen);
  print_scmp_vec("scvec:", scvec, sclen);
  print_dcmp_vec("dcvec:", dcvec, dclen);

  printf("Finalizing...\n");

  grpc_finalize();

  printf("Done.\n");

  die(EXIT_SUCCESS);
}
