/**
 * @file
 *
 * This program tests the grpc_wait_all() function which blocks until
 * all previously submitted requests have completed.
 *
 */
/* $Id: wait_all_example.c,v 1.8 2005/09/30 06:49:33 seymour Exp $ */
/* $UTK_Copyright: $ */

#include <stdio.h>
#include <stdlib.h>

#include "grpc.h"
#include "totaltest.h"

#define NUM_REQ 5

int
main(int argc, char *argv[])
{
  grpc_sessionid_t sessionID[NUM_REQ];
  grpc_function_handle_t handle;
  int i, len[] = {15,8,5,10,20};
  grpc_error_t status;

  if(grpc_initialize(NULL) != GRPC_NO_ERROR) {
    grpc_perror("grpc_initialize");
    die(EXIT_FAILURE);
  }

  if(grpc_function_handle_default(&handle, "sleeptest") != GRPC_NO_ERROR) {
    fprintf(stderr,"Error creating function handle\n");
    die(EXIT_FAILURE);
  }

  for(i=0;i<NUM_REQ;i++) {
    status = grpc_call_async(&handle, &(sessionID[i]), len[i]);
    if(status != GRPC_NO_ERROR) {
      grpc_perror("grpc_call");
      die(sessionID[i]);
    }

    printf("sleeptest session ID = %d\n", sessionID[i]);
  }

  printf("Waiting for all calls to complete\n");
  status = grpc_wait_all();

  if(status != GRPC_NO_ERROR) {
    printf("Warning: one or more calls failed.\n");
    die(EXIT_FAILURE);
  }
    
  grpc_function_handle_destruct(&handle);

  printf("Finalizing...\n");

  grpc_finalize();

  printf("Done.\n");

  die(EXIT_SUCCESS);
}
