/**
 * @file
 *
 * This program tests the grpc_wait_and() function which blocks until
 * all of a subset of previously submitted requests have completed.
 *
 */
/* $Id: wait_and_example.c,v 1.8 2005/09/30 06:49:33 seymour Exp $ */
/* $UTK_Copyright: $ */

#include <stdio.h>
#include <stdlib.h>

#include "grpc.h"
#include "totaltest.h"

#define NUM_REQ 5

int
main(int argc, char *argv[])
{
  int ss1 = NUM_REQ/2, ss2 = NUM_REQ - (NUM_REQ/2);
  grpc_sessionid_t sessionID[NUM_REQ];
  grpc_function_handle_t handle;
  int i, len[] = {15,8,5,10,20};
  grpc_error_t status;
  int saw_failure = 0;

  if(grpc_initialize(NULL) != GRPC_NO_ERROR) {
    grpc_perror("grpc_initialize");
    die(EXIT_FAILURE);
  }

  if(grpc_function_handle_default(&handle, "sleeptest") != GRPC_NO_ERROR) {
    fprintf(stderr,"Error creating function handle\n");
    die(EXIT_FAILURE);
  }

  for(i=0;i<NUM_REQ;i++) {
    status = grpc_call_async(&handle, &(sessionID[i]), len[i]);
    if(status != GRPC_NO_ERROR) {
      grpc_perror("grpc_call");
      die(sessionID[i]);
    }

    printf("sleeptest session ID = %d\n", sessionID[i]);
  }

  printf("Waiting for first set of calls\n");
  status = grpc_wait_and(sessionID,ss1);

  if(status != GRPC_NO_ERROR) {
    for(i=0;i<ss1;i++)
      if(grpc_get_error(sessionID[i]) != GRPC_NO_ERROR) {
        printf("Warning: call %d failed.\n", i);
        saw_failure = 1;
      }
  }

  printf("Waiting for second set of calls\n");
  status = grpc_wait_and(sessionID+ss1,ss2);

  if(status != GRPC_NO_ERROR) {
    for(i=ss1;i<NUM_REQ;i++)
      if(grpc_get_error(sessionID[i]) != GRPC_NO_ERROR) {
        printf("Warning: call %d failed.\n", i);
        saw_failure = 1;
      }
  }
 
  grpc_function_handle_destruct(&handle);

  printf("Finalizing...\n");

  grpc_finalize();

  printf("Done.\n");

  die(saw_failure ? EXIT_FAILURE : EXIT_SUCCESS);
}
