/**
 * @file
 *
 * This program tests the grpc_wait_or() function which blocks until
 * any one of a subset of previously submitted requests has completed.
 *
 */
/* $Id: wait_or_example.c,v 1.8 2005/09/30 06:49:34 seymour Exp $ */
/* $UTK_Copyright: $ */

#include <stdio.h>
#include <stdlib.h>

#include "grpc.h"
#include "totaltest.h"

#define NUM_REQ 5

int
main(int argc, char *argv[])
{
  int ss1 = NUM_REQ/2, ss2 = NUM_REQ - (NUM_REQ/2);
  grpc_sessionid_t sessionID[NUM_REQ];
  grpc_function_handle_t handle;
  int i, len[] = {15,8,5,10,20};
  grpc_error_t status;
  int saw_failure = 0;

  if(grpc_initialize(NULL) != GRPC_NO_ERROR) {
    grpc_perror("grpc_initialize");
    die(EXIT_FAILURE);
  }

  if(grpc_function_handle_default(&handle, "sleeptest") != GRPC_NO_ERROR) {
    fprintf(stderr,"Error creating function handle\n");
    die(EXIT_FAILURE);
  }

  for(i=0;i<NUM_REQ;i++) {
    status = grpc_call_async(&handle, &(sessionID[i]), len[i]);
    if(status != GRPC_NO_ERROR) {
      grpc_perror("grpc_call");
      die(sessionID[i]);
    }

    printf("sleeptest session ID = %d\n", sessionID[i]);
  }

  for(i=0;i<ss1;i++) {
    int id;

    printf("Waiting for anything from the first set of calls.\n");

    status = grpc_wait_or(sessionID, ss1, &id);

    printf("%d completed\n", id);
    
    if(status != GRPC_NO_ERROR) {
      printf("Warning: one or more calls in the first set failed.\n");
      saw_failure = 1;
    }
  }

  for(i=0;i<ss2;i++) {
    int id;

    printf("Waiting for anything from the second set of calls.\n");

    status = grpc_wait_or(sessionID+ss1, ss2, &id);

    printf("%d completed\n", id);
    
    if(status != GRPC_NO_ERROR) {
      printf("Warning: one or more calls in the second set failed.\n");
      saw_failure = 1;
    }
  }
    
  grpc_function_handle_destruct(&handle);

  printf("Finalizing...\n");

  grpc_finalize();

  printf("Done.\n");

  die(saw_failure ? EXIT_FAILURE : EXIT_SUCCESS);
}
