/**
 * @file
 *
 * This program tests the grpc_wait() function which blocks until
 * the specified request has completed.
 *
 */
/* $Id: wait_test.c,v 1.8 2006/01/30 17:57:36 yarkhan Exp $ */
/* $UTK_Copyright: $ */

#include <stdio.h>
#include <stdlib.h>

#include "portability.h"	/* for <unistd.h> */
#include "grpc.h"
#include "totaltest.h"

#define WAIT_TIME 10

int
main(int argc, char *argv[])
{
  grpc_function_handle_t handle;
  grpc_sessionid_t sessionID;
  grpc_error_t status;

  if(grpc_initialize(NULL) != GRPC_NO_ERROR) {
    grpc_perror("grpc_initialize");
    die(EXIT_FAILURE);
  }

  if(grpc_function_handle_default(&handle, "sleeptest") != GRPC_NO_ERROR) {
    fprintf(stderr,"Error creating function handle\n");
    die(EXIT_FAILURE);
  }

  status = grpc_call_async(&handle, &sessionID, WAIT_TIME);

  if(status != GRPC_NO_ERROR) {
    grpc_perror("grpc_call");
    die(sessionID);
  }

  printf("Waiting for result.\n");
  
  status = grpc_wait(sessionID);
 
  if(status != GRPC_NO_ERROR) {
    fprintf(stderr,"grpc_wait failed\n");
    die(EXIT_FAILURE);
  }

  printf("Success.\n\n");

  status = grpc_call_async(&handle, &sessionID, WAIT_TIME);

  if(status != GRPC_NO_ERROR) {
    grpc_perror("grpc_call");
    die(sessionID);
  }

  printf("Sleeping before wait to see if it detects it's already done.\n");
  sleep(WAIT_TIME+2);

  status = grpc_wait(sessionID);

  if(status != GRPC_NO_ERROR) {
    fprintf(stderr,"grpc_wait failed\n");
    die(EXIT_FAILURE);
  }

  printf("Success.\n\n");

  printf("Doing blocking call.\n");
  status = grpc_call(&handle, WAIT_TIME);

  if(status != GRPC_NO_ERROR) {
    printf("GRPC error status  = %d\n", status);
    grpc_perror("grpc_call");
    die(status);
  }

  printf("Success.\n\n");

  grpc_function_handle_destruct(&handle);

  printf("Finalizing...\n");

  grpc_finalize();

  printf("Done.\n");

  die(EXIT_SUCCESS);
}
