/**
 * @file
 *
 * This program makes a blocking GridRPC call to calculate the dot
 * product of two complex vectors.
 *
 */
/* $Id: zdot_test.c,v 1.4 2005/11/10 20:46:06 seymour Exp $ */
/* $UTK_Copyright: $ */

#include <stdio.h>
#include <stdlib.h>

#include "grpc.h"
#include "totaltest.h"

int
main(int argc, char *argv[])
{
  int n, incx, incy, status;
  gs_dcomplex correct_result = {13190.0,-250.0};
  gs_dcomplex gs_result = {0.0, 0.0};
  gs_dcomplex dx[] = {{10.0, 2.0}, {20.0, 3.0}, {30.0, 4.0}, {40.0, 5.0}, {50.0, 6.0}};
  gs_dcomplex dy[] = {{60.0, 7.0}, {70.0, 8.0}, {80.0, 9.0}, {90.0, 10.0}, {100.0, 11.0}};
  grpc_function_handle_t handle;
  char answer[100];

  n = 5;

  incx = incy = 1;

  if(grpc_initialize(NULL) != GRPC_NO_ERROR) {
    grpc_perror("grpc_initialize");
    die(EXIT_FAILURE);
  }

  if(grpc_function_handle_default(&handle, "zdotc") != GRPC_NO_ERROR) {
    fprintf(stderr,"Error creating function handle\n");
    die(EXIT_FAILURE);
  }

  status = grpc_call(&handle, n, dx, incx, dy, incy, &gs_result);

  if(status != GRPC_NO_ERROR) {
    printf("GRPC error status  = %d\n", status);
    grpc_perror("grpc_call");
    die(status);
  }

  if(GS_NE(gs_result.r, correct_result.r) || 
     GS_NE(gs_result.i, correct_result.i)) {
    printf("\nresult from GRPC = %g+%gi (correct result should be %g+%gi)\n", 
     gs_result.r, gs_result.i, correct_result.r, correct_result.i);
    die(EXIT_FAILURE);
  }
  else
    printf("\nresult is correct\n");

  grpc_function_handle_destruct(&handle);

  printf("\n\nNow testing whether assigned-server feature works.\n");

  printf("\nEnter the hostname of a specific GridSolve server\n");
  printf("or hit enter to skip this test: ");

  if(fgets(answer,100,stdin) != NULL) {
    answer[strlen(answer)-1] = 0;

    if(strcmp(answer,"") != 0) {
      grpc_function_handle_init(&handle, answer, "zdotc");

      printf("Ok, going to try to use server '%s'..\n", answer);

      status = grpc_call(&handle, n, dx, incx, dy, incy, &gs_result);

      if(status != GRPC_NO_ERROR) {
        printf("GRPC error status  = %d\n", status);
        grpc_perror("grpc_call");
        die(status);
      }

      if(GS_NE(gs_result.r, correct_result.r) || 
         GS_NE(gs_result.i, correct_result.i)) {
        printf("\nresult from GRPC = %g+%gi (correct result should be %g+%gi)\n", 
           gs_result.r, gs_result.i, correct_result.r, correct_result.i);
        die(EXIT_FAILURE);
      }
      else
        printf("\nresult is correct\n");
    }
  }

  printf("Finalizing...\n");

  grpc_finalize();

  printf("Done.\n");

  die(EXIT_SUCCESS);
}
