%*****************************************************************/
% blas_test.m                                              
% Victor Eijkhout
% Modified/updated by Asim YarKhan (2005/02/10) 
%/*****************************************************************/

test_dgemm=1;
% test_daxpy=1;
test_ddot=1;
% test_dmatmul=1;
% test_zgemm=1;
% test_zmatmul=1;
% test_zaxpy=1;

testsizes = [1,5,12,37];

%
% ddot
%
if test_ddot==1,
  for sizen=testsizes,
    fprintf('Testing DDOT for size %d\n',sizen);
    if sizen==1, xrange=1; yrange=1;
    else, xrange = 1:3:sizen/2; yrange = 1:5:2*sizen/3; end;
    %xrange=1; yrange=1;
    for incx=xrange;
      for incy=yrange;
        nx = round(sizen/incx); ny = round(sizen/incy);
        n = min(nx,ny); len = max(n*incx,n*incy);
        x = rand(len,1); y = rand(len,1);
        zn = gridsolve('ddot',n,x,incx,y,incy);
        [e] = gridsolve_err;
        if (e ~= 0)
          fprintf(2,'%s\n',gridsolve_errmsg(e))
        else
          zm = y; zm(1+incy*(0:n-1)) = y(1+incy*(0:n-1))'*x(1+incx*(0:n-1));
          err = norm(zn-zm);
          if (err>15.0)
            fprintf('incx=%d incy=%d, residual norm=%e\n',incx,incy,err);
            fprintf('Result: Failuer BLAS DDOT size %d (x[%d] y[%d])\n', sizen,size(x,1),size(y,1));
          else
            fprintf('Result: Success BLAS DDOT size %d (x[%d] y[%d])\n', sizen,size(x,1),size(y,1));
          end;
        end;
      end;
    end;
  end;
end;

%
% daxpy
%
if test_daxpy==1,
  for sizen=testsizes,
    fprintf('Testing DAXPY for size %d\n',sizen);
    if sizen==1, xrange=1; yrange=1;
    else, xrange = 1:3:sizen/2; yrange = 1:5:2*sizen/3; end;
    %xrange=1; yrange=1;
    for incx=xrange;
      for incy=yrange;
        nx = floor((sizen-1)/incx); ny = floor((sizen-1)/incy);
        n = min(nx,ny); %len = 1+max(n*incx,n*incy);
        x = rand(sizen,1); y = rand(sizen,1); alpha = rand; alpha=1;
        % n, nx, ny, incx, incy
        zn = y;
        zn(1:n*abs(incy)) = gridsolve('daxpy',n,alpha,x,incx,y,incy);
        [e] = gridsolve_err;
        if (e ~= 0)
          fprintf(2,'%s\n',gridsolve_errmsg(e));
        else
          zm = y; zm(1+incy*(0:n-1)) = y(1+incy*(0:n-1)) + alpha*x(1+incx*(0:n-1));
          % size(zn), size(zm),
          err = norm(zn-zm);
          if err>15.0, 
            fprintf('incx=%d incy=%d, residual norm=%e\n',incx,incy,err);
            fprintf('Result: Failure BLAS DAXPY size %d\n', sizen);
          else,
            fprintf('Result: Success BLAS DAXPY size %d\n', sizen);
          end;
        end;
      end;
    end;
  end;
end;

%
% dgemm
%
if test_dgemm==1,
  alpha = rand; 
  beta = rand;  
  for sizen=testsizes,
    fprintf('Testing DGEMM for size %d\n',sizen);
    A = rand(sizen); B = rand(sizen); C = rand(sizen);
    for transa=['n','t'],
      if transa=='t', opA = A'; else, opA = A; end;
      for transb=['n','t'],
        if transb=='t', opB = B'; else, opB = B; end;
        fprintf('..case %s %s:\n',transa,transb);
        [Cn] = gs_call('dgemm',transa,transb,sizen,sizen,sizen,alpha,A,sizen,B,sizen,beta,C,sizen);
        [e] = gridsolve_err;
        if (e ~= 0)
          fprintf(2,'%s\n',gridsolve_errmsg(e))
        else
          Cm = alpha*opA*opB+beta*C;
          error = norm(Cn-Cm);
          if(error>15.0)
            fprintf(' residual is %e\n',error);
            fprintf('Result: Failure BLAS DGEMM size %d\n', sizen);
          else
            fprintf('Result: Success BLAS DGEMM size %d\n', sizen);
          end;
        end;
      end;
    end;
  end;
end;

