% 
% Some matlab tests.
%
% To run this script, your MATLABPATH will need to be setup to include:
%
% setenv MATLABPATH ${MATLABPATH}:${GRIDSOLVE_ROOT}/src/matlab_client
% 
% Then run matlab and this script.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

accepterror = 1.0e-05;
M = 50;
N = 50;
density = 0.2;

testcount=0;
failedcount=0;

pause_time = 0;
async_pause = 0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fname = 'return_double_matrix';
inintm = M;
inintn = N;
indoublematmn = rand(inintm, inintn)*100;
correctdoublemn = indoublematmn+1;
% gs_call(fname, indoublematmn, inintm, inintn, resultdoublemn);
sid = gs_call_async(fname, indoublematmn, inintm, inintn);
if (sid >= 0) 
  pause(async_pause);
  [resultdoublemn] = gs_wait(sid);
end;
testcount = testcount+1;
if (size(resultdoublemn) == [0,0])
  fprintf('Result: Failed %s (async) test\n', fname); 
  failedcount = failedcount+1;
elseif (max(abs(resultdoublemn-correctdoublemn)) > accepterror)
  fprintf('Result: Failed %s (async) test\n', fname); 
  failedcount = failedcount+1;
else 
  fprintf('Result: Success %s (async) test\n', fname); 
end

pause(pause_time);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fname = 'sparse_double_matrix_test';
inmn = sprand(M, N, density)*100;
correctmn = spones(inmn) + inmn;
[resultmn] = gs_call(fname, nnz(inmn), inmn, nnz(inmn), M, N); 
testcount = testcount+1;
nonz = nonzeros(inmn); 
nonz = nonzeros(resultmn);
if (size(resultmn) == [0,0])
  fprintf('Result: Failed %s test\n', fname); 
  failedcount = failedcount+1;
elseif (max(abs(resultmn-correctmn)) > accepterror)
  fprintf('Result: Failed %s test\n', fname); 
  failedcount = failedcount+1;
else 
  fprintf('Result: Success %s test\n', fname); 
end

pause(pause_time);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fname = 'return_float_vector';
inintn = N;
infloatvecn = rand(inintn,1)*100;
correctfloatn = infloatvecn+1;
% gs_call(fname, infloatvecn, inintn, resultfloatn);
sid = gs_call_async(fname, infloatvecn, inintn);
if (sid >= 0) 
  pause(async_pause);
  [resultfloatn] = gs_wait(sid);
end;
testcount = testcount+1;
if (size(resultfloatn) == [0,0]) 
  fprintf('Result: Failed %s (async) test\n', fname); 
  failedcount = failedcount+1;
elseif (max(abs(resultfloatn-correctfloatn)) > accepterror)
  fprintf('Result: Failed %s (async) test\n', fname); 
  failedcount = failedcount+1;
else 
  fprintf('Result: Success %s (async) test\n', fname); 
end

pause(pause_time);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fname = 'return_float_scalar';
infloatn = rand(1)*100;
resultfloatn = 0;
correctfloatn = infloatn+1;
[resultfloatn] = gs_call(fname, infloatn);
testcount = testcount+1;
if (size(resultfloatn) == [0,0])
  fprintf('Result: Failed %s test\n', fname); 
  failedcount = failedcount+1;
elseif (max(abs(resultfloatn-correctfloatn)) > accepterror)
  fprintf('Result: Failed %s test\n', fname); 
  failedcount = failedcount+1;
else 
  fprintf('Result: Success %s test\n', fname); 
end

pause(pause_time);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fname = 'return_double_vector';
inintn = N;
indoublevecn = rand(inintn,1)*100;
correctdoublen = indoublevecn+1;
sid = gs_call_async(fname, indoublevecn, inintn);
if (sid >= 0) 
  pause(async_pause);
  [resultdoublen] = gs_wait(sid);
end;
testcount = testcount+1;
if (size(resultdoublen) == [0,0])
  fprintf('Result: Failed %s (async) test\n', fname); 
  failedcount = failedcount+1;
elseif (max(abs(resultdoublen-correctdoublen)) > accepterror)
  fprintf('Result: Failed %s (async) test\n', fname); 
  failedcount = failedcount+1;
else 
  fprintf('Result: Success %s (async) test\n', fname); 
end

pause(pause_time);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fname = 'return_float_matrix';
inintm = M;
inintn = N;
infloatmatmn = rand(inintm, inintn)*100;
correctfloatmn = infloatmatmn+1;
% gs_call(fname, infloatmatmn, inintm, inintn, resultfloatmn);
[sid] = gs_call_async(fname, infloatmatmn, inintm, inintn);
if (sid >= 0) 
  pause(async_pause);
  [resultfloatmn] = gs_wait(sid, infloatmatmn, inintm, inintn);
end
testcount = testcount+1;
if (size(resultfloatmn) == [0,0])
  fprintf('Result: Failed %s (async) test\n', fname); 
  failedcount = failedcount+1;
elseif (max(abs(resultfloatmn-correctfloatmn)) > accepterror)
  fprintf('Result: Failed %s (async) test\n', fname); 
  failedcount = failedcount+1;
else 
  fprintf('Result: Success %s (async) test\n', fname); 
end

pause(pause_time);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fname = 'return_int_matrix';
inintm = M;
inintn = N;
inintmatmn = floor(rand(inintm, inintn)*100);
resultintmn = zeros(inintm, inintn);
correctintmn = inintmatmn+1;
% gs_call(fname, inintmatmn, inintm, inintn, resultintmn);
sid = gs_call_async(fname, inintmatmn, inintm, inintn);
if (sid >= 0) 
  pause(async_pause);
  [resultintmn] = gs_wait(sid);
end;
testcount = testcount+1;
if (size(resultintmn) == [0,0]);
  fprintf('Result: Failed %s (async) test\n', fname); 
  failedcount = failedcount+1;
elseif (max(abs(resultintmn-correctintmn)) > accepterror)
  fprintf('Result: Failed %s (async) test\n', fname); 
  failedcount = failedcount+1;
else 
  fprintf('Result: Success %s (async) test\n', fname); 
end

pause(pause_time);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fname = 'return_int_vector';
inintn = N;
inintvecn = floor(rand(inintn,1)*100);
resultintn = zeros(inintn,1);
correctintn = inintvecn+1;
% gs_call(fname, inintvecn, inintn, resultintn);
sid = gs_call_async(fname, inintvecn, inintn);
if (sid >= 0) 
  pause(async_pause);
  [resultintn] = gs_wait(sid);
end;
testcount = testcount+1;
if (size(resultintn) == [0,0]);
  fprintf('Result: Failed %s (async) test\n', fname); 
  failedcount = failedcount+1;
elseif (max(abs(resultintn-correctintn)) > accepterror)
  fprintf('Result: Failed %s (async) test\n', fname); 
  failedcount = failedcount+1;
else 
  fprintf('Result: Success %s (async) test\n', fname); 
end

pause(pause_time);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fname = 'return_int_scalar';
inintn = floor(rand(1)*100);
resultintn = 0;
correctintn = inintn+1;
% gs_call(fname, inintn, resultintn);
sid = gs_call_async(fname, inintn);
if (sid >= 0) 
  pause(async_pause);
  [resultintn] = gs_wait(sid);
end;
testcount = testcount+1;
if (size(resultintn) == [0,0]);
  fprintf('Result: Failed %s (async) test\n', fname); 
  failedcount = failedcount+1;
elseif (max(abs(resultintn-correctintn)) > accepterror)
  fprintf('Result: Failed %s (async) test\n', fname); 
  failedcount = failedcount+1;
else 
  fprintf('Result: Success %s (async) test\n', fname); 
end

pause(pause_time);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fname = 'return_double_scalar';
indoublen = (rand(1)*100);
resultdoublen = 0;
correctdoublen = indoublen+1;
% gs_call(fname, indoublen, resultdoublen);
sid = gs_call_async(fname, indoublen);
if (sid >= 0) 
  pause(async_pause);
  [resultdoublen] = gs_wait(sid);
end;
testcount = testcount+1;
if (size(resultdoublen) == [0,0])
  fprintf('Result: Failed %s (async) test\n', fname); 
  failedcount = failedcount+1;
elseif (max(abs(resultdoublen-correctdoublen)) > accepterror)
  fprintf('Result: Failed %s (async) test\n', fname); 
  failedcount = failedcount+1;
else 
  fprintf('Result: Success %s (async) test\n', fname); 
end

pause(pause_time);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fname = 'return_float_scalar';
infloatn = rand(1)*100;
resultfloatn = 0;
correctfloatn = infloatn+1;
% gs_call(fname, infloatn, resultfloatn);
sid = gs_call_async(fname, infloatn);
if (sid >= 0) 
  pause(async_pause);
  [resultfloatn] = gs_wait(sid);
end;
testcount = testcount+1;
if (size(resultfloatn) == [0,0])
  fprintf('Result: Failed %s (async) test\n', fname); 
  failedcount = failedcount+1;
elseif (max(abs(resultfloatn-correctfloatn)) > accepterror)
  fprintf('Result: Failed %s (async) test\n', fname); 
  failedcount = failedcount+1;
else 
  fprintf('Result: Success %s (async) test\n', fname); 
end

pause(pause_time);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fname = 'return_float_matrix';
inintm = M;
inintn = N;
infloatmatmn = rand(inintm, inintn)*100;
resultfloatmn = zeros(inintm, inintn);
correctfloatmn = infloatmatmn+1;
[resultfloatmn] = gs_call(fname, infloatmatmn, inintm, inintn);
testcount = testcount+1;
if (size(resultfloatmn) == [0,0])
  fprintf('Result: Failed %s test\n', fname); 
  failedcount = failedcount+1;
elseif (max(abs(resultfloatmn-correctfloatmn)) > accepterror)
  fprintf('Result: Failed %s test\n', fname); 
  failedcount = failedcount+1;
else 
  fprintf('Result: Success %s test\n', fname); 
end

pause(pause_time);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fname = 'return_double_matrix';
inintm = M;
inintn = N;
indoublematmn = rand(inintm, inintn)*100;
resultdoublemn = zeros(inintm, inintn);
correctdoublemn = indoublematmn+1;
[resultdoublemn] = gs_call(fname, indoublematmn, inintm, inintn);
testcount = testcount+1;
if (size(resultdoublemn) == [0,0])
  fprintf('Result: Failed %s test\n', fname); 
  failedcount = failedcount+1;
elseif (max(abs(resultdoublemn-correctdoublemn)) > accepterror)
  fprintf('Result: Failed %s test\n', fname); 
  failedcount = failedcount+1;
else 
  fprintf('Result: Success %s test\n', fname); 
end

pause(pause_time);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fname = 'return_int_matrix';
inintm = M;
inintn = N;
inintmatmn = floor(rand(inintm, inintn)*100);
resultintmn = zeros(inintm, inintn);
correctintmn = inintmatmn+1;
[resultintmn] = gs_call(fname, inintmatmn, inintm, inintn);
testcount = testcount+1;
if (size(resultintmn) == [0,0]);
  fprintf('Result: Failed %s test\n', fname); 
  failedcount = failedcount+1;
elseif (max(abs(resultintmn-correctintmn)) > accepterror)
  fprintf('Result: Failed %s test\n', fname); 
  failedcount = failedcount+1;
else 
  fprintf('Result: Success %s test\n', fname); 
end

pause(pause_time);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fname = 'return_float_vector';
inintn = N;
infloatvecn = rand(inintn,1)*100;
resultfloatn = zeros(inintn,1);
correctfloatn = infloatvecn+1;
[resultfloatn] = gs_call(fname, infloatvecn, inintn);
testcount = testcount+1;
if (size(resultfloatn) == [0,0])
  fprintf('Result: Failed %s test\n', fname); 
  failedcount = failedcount+1;
elseif (max(abs(resultfloatn-correctfloatn)) > accepterror)
  fprintf('Result: Failed %s test\n', fname); 
  failedcount = failedcount+1;
else 
  fprintf('Result: Success %s test\n', fname); 
end

pause(pause_time);
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fname = 'return_double_vector';
inintn = N;
indoublevecn = rand(inintn, 1)*100;
resultdoublen = zeros(inintn,1);
correctdoublen = indoublevecn+1;
[resultdoublen] = gs_call(fname, indoublevecn, inintn);
testcount = testcount+1;
if (size(resultdoublen) == [0,0])
  fprintf('Result: Failed %s test\n', fname); 
  failedcount = failedcount+1;
elseif (max(abs(resultdoublen-correctdoublen)) > accepterror)
  fprintf('Result: Failed %s test\n', fname); 
  failedcount = failedcount+1;
else 
  fprintf('Result: Success %s test\n', fname); 
end

pause(pause_time);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fname = 'return_int_vector';
inintn = N;
inintvecn = floor(rand(inintn,1)*100);
resultintn = zeros(inintn,1);
correctintn = inintvecn+1;
[resultintn] = gs_call(fname, inintvecn, inintn);
testcount = testcount+1;
if (size(resultintn) == [0,0]);
  fprintf('Result: Failed %s test\n', fname); 
  failedcount = failedcount+1;
elseif (max(abs(resultintn-correctintn)) > accepterror)
  fprintf('Result: Failed %s test\n', fname); 
  failedcount = failedcount+1;
else 
  fprintf('Result: Success %s test\n', fname); 
end

pause(pause_time);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fname = 'return_int_scalar';
inintn = floor(rand(1)*100);
resultintn = 0;
correctintn = inintn+1;
[resultintn] = gs_call(fname, inintn);
testcount = testcount+1;
if (size(resultintn) == [0,0]);
  fprintf('Result: Failed %s test\n', fname); 
  failedcount = failedcount+1;
elseif (max(abs(resultintn-correctintn)) > accepterror)
  fprintf('Result: Failed %s test\n', fname); 
  failedcount = failedcount+1;
else 
  fprintf('Result: Success %s test\n', fname); 
end

pause(pause_time);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fname = 'return_double_scalar';
indoublen = (rand(1)*100);
resultdoublen = 0;
correctdoublen = indoublen+1;
[resultdoublen] = gs_call(fname, indoublen);
testcount = testcount+1;
if (size(resultdoublen) == [0,0])
  fprintf('Result: Failed %s test\n', fname); 
  failedcount = failedcount+1;
elseif (max(abs(resultdoublen-correctdoublen)) > accepterror)
  fprintf('Result: Failed %s test\n', fname); 
  failedcount = failedcount+1;
else 
  fprintf('Result: Success %s test\n', fname); 
end

pause(pause_time);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% fname = 'return_char_scalar';
% incharn = char(floor(rand(1)*100));
% resultcharn = 0;
% correctcharn = incharn+1;
% [resultcharn] = gs_call(fname, incharn);
% testcount = testcount+1
% if (size(resultcharn) == [0,0]);
%   fprintf('Result: Failed %s test\n', fname); 
%   failedcount = failedcount+1;
% elseif (max(abs(resultcharn-correctcharn)) > accepterror)
%   fprintf('Result: Failed %s test\n', fname); 
%   failedcount = failedcount+1;
% else 
%   fprintf('Result: Success %s test\n', fname); 
% end

pause(pause_time);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fname = 'return_scomplex_scalar';
in=0; correct=0; result=0; sid=0;
in = rand(1,1)*100 + i*rand(1,1)*100;
correct = in + (1+1i);
sid = gs_call_async(fname, in);
if (sid >= 0) 
  pause(async_pause);
  [result] = gs_wait(sid);
end;
testcount = testcount+1;
if (size(result) == [0,0])
  fprintf('Result: Failed %s (async) test\n', fname); 
  failedcount = failedcount+1;
elseif (max(abs(result-correct)) > accepterror)
  fprintf('Result: Failed %s (async) test\n', fname); 
  failedcount = failedcount+1;
else 
  fprintf('Result: Success %s (async) test\n', fname); 
end
pause(pause_time);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fname = 'return_scomplex_scalar';
in=0; correct=0; result=0; sid=0;
in = rand(1,1)*100 + i*rand(1,1)*100;
correct = in + (1+1i);
[result] = gs_call(fname, in);
testcount = testcount+1;
if (size(result) == [0,0])
  fprintf('Result: Failed %s test\n', fname); 
  failedcount = failedcount+1;
elseif (max(abs(result-correct)) > accepterror)
  fprintf('Result: Failed %s test\n', fname); 
  failedcount = failedcount+1;
else 
  fprintf('Result: Success %s test\n', fname); 
end
pause(pause_time);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fname = 'return_scomplex_vector';
in=0; correct=0; result=0; sid=0;
in = rand(N,1)*100 + i*rand(N,1)*100;
correct = in + (1+1i);
sid = gs_call_async(fname, in, N);
if (sid >= 0) 
  pause(async_pause);
  [result] = gs_wait(sid);
end;
testcount = testcount+1;
if (size(result) == [0,0])
  fprintf('Result: Failed %s (async) test\n', fname); 
  failedcount = failedcount+1;
elseif (max(abs(result-correct)) > accepterror)
  fprintf('Result: Failed %s (async) test\n', fname); 
  failedcount = failedcount+1;
else 
  fprintf('Result: Success %s (async) test\n', fname); 
end
pause(pause_time);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fname = 'return_scomplex_vector';
in=0; correct=0; result=0; sid=0;
in = rand(N,1)*100 + i*rand(N,1)*100;
correct = in + (1+1i);
[result] = gs_call(fname, in, N);
testcount = testcount+1;
if (size(result) == [0,0])
  fprintf('Result: Failed %s test\n', fname); 
  failedcount = failedcount+1;
elseif (max(abs(result-correct)) > accepterror)
  fprintf('Result: Failed %s test\n', fname); 
  failedcount = failedcount+1;
else 
  fprintf('Result: Success %s test\n', fname); 
end
pause(pause_time);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fname = 'return_scomplex_matrix';
in=0; correct=0; result=0; sid=0;
in = rand(M,N)*100 + i*rand(M,N)*100;
correct = in + (1+1i);
sid = gs_call_async(fname, in, N, M);
if (sid >= 0) 
  pause(async_pause);
  [result] = gs_wait(sid);
end;
testcount = testcount+1;
if (size(result) == [0,0])
  fprintf('Result: Failed %s (async) test\n', fname); 
  failedcount = failedcount+1;
elseif (max(abs(result-correct)) > accepterror)
  fprintf('Result: Failed %s (async) test\n', fname); 
  failedcount = failedcount+1;
else 
  fprintf('Result: Success %s (async) test\n', fname); 
end
pause(pause_time);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fname = 'return_scomplex_matrix';
in=0; correct=0; result=0; sid=0;
in = rand(M,N)*100 + i*rand(M,N)*100;
correct = in + (1+1i);
[result] = gs_call(fname, in, N, M);
testcount = testcount+1;
if (size(result) == [0,0])
  fprintf('Result: Failed %s test\n', fname); 
  failedcount = failedcount+1;
elseif (max(abs(result-correct)) > accepterror)
  fprintf('Result: Failed %s test\n', fname); 
  failedcount = failedcount+1;
else 
  fprintf('Result: Success %s test\n', fname); 
end
pause(pause_time);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fname = 'return_dcomplex_scalar';
in=0; correct=0; result=0; sid=0;
in = rand(1,1)*100 + i*rand(1,1)*100;
correct = in + (1+1i);
sid = gs_call_async(fname, in);
if (sid >= 0) 
  pause(async_pause);
  [result] = gs_wait(sid);
end;
testcount = testcount+1;
if (size(result) == [0,0])
  fprintf('Result: Failed %s (async) test\n', fname); 
  failedcount = failedcount+1;
elseif (max(abs(result-correct)) > accepterror)
  fprintf('Result: Failed %s (async) test\n', fname); 
  failedcount = failedcount+1;
else 
  fprintf('Result: Success %s (async) test\n', fname); 
end
pause(pause_time);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fname = 'return_dcomplex_scalar';
in=0; correct=0; result=0; sid=0;
in = rand(1,1)*100 + i*rand(1,1)*100;
correct = in + (1+1i);
[result] = gs_call(fname, in);
testcount = testcount+1;
if (size(result) == [0,0])
  fprintf('Result: Failed %s test\n', fname); 
  failedcount = failedcount+1;
elseif (max(abs(result-correct)) > accepterror)
  fprintf('Result: Failed %s test\n', fname); 
  failedcount = failedcount+1;
else 
  fprintf('Result: Success %s test\n', fname); 
end
pause(pause_time);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fname = 'return_dcomplex_vector';
in=0; correct=0; result=0; sid=0;
in = rand(N,1)*100 + i*rand(N,1)*100;
correct = in + (1+1i);
sid = gs_call_async(fname, in, N);
if (sid >= 0) 
  pause(async_pause);
  [result] = gs_wait(sid);
end;
testcount = testcount+1;
if (size(result) == [0,0])
  fprintf('Result: Failed %s (async) test\n', fname); 
  failedcount = failedcount+1;
elseif (max(abs(result-correct)) > accepterror)
  fprintf('Result: Failed %s (async) test\n', fname); 
  failedcount = failedcount+1;
else 
  fprintf('Result: Success %s (async) test\n', fname); 
end
pause(pause_time);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fname = 'return_dcomplex_vector';
in=0; correct=0; result=0; sid=0;
in = rand(N,1)*100 + i*rand(N,1)*100;
correct = in + (1+1i);
[result] = gs_call(fname, in, N);
testcount = testcount+1;
if (size(result) == [0,0])
  fprintf('Result: Failed %s test\n', fname); 
  failedcount = failedcount+1;
elseif (max(abs(result-correct)) > accepterror)
  fprintf('Result: Failed %s test\n', fname); 
  failedcount = failedcount+1;
else 
  fprintf('Result: Success %s test\n', fname); 
end
pause(pause_time);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fname = 'return_dcomplex_matrix';
in=0; correct=0; result=0; sid=0;
in = rand(M,N)*100 + i*rand(M,N)*100;
correct = in + (1+1i);
sid = gs_call_async(fname, in, N, M);
if (sid >= 0) 
  pause(async_pause);
  [result] = gs_wait(sid);
end;
testcount = testcount+1;
if (size(result) == [0,0])
  fprintf('Result: Failed %s (async) test\n', fname); 
  failedcount = failedcount+1;
elseif (max(abs(result-correct)) > accepterror)
  fprintf('Result: Failed %s (async) test\n', fname); 
  failedcount = failedcount+1;
else 
  fprintf('Result: Success %s (async) test\n', fname); 
end
pause(pause_time);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fname = 'return_dcomplex_matrix';
in=0; correct=0; result=0; sid=0;
in = rand(M,N)*100 + i*rand(M,N)*100;
correct = in + (1+1i);
[result] = gs_call(fname, in, N, M);
testcount = testcount+1;
if (size(result) == [0,0])
  fprintf('Result: Failed %s test\n', fname); 
  failedcount = failedcount+1;
elseif (max(abs(result-correct)) > accepterror)
  fprintf('Result: Failed %s test\n', fname); 
  failedcount = failedcount+1;
else 
  fprintf('Result: Success %s test\n', fname); 
end
pause(pause_time);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% fname = 'return_char_matrix';
% incharm = M;
% incharn = N;
% incharmatmn = char(floor(rand(incharm, incharn)*127));
% resultcharmn = zeros(incharm, incharn);
% correctcharmn = incharmatmn+1;
% % gs_call(fname, incharmatmn, incharm, incharn, resultcharmn);
% sid = gs_call_async(fname, incharmatmn, incharm, incharn);
% if (sid >= 0) 
%   pause(async_pause);
%   [resultcharmn] = gs_wait(sid);
% end;
% testcount = testcount+1;
% if (size(resultcharmn) == [0,0])
%   fprintf('Result: Failed %s (async) test\n', fname); 
%   failedcount = failedcount+1;
% else if (max(abs(resultcharmn-correctcharmn)) > accepterror)
%   fprintf('Result: Failed %s (async) test\n', fname); 
%   failedcount = failedcount+1;
% else 
%   fprintf('Result: Success %s (async) test\n', fname); 
% end

pause(pause_time);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% fname = 'return_char_vector';
% incharn = N;
% incharvecn = char(floor(rand(incharn,1)*127));
% resultcharn = zeros(incharn,1);
% correctcharn = incharvecn+1;
% % gs_call(fname, incharvecn, incharn, resultcharn);
% sid = gs_call_async(fname, incharvecn, incharn);
% if (sid >= 0) 
%   pause(async_pause);
%   [resultcharn] = gs_wait(sid);
% end;
% testcount = testcount+1
% if (size(resultcharn) == [0,0]);
%   fprintf('Result: Failed %s (async) test\n', fname); 
%   failedcount = failedcount+1;
% else if (max(abs(resultcharn-correctcharn)) > accepterror)
%   fprintf('Result: Failed %s (async) test\n', fname); 
%   failedcount = failedcount+1;
% else 
%   fprintf('Result: Success %s (async) test\n', fname); 
% end

pause(pause_time);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% fname = 'return_char_scalar';
% incharn = char(floor(rand(1)*100));
% resultcharn = 0;
% correctcharn = incharn+1;
% % gs_call(fname, incharn, resultcharn);
% sid = gs_call_async(fname, incharn);
% if (sid >= 0) 
%   pause(async_pause);
%   [resultcharn] = gs_wait(sid);
% end;
% testcount = testcount+1
% if (size(resultcharn) == [0,0]);
%   fprintf('Result: Failed %s (async) test\n', fname); 
%   failedcount = failedcount+1;
% else % if (max(abs(resultcharn-correctcharn)) > accepterror)
%   fprintf('Result: Failed %s (async) test\n', fname); 
%   failedcount = failedcount+1;
% else 
%   fprintf('Result: Success %s (async) test\n', fname); 
% end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% fname = 'return_char_matrix';
% incharm = M;
% incharn = N;
% incharmatmn = char(floor(rand(incharm, incharn)*127));
% resultcharmn = zeros(incharm, incharn);
% correctcharmn = incharmatmn+1;
% gs_call(fname, incharmatmn, incharm, incharn, resultcharmn);
% testcount = testcount+1;
% if (size(resultcharmn) == [0,0])
%   fprintf('Result: Failed %s test\n', fname); 
%   failedcount = failedcount+1;
% elseif (max(abs(resultcharmn-correctcharmn)) > accepterror)
%   fprintf('Result: Failed %s test\n', fname); 
%   failedcount = failedcount+1;
% else 
%   fprintf('Result: Success %s test\n', fname); 
% end

pause(pause_time);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% fname = 'return_char_vector';
% incharn = N;
% incharvecn = char(floor(rand(incharn,1)*127));
% resultcharn = zeros(incharn,1);
% correctcharn = incharvecn+1;
% gs_call(fname, incharvecn, incharn, resultcharn);
% testcount = testcount+1
% if (size(resultcharn) == [0,0]);
%   fprintf('Result: Failed %s test\n', fname); 
%   failedcount = failedcount+1;
% elseif (max(abs(resultcharn-correctcharn)) > accepterror)
%   fprintf('Result: Failed %s test\n', fname); 
%   failedcount = failedcount+1;
% else 
%   fprintf('Result: Success %s test\n', fname); 
% end

pause(pause_time);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fprintf('\n');
if (failedcount > 0)
  fprintf('Result: Failed %d of %d tests\n', ...
          failedcount, testcount);
else
  fprintf('Result: Passed all %d tests \n', ...
          testcount);
end


