%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% This tests to see if types are being properly coerced from Matlab to
% GridSolve.  Return data is always of double precision.
%
% To run this script, your MATLABPATH will need to be setup to include:
%
% setenv MATLABPATH ${MATLABPATH}:${GRIDSOLVE_BUILD_ROOT}/src/matlab_client:${GRIDSOLVE_SOURCE_ROOT}/src/matlab_client
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% >> help datatypes
%    double          - Convert to double precision.
%    logical         - Convert numeric values to logical.
%    cell            - Create cell array.
%    struct          - Create or convert to structure array.
%    single          - Convert to single precision.
%    uint8           - Convert to unsigned 8-bit integer.
%    uint16          - Convert to unsigned 16-bit integer.
%    uint32          - Convert to unsigned 32-bit integer.
%    uint64          - Convert to unsigned 64-bit integer.
%    int8            - Convert to signed 8-bit integer.
%    int16           - Convert to signed 16-bit integer.
%    int32           - Convert to signed 32-bit integer.
%    int64           - Convert to signed 64-bit integer.
%    inline          - Construct INLINE object.
%    function_handle - Function handle array.
%    javaArray       - Construct a Java Array object.
%    javaMethod      - Invoke a Java method.
%    javaObject      - Invoke a Java object constructor.
% 
%    cast            - Cast a variable to a different data type or class.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

accepterror = 1.0e-05;
np=0;
nf=0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

inval = rand(1)*100;

% lhs = function, coercion, rhs1, input, rhs2, correctoutput)
[np,nf]=test_type_coersion_support1('result', 'return_float_scalar', 'double', '', inval, '', double(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_float_scalar', 'single', '', inval, '', single(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_float_scalar', 'int32', '', inval, '', int32(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_float_scalar', 'uint32', '', inval, '', uint32(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_float_scalar', 'int16', '', inval, '', int16(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_float_scalar', 'uint16','',  inval, '', uint16(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_float_scalar', 'int8', '', inval, '', int8(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_float_scalar', 'uint8', '', inval, '', uint8(inval)+1, accepterror,np,nf);

[np,nf]=test_type_coersion_support1('result', 'return_double_scalar', 'double', '', inval, '', double(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_double_scalar', 'single', '', inval, '', single(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_double_scalar', 'int32', '', inval, '', int32(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_double_scalar', 'uint32', '', inval, '', uint32(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_double_scalar', 'int16', '', inval, '', int16(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_double_scalar', 'uint16','',  inval, '', uint16(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_double_scalar', 'int8', '', inval, '', int8(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_double_scalar', 'uint8', '', inval, '', uint8(inval)+1, accepterror,np,nf);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

N = 100;
inval = rand(N, 1)*100;

[np,nf]=test_type_coersion_support1('result', 'return_double_vector', 'double', '', inval, ',size(inval,1)', double(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_double_vector', 'single', '', inval, ',size(inval,1)', single(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_double_vector', 'int32', '', inval, ',size(inval,1)', int32(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_double_vector', 'uint32', '', inval, ',size(inval,1)', uint32(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_double_vector', 'int16', '', inval, ',size(inval,1)', int16(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_double_vector', 'uint16', '', inval, ',size(inval,1)', uint16(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_double_vector', 'int8', '', inval, ',size(inval,1)', int8(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_double_vector', 'uint8', '', inval, ',size(inval,1)', uint8(inval)+1, accepterror,np,nf);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

N = 100;
inval = rand(N, 1)*100;

[np,nf]=test_type_coersion_support1('result', 'return_double_vector', 'double', '', inval, ',size(inval,1)', double(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_double_vector', 'single', '', inval, ',size(inval,1)', single(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_double_vector', 'int32', '', inval, ',size(inval,1)', int32(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_double_vector', 'uint32', '', inval, ',size(inval,1)', uint32(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_double_vector', 'int16', '', inval, ',size(inval,1)', int16(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_double_vector', 'uint16', '', inval, ',size(inval,1)', uint16(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_double_vector', 'int8', '', inval, ',size(inval,1)', int8(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_double_vector', 'uint8', '', inval, ',size(inval,1)', uint8(inval)+1, accepterror,np,nf);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

N = 3;
M = 3;
inval = rand(M, N)*100;

[np,nf]=test_type_coersion_support1('result', 'return_int_matrix', 'double', '', inval, ',size(inval,1), size(inval,2)', int32(floor(inval))+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_int_matrix', 'single', '', inval, ',size(inval,1), size(inval,2)', int32(floor(inval))+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_int_matrix', 'int32', '', inval, ',size(inval,1), size(inval,2)', int32(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_int_matrix', 'uint32', '', inval, ',size(inval,1), size(inval,2)', uint32(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_int_matrix', 'int16', '', inval, ',size(inval,1), size(inval,2)', int16(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_int_matrix', 'uint16', '', inval, ',size(inval,1), size(inval,2)', uint16(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_int_matrix', 'int8', '', inval, ',size(inval,1), size(inval,2)', int8(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_int_matrix', 'uint8', '', inval, ',size(inval,1), size(inval,2)', uint8(inval)+1, accepterror,np,nf);


[np,nf]=test_type_coersion_support1('result', 'return_float_matrix', 'double', '', inval, ',size(inval,1), size(inval,2)', double(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_float_matrix', 'single', '', inval, ',size(inval,1), size(inval,2)', single(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_float_matrix', 'int32', '', inval, ',size(inval,1), size(inval,2)', int32(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_float_matrix', 'uint32', '', inval, ',size(inval,1), size(inval,2)', uint32(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_float_matrix', 'int16', '', inval, ',size(inval,1), size(inval,2)', int16(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_float_matrix', 'uint16', '', inval, ',size(inval,1), size(inval,2)', uint16(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_float_matrix', 'int8', '', inval, ',size(inval,1), size(inval,2)', int8(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_float_matrix', 'uint8', '', inval, ',size(inval,1), size(inval,2)', uint8(inval)+1, accepterror,np,nf);


[np,nf]=test_type_coersion_support1('result', 'return_double_matrix', 'double', '', inval, ',size(inval,1), size(inval,2)', double(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_double_matrix', 'single', '', inval, ',size(inval,1), size(inval,2)', single(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_double_matrix', 'int32', '', inval, ',size(inval,1), size(inval,2)', int32(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_double_matrix', 'uint32', '', inval, ',size(inval,1), size(inval,2)', uint32(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_double_matrix', 'int16', '', inval, ',size(inval,1), size(inval,2)', int16(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_double_matrix', 'uint16', '', inval, ',size(inval,1), size(inval,2)', uint16(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_double_matrix', 'int8', '', inval, ',size(inval,1), size(inval,2)', int8(inval)+1, accepterror,np,nf);
[np,nf]=test_type_coersion_support1('result', 'return_double_matrix', 'uint8', '', inval, ',size(inval,1), size(inval,2)', uint8(inval)+1, accepterror,np,nf);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fprintf('Failed %d tests (out of %d)\n', nf, nf+np);

