/*****************************************************************/
/*      farmtest.c                                               */
/*      Version 1.0                      Henri Casanova          */
/*---------------------------------------------------------------*/
/*****************************************************************/

#include "netsolve.h"
#include "netsolveclient.h"
#include <time.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
 
double win_drand48(){
	return ((double)rand()/32767.0);
}

int main(int argc,char **argv)
{
  int i,n; 
  int *status;
  char buf[128];
  double **a;
  double **b;
  int ok;
  
  if (argc != 2)
  {
    fprintf(stdout,"Usage: %s <nb requests>\n",argv[0]);
    exit(-1);
  }

  srand((unsigned)time(NULL));

  fprintf(stdout,"Testing the Farming feature...\n");
  n = atoi(argv[1]);

  fprintf(stdout,"Input Data Generated.\n");
  a = (double **)calloc(n,sizeof(double*));
  b = (double **)calloc(n,sizeof(double*));
  for (i=0;i<n;i++)
  {
    a[i] = (double *)calloc(1,sizeof(double));
    b[i] = (double *)calloc(1,sizeof(double));
    *(a[i]) =((double)n)*win_drand48();
    *(b[i]) = *(a[i]);
  }

  sprintf(buf,"i=0,%d",n-1);
  status = netsl_farm(buf,"return_double_scalar()",ns_ptr_array((void**)a,"$i"),ns_ptr_array((void **)b,"$i"));
  if (status[0] == -1)
  {
    for (i=0;i<n;i++){
    fprintf(stdout,"requests #%d: \n", i);
    }
    fprintf(stdout, "\n%s\n", "Result: Failure");
    exit(-1);
  }
  else
  {
    ok = 1;
    for (i=0;i<n;i++)
    {
      double dif = *a[i]+1.0 - *b[i];
      if ( 1E-6 < fabs(dif) )
      {
        fprintf(stdout,"i %d, %g ---> %g (dif %f)\n",i, *b[i], *a[i], dif);
        fprintf(stdout,"%p ---> %p\n",b[i],a[i]);
        ok = 0;
      }
    }
    if (!ok){
      fprintf(stdout,"Incorrect output result\n");
      fprintf(stdout, "\n%s\n", "Result: Failure");
      exit(-1);
    }
    else
      fprintf(stdout, "\n%s\n", "Result: Success");
  }
  free(status);

  exit(0);
}
