**
* @file
*
* This file contains common code used by Fortran programs that
* implement totaltest.
*
* $Id: f_totaltest_common.f,v 1.8 2005/06/10 18:04:22 seymour Exp $
* $UTK_Copyright: $
**
**
* Wbrief Common subroutine for Fortran totaltest.
*
* This subroutine makes the NetSolve call to ftotal().
*
* @param foo -- the NetSolve subroutine to be called (either
*   fnetsl for blocking or fnetslnb for non-blocking.
* @param blocking -- if true, this is a blocking request,
*   so it's not necessary to wait for the request since the
*   NetSolve call will block.
**
      subroutine do_test(foo, blocking)
      external foo
      logical blocking
      include 'fnetsolve.h'
      include 'f_totaltest.h'
      include 'f_totaltest_arg_decl.f'
      include 'f_totaltest_arg_init.f'
*
      include 'f_totaltest_call.f'
*
      if(status.lt.0) then
        call fnetslerr(status)
        write(*,'(A75)') 'Result: Failure'
        return
      endif
*
      if(.not.blocking) then
        request = status
        IF (request.GE.0) THEN
          CALL fnetslpr(request,status)
          WRITE(*,'(A,$)') '...'
 10       CONTINUE
          IF (status.NE.NetSolveNotReady) THEN
            GOTO 20
          ENDIF
          DO 15 ii=1,500
             iii = ii + 3*ii
 15       CONTINUE
          CALL fnetslpr(request,status)
          GOTO 10
 20       WRITE(*,*) ' '
          IF (status.EQ.NetSolveOK) THEN
            CALL fnetslwt(request,status)
          ENDIF
        ENDIF
      endif
      include 'f_totaltest_results.f'
      write(*,*) 'Result: Success'
*
      return
      end
**
* Determine whether two single precision numbers are "pretty close"
* to equal (within an arbitrary threshold of 0.5).
*
* @param x -- first number to be compared
* @param y -- second number to be compared
*
* @returns .true. if abs(x-y) .gt. 0.5 (i.e. x != y)
*    returns .false. otherwise
**
      logical function float_ne(x, y)
      real x, y
*
      float_ne = .false.
      if( abs(x - y) .gt. 0.5 ) then
        float_ne = .true.
      endif
*
      return
      end
**
* Determine whether two double precision numbers are "pretty close"
* to equal (within an arbitrary threshold of 0.5).
*
* @param x -- first number to be compared
* @param y -- second number to be compared
*
* @returns .true. if abs(x-y) .gt. 0.5 (i.e. x != y)
*    returns .false. otherwise
**
      logical function dbl_ne(x, y) 
      double precision x, y
*
      dbl_ne = .false.
      if( (x - y) .gt. 0.5 ) then
        dbl_ne = .true.
      endif
*
      return 
      end
**
* Determine whether two single precision complex numbers are "pretty close"
* to equal (within an arbitrary threshold of 0.5).
*     
* @param x -- first number to be compared
* @param y -- second number to be compared
* 
* @returns .true. if abs(x-y) .gt. 0.5 (i.e. x != y)
*    returns .false. otherwise
**    
      logical function dcmp_ne(x, y)
      double complex x, y
      double precision z
*     
      z = abs(dble(x) - dble(y))
      dcmp_ne = .false.
      if( z .gt. 0.5 ) then
        dcmp_ne = .true.
      endif
      z = abs(dimag(x) - dimag(y))
      if( z .gt. 0.5 ) then
        dcmp_ne = .true.
      endif
*         
      return 
      end
**
* Determine whether two complex numbers are "pretty close"
* to equal (within an arbitrary threshold of 0.5).
* 
* @param x -- first number to be compared
* @param y -- second number to be compared 
* 
* @returns .true. if abs(x-y) .gt. 0.5 (i.e. x != y)
*    returns .false. otherwise
**    
      logical function scmp_ne(x, y)
      complex x, y
      real z
*    
      z = abs(real(x) - real(y))
      scmp_ne = .false.
      if( z .gt. 0.5 ) then
        scmp_ne = .true.
      endif
      z = abs(aimag(x) - aimag(y))
      if( z .gt. 0.5 ) then
        scmp_ne = .true.
      endif
* 
      return 
      end
**
* Prints the given integer matrix to the console.
*
* @param str -- string to be printed before the matrix
* @param mat -- the matrix to be printed
* @param rows -- the number of rows in the matrix
* @param cols -- the number of columns in the matrix
**
      subroutine print_imat(str, mat, rows, cols)
      character* (*) str
      integer rows, cols
      integer mat(rows,cols), i, j
*
      write(*,*) str,':'
      do 2 i=1,rows
        do 1 j=1,cols
          write(*,'(1i6, $)') mat(i,j)
  1     continue
        write(*,*) ''
  2   continue
*
      return
      end
**
* Prints the given single precision matrix to the console.
*
* @param str -- string to be printed before the matrix
* @param mat -- the matrix to be printed
* @param rows -- the number of rows in the matrix
* @param cols -- the number of columns in the matrix
**
      subroutine print_fmat(str, mat, rows, cols)
      character* (*) str
      integer rows, cols, i, j
      real mat(rows,cols)
*
      write(*,*) str,':'
      do 2 i=1,rows
        do 1 j=1,cols
          write(*,'(1f8.2, $)') mat(i,j)
  1     continue
        write(*,*) ''
  2   continue
*
      return
      end
**
* Prints the given double precision matrix to the console.
*
* @param str -- string to be printed before the matrix
* @param mat -- the matrix to be printed
* @param rows -- the number of rows in the matrix
* @param cols -- the number of columns in the matrix
**
      subroutine print_dmat(str, mat, rows, cols)
      character* (*) str
      integer rows, cols, i, j
      double precision mat(rows,cols)
*
      write(*,*) str,':'
      do 2 i=1,rows
        do 1 j=1,cols
          write(*,'(1f8.2, $)') mat(i,j)
  1     continue
        write(*,*) ''
  2   continue
*
      return
      end
**    
* Prints the given single precision complex matrix to the console.
* 
* @param str -- string to be printed before the matrix
* @param mat -- the matrix to be printed
* @param rows -- the number of rows in the matrix
* @param cols -- the number of columns in the matrix
**
      subroutine print_scmp_mat(str, mat, rows, cols)
      character* (*) str
      integer rows, cols, i, j
      complex mat(rows,cols)
*     
      write(*,*) str,':'
      do 2 i=1,rows
        do 1 j=1,cols
          write(*,'(1a, 1f8.2, 1f8.2, 1a, $)') '(', mat(i,j), ')'
  1     continue
        write(*,*) ''
  2   continue
* 
      return
      end
**
* Prints the given double precision complex matrix to the console.
* 
* @param str -- string to be printed before the matrix
* @param mat -- the matrix to be printed
* @param rows -- the number of rows in the matrix
* @param cols -- the number of columns in the matrix
**
      subroutine print_dcmp_mat(str, mat, rows, cols)
      character* (*) str
      integer rows, cols, i, j
      double complex mat(rows,cols)
*
      write(*,*) str,':'
      do 2 i=1,rows
        do 1 j=1,cols
          write(*,'(1a, 1f8.2, 1f8.2, 1a, $)') '(', mat(i,j), ')'
  1     continue
        write(*,*) ''
  2   continue
*
      return
      end
**
* Prints the given character matrix to the console.
*
* @param str -- string to be printed before the matrix
* @param mat -- the matrix to be printed
* @param rows -- the number of rows in the matrix
* @param cols -- the number of columns in the matrix
**
      subroutine print_cmat(str, mat, rows, cols)
      character* (*) str
      integer rows, cols, i, j
      character mat(rows,cols)
*
      write(*,*) str,':'
      do 2 i=1,rows
        do 1 j=1,cols
          write(*,'(1a6, $)') mat(i,j)
  1     continue
        write(*,*) ''
  2   continue
*
      return
      end
**
* Prints the given integer vector to the console.
*
* @param str -- string to be printed before the vector
* @param vec -- the vector to be printed
* @param len -- the number of elements in the vector
**
      subroutine print_ivec(str, vec, len)
      character* (*) str
      integer len, i 
      integer vec(len)
*
      write(*,*) str,':'
      do 1 i=1,len
        write(*,'(1i6, $)') vec(i)
  1   continue
      write(*,*) ''
*
      return
      end
**
* Prints the given single precision vector to the console.
*
* @param str -- string to be printed before the vector
* @param vec -- the vector to be printed
* @param len -- the number of elements in the vector
**
      subroutine print_fvec(str, vec, len)
      character* (*) str
      integer len, i
      real vec(len)
*
      write(*,*) str,':'
      do 1 i=1,len
        write(*,'(1f8.2, $)') vec(i)
  1   continue
      write(*,*) ''
*
      return
      end
**
* Prints the given double precision vector to the console.
*
* @param str -- string to be printed before the vector
* @param vec -- the vector to be printed
* @param len -- the number of elements in the vector
**
      subroutine print_dvec(str, vec, len)
      character* (*) str
      integer len, i
      double precision vec(len)
*
      write(*,*) str,':'
      do 1 i=1,len
        write(*,'(1f8.2, $)') vec(i)
  1   continue
      write(*,*) ''
*
      return
      end
**
* Prints the given single precision complex vector to the console.
*
* @param str -- string to be printed before the vector
* @param vec -- the vector to be printed
* @param len -- the number of elements in the vector
**    
      subroutine print_scmp_vec(str, vec, len)
      character* (*) str
      integer len, i
      complex vec(len)
*     
      write(*,*) str,':'
      do 1 i=1,len
        write(*,'(1a, 1f8.2, 1f8.2, 1a, $)') '(', vec(i), ')'
  1   continue
      write(*,*) ''
*     
      return
      end
**
* Prints the given double precision complex vector to the console.
* 
* @param str -- string to be printed before the vector
* @param vec -- the vector to be printed
* @param len -- the number of elements in the vector
**    
      subroutine print_dcmp_vec(str, vec, len)
      character* (*) str
      integer len, i 
      double complex vec(len)
*     
      write(*,*) str,':'
      do 1 i=1,len
        write(*,'(1a, 1f8.2, 1f8.2, 1a, $)') '(', vec(i), ')'
  1   continue
      write(*,*) ''
*     
      return
      end
**
* Prints the given character vector to the console.
*
* @param str -- string to be printed before the vector
* @param vec -- the vector to be printed
* @param len -- the number of elements in the vector
**
      subroutine print_cvec(str, vec, len)
      character* (*) str
      integer len, i
      character vec(len)
*
      write(*,*) str,':'
      do 1 i=1,len
        write(*,'(1a6, $)') vec(i)
  1   continue
      write(*,*) ''
*
      return
      end
**
* Verifies that the results from calling totaltest are correct.
* This routine verifies the integer matrix arguments.
*
* @param in -- the input-only matrix
* @param inout -- the input-output matrix
* @param out -- the output-only matrix
* @param rows -- the number of rows in the matrices
* @param cols -- the number of columns in the matrices
* @param info -- on success, info is unchanged.
*    on failure, info is set to -1.
**
      subroutine verify_imat(in, inout, out, rows, cols, info)
      integer rows, cols, info
      integer in(rows,cols), i, j
      integer inout(rows,cols)
      integer out(rows,cols)
*
      do 2 i=1,rows
        do 1 j=1,cols
          if(in(i,j) .ne. inout(i,j) - 101) then
            write(*,*) 'Integer matrix in/out incorrect'
            info = -1
            return
          endif 
          if(in(i,j) .ne. out(i,j) - 1) then
            write(*,*) 'Integer matrix out incorrect'
            info = -1
            return
          endif 
  1     continue
  2   continue
*
      return
      end
**
* Verifies that the results from calling totaltest are correct.
* This routine verifies the single precision matrix arguments.
*
* @param in -- the input-only matrix
* @param inout -- the input-output matrix
* @param out -- the output-only matrix
* @param rows -- the number of rows in the matrices
* @param cols -- the number of columns in the matrices
* @param info -- on success, info is unchanged.
*    on failure, info is set to -1.
**
      subroutine verify_fmat(in, inout, out, rows, cols, info)
      integer rows, cols, info, i, j
      real in(rows,cols)
      real inout(rows,cols)
      real out(rows,cols)
      logical float_ne
*
      do 2 i=1,rows
        do 1 j=1,cols
          if(float_ne(in(i,j), inout(i,j) - 101.0)) then
            write(*,*) 'Single precision matrix in/out incorrect'
            info = -1
            return
          endif
          if(float_ne(in(i,j), out(i,j) - 1.0)) then
            write(*,*) 'Single precision matrix out incorrect'
            info = -1
            return
          endif
  1     continue
  2   continue
*
      return
      end
**
* Verifies that the results from calling totaltest are correct.
* This routine verifies the double precision matrix arguments.
*
* @param in -- the input-only matrix
* @param inout -- the input-output matrix
* @param out -- the output-only matrix
* @param rows -- the number of rows in the matrices
* @param cols -- the number of columns in the matrices
* @param info -- on success, info is unchanged.
*    on failure, info is set to -1.
**
      subroutine verify_dmat(in, inout, out, rows, cols, info)
      integer rows, cols, info, i, j
      double precision in(rows,cols)
      double precision inout(rows,cols)
      double precision out(rows,cols)
      logical dbl_ne
*
      do 2 i=1,rows
        do 1 j=1,cols
          if(dbl_ne(in(i,j), inout(i,j) - 101.0)) then
            write(*,*) 'Double precision matrix in/out incorrect'
            info = -1
            return
          endif
          if(dbl_ne(in(i,j), out(i,j) - 1.0)) then
            write(*,*) 'Double precision matrix out incorrect'
            info = -1
            return
          endif
  1     continue
  2   continue
*
      return
      end
**
* Verifies that the results from calling totaltest are correct.
* This routine verifies the single precision complex matrix arguments.
*
* @param in -- the input-only matrix
* @param inout -- the input-output matrix
* @param out -- the output-only matrix
* @param rows -- the number of rows in the matrices
* @param cols -- the number of columns in the matrices
* @param info -- on success, info is unchanged.
*    on failure, info is set to -1.
**
      subroutine verify_scmp_mat(in, inout, out, rows, cols, info)
      integer rows, cols, info, i, j
      complex in(rows,cols)
      complex inout(rows,cols)
      complex out(rows,cols)
      logical scmp_ne
*
      do 2 i=1,rows
        do 1 j=1,cols
          if(scmp_ne(in(i,j), inout(i,j) - (101.0,1.0))) then
            write(*,*) 'scomplex matrix in/out incorrect'
            info = -1
            return
          endif
          if(scmp_ne(in(i,j), out(i,j) - (1.0,1.0))) then
            write(*,*) 'scomplex matrix out incorrect'
            info = -1
            return
          endif
  1     continue
  2   continue
*
      return
      end
**
* Verifies that the results from calling totaltest are correct.
* This routine verifies the double precision complex matrix arguments.
*
* @param in -- the input-only matrix
* @param inout -- the input-output matrix
* @param out -- the output-only matrix
* @param rows -- the number of rows in the matrices
* @param cols -- the number of columns in the matrices
* @param info -- on success, info is unchanged.
*    on failure, info is set to -1.
**
      subroutine verify_dcmp_mat(in, inout, out, rows, cols, info)
      integer rows, cols, info, i, j
      double complex in(rows,cols)
      double complex inout(rows,cols)
      double complex out(rows,cols)
      logical dcmp_ne
*
      do 2 i=1,rows
        do 1 j=1,cols
          if(dcmp_ne(in(i,j), inout(i,j) - (101.0,1.0))) then
            write(*,*) 'dcomplex matrix in/out incorrect'
            info = -1
            return
          endif
          if(dcmp_ne(in(i,j), out(i,j) - (1.0,1.0))) then
            write(*,*) 'dcomplex matrix out incorrect'
            info = -1
            return
          endif
  1     continue
  2   continue
*
      return
      end
**
* Verifies that the results from calling totaltest are correct.
* This routine verifies the character matrix arguments.
*
* @param in -- the input-only matrix
* @param inout -- the input-output matrix
* @param out -- the output-only matrix
* @param rows -- the number of rows in the matrices
* @param cols -- the number of columns in the matrices
* @param info -- on success, info is unchanged.
*    on failure, info is set to -1.
**
      subroutine verify_cmat(in, inout, out, rows, cols, info)
      integer rows, cols, info, i, j
      character in(rows,cols)
      character inout(rows,cols)
      character out(rows,cols)
*
      do 2 i=1,rows
        do 1 j=1,cols
          if(ichar(in(i,j)) .ne. ichar(inout(i,j)) - 
     +         (ichar('j')-ichar('a'))) then
            write(*,*) 'Character matrix in/out incorrect'
            info = -1
            return
          endif
          if(ichar(in(i,j)) .ne. ichar(out(i,j)) - 1) then
            write(*,*) 'Character matrix out incorrect'
            info = -1
            return
          endif
  1     continue
  2   continue
*
      return
      end
**
* Verifies that the results from calling totaltest are correct.
* This routine verifies the integer vector arguments.
*
* @param in -- the input-only vector
* @param inout -- the input-output vector
* @param out -- the output-only vector
* @param len -- the number of elements in the vectors
* @param info -- on success, info is unchanged.
*    on failure, info is set to -1.
**
      subroutine verify_ivec(in, inout, out, len, info)
      integer len, i, info
      integer in(len)
      integer inout(len)
      integer out(len)
*
      do 1 i=1,len
        if(in(i) .ne. inout(i) - 101) then
          write(*,*) 'Integer vector in/out incorrect'
          info = -1
          return
        endif 
        if(in(i) .ne. out(i) - 1) then
          write(*,*) 'Integer vector out incorrect'
          info = -1
          return
        endif 
  1   continue
*
      return
      end
**
* Verifies that the results from calling totaltest are correct.
* This routine verifies the single precision vector arguments.
*
* @param in -- the input-only vector
* @param inout -- the input-output vector
* @param out -- the output-only vector
* @param len -- the number of elements in the vectors
* @param info -- on success, info is unchanged.
*    on failure, info is set to -1.
**
      subroutine verify_fvec(in, inout, out, len, info)
      integer len, i, info
      real in(len)
      real inout(len)
      real out(len)
      logical float_ne
*
      do 1 i=1,len
        if(float_ne(in(i), inout(i) - 101.0)) then
          write(*,*) 'Single precision vector in/out incorrect'
          info = -1
          return
        endif
        if(float_ne(in(i), out(i) - 1.0)) then
          write(*,*) 'Single precision vector out incorrect'
          info = -1
          return
        endif
  1   continue
*
      return
      end
**
* Verifies that the results from calling totaltest are correct.
* This routine verifies the double precision vector arguments.
*
* @param in -- the input-only vector
* @param inout -- the input-output vector
* @param out -- the output-only vector
* @param len -- the number of elements in the vectors
* @param info -- on success, info is unchanged.
*    on failure, info is set to -1.
**
      subroutine verify_dvec(in, inout, out, len, info)
      integer len, i, info
      double precision in(len)
      double precision inout(len)
      double precision out(len)
      logical dbl_ne
*
      do 1 i=1,len
        if(dbl_ne(in(i), inout(i) - 101.0)) then
          write(*,*) 'Double precision vector in/out incorrect'
          info = -1
          return
        endif
        if(dbl_ne(in(i), out(i) - 1.0)) then
          write(*,*) 'Double precision vector out incorrect'
          info = -1
          return
        endif
  1   continue
*
      return
      end
**
* Verifies that the results from calling totaltest are correct.
* This routine verifies the double precision vector arguments.
*
* @param in -- the input-only vector
* @param inout -- the input-output vector
* @param out -- the output-only vector
* @param len -- the number of elements in the vectors
* @param info -- on success, info is unchanged.
*    on failure, info is set to -1.
**
      subroutine verify_scmp_vec(in, inout, out, len, info)
      integer len, i, info
      complex in(len)
      complex inout(len)
      complex out(len)
      logical scmp_ne
*
      do 1 i=1,len
        if(scmp_ne(in(i), inout(i) - (101.0,1.0))) then
          write(*,*) 'scomplex vector in/out incorrect'
          info = -1
          return
        endif
        if(scmp_ne(in(i), out(i) - (1.0,1.0))) then
          write(*,*) 'scomplex vector out incorrect'
          info = -1
          return
        endif
  1   continue
*
      return
      end
**      
* Verifies that the results from calling totaltest are correct.
* This routine verifies the double precision vector arguments.
*         
* @param in -- the input-only vector
* @param inout -- the input-output vector
* @param out -- the output-only vector
* @param len -- the number of elements in the vectors
* @param info -- on success, info is unchanged.
*    on failure, info is set to -1.
**
      subroutine verify_dcmp_vec(in, inout, out, len, info)
      integer len, i, info
      double complex in(len)
      double complex inout(len)
      double complex out(len)
      logical dcmp_ne
* 
      do 1 i=1,len
        if(dcmp_ne(in(i), inout(i) - (101.0,1.0))) then
          write(*,*) 'dcomplex vector in/out incorrect'
          info = -1
          return
        endif
        if(dcmp_ne(in(i), out(i) - (1.0,1.0))) then
          write(*,*) 'dcomplex vector out incorrect'
          info = -1
          return
        endif
  1   continue
*         
      return
      end
**
* Verifies that the results from calling totaltest are correct.
* This routine verifies the character vector arguments.
*
* @param in -- the input-only vector
* @param inout -- the input-output vector
* @param out -- the output-only vector
* @param len -- the number of elements in the vectors
* @param info -- on success, info is unchanged.
*    on failure, info is set to -1.
**
      subroutine verify_cvec(in, inout, out, len, info)
      integer len, i, info
      character in(len)
      character inout(len)
      character out(len)
*
      do 1 i=1,len
        if(ichar(in(i)) .ne. ichar(inout(i)) -
     +         (ichar('j')-ichar('a'))) then
          write(*,*) 'Character vector in/out incorrect'
          info = -1
          return
        endif
        if(ichar(in(i)) .ne. ichar(out(i)) - 1) then
          write(*,*) 'Character vector out incorrect'
          info = -1
          return
        endif
  1   continue
*
      return
      end
**
* Verifies that the results from calling totaltest are correct.
* This routine verifies all the scalar arguments.
*
* @param int_in -- input-only integer scalar
* @param int_inout -- input-output integer scalar
* @param int_out -- output-only integer scalar
* @param float_in -- input-only single precision scalar
* @param float_inout -- input-output single precision scalar
* @param float_out -- output-only single precision scalar
* @param double_in -- input-only double precision scalar
* @param double_inout -- input-output double precision scalar
* @param double_out -- output-only double precision scalar
* @param char_in -- input-only character scalar
* @param char_inout -- input-output character scalar
* @param char_out -- output-only character scalar
* @param info -- on success, info is unchanged.
*    on failure, info is set to -1.
**
      subroutine verify_scalars(int_in, int_inout, int_out,
     +  float_in, float_inout, float_out,
     +  double_in, double_inout, double_out,
     +  scmp_in, scmp_inout, scmp_out,
     +  dcmp_in, dcmp_inout, dcmp_out,
     +  char_in, char_inout, char_out, info)
      integer info
      integer int_in, int_inout, int_out
      real float_in, float_inout, float_out
      double precision double_in, double_inout, double_out
      complex scmp_in, scmp_inout, scmp_out
      double complex dcmp_in, dcmp_inout, dcmp_out
      character char_in, char_inout, char_out
      logical float_ne, dbl_ne
      logical scmp_ne, dcmp_ne
*
      if(int_in .ne. int_inout - 101) then
        write(*,*) 'Integer scalar in/out incorrect'
        info = -1
        return
      endif
      if(int_in .ne. int_out - 1) then
        write(*,*) 'Integer scalar out incorrect'
        info = -1
        return
      endif
*
      if(float_ne(float_in, float_inout - 101.0)) then
        write(*,*) 'Single precision scalar in/out incorrect'
        info = -1
        return
      endif
      if(float_ne(float_in, float_out - 1.0)) then
        write(*,*) 'Single precision scalar out incorrect'
        info = -1
        return
      endif
*
      if(dbl_ne(double_in, double_inout - 101.0)) then
        write(*,*) 'Double precision scalar in/out incorrect'
        info = -1
        return
      endif
      if(dbl_ne(double_in, double_out - 1.0)) then
        write(*,*) 'Double precision scalar out incorrect'
        info = -1
        return
      endif
*
      if(scmp_ne(scmp_in, scmp_inout - (101.0,101.0))) then
        write(*,*) 'scomplex scalar in/out incorrect'
        info = -1
        return
      endif
      if(scmp_ne(scmp_in, scmp_out - (1.0,1.0))) then
        write(*,*) 'scomplex scalar out incorrect'
        info = -1
        return
      endif
*
      if(dcmp_ne(dcmp_in, dcmp_inout - (101.0,101.0))) then
        write(*,*) 'dcomplex scalar in/out incorrect'
        info = -1
        return
      endif
      if(dcmp_ne(dcmp_in, dcmp_out - (1.0,1.0))) then
        write(*,*) 'dcomplex scalar out incorrect'
        info = -1
        return
      endif
*
      if(ichar(char_in) .ne. ichar(char_inout) -
     +         (ichar('j')-ichar('a'))) then
        write(*,*) 'Character scalar in/out incorrect'
        info = -1
        return
      endif
      if(ichar(char_in) .ne. ichar(char_out) - 1) then
        write(*,*) 'Character scalar out incorrect'
        info = -1
        return
      endif
*
      return
      end
