/**
 * @file
 *
 * Test of the "return_double_scalar" problem.  This submits
 * a number of non-blocking requests specified on the command line:
 *    # return_test 5
 */
/* $Id: return_test.c,v 1.1 2005/03/12 06:29:06 seymour Exp $ */
/* $UTK_Copyright: $ */

#include "config.h"

#include<stdio.h>
#include<stdlib.h>
#ifdef HAVE_VALUES_H
#include<values.h>
#else
#ifdef HAVE_LIMITS_H
#include <limits.h>
#endif
#ifdef HAVE_FLOAT_H
#include <float.h>
#define MAXDOUBLE DBL_MAX
#endif
#endif
#include<sys/time.h>
#include "netsolve.h"
#include "netsolveclient.h"
#include "totaltest.h"

int 
main(int argc, char** argv)
{
  int i, num_req, *status, failures;
  double d, *foo;

  if(argc != 2) {
    fprintf(stderr,"Usage: %s [num]\n", argv[0]);
    die(EXIT_FAILURE);
  }

  num_req = atoi(argv[1]);

  d = 666.0;

  status = (int *)malloc(num_req * sizeof(int));
  foo = (double *)malloc(num_req * sizeof(double));

  for(i=0;i<num_req;i++) {
    foo[i] = 0.0;
    status[i] = netslnb("return_double_scalar()", d, &(foo[i]));
    printf("Submitted request %d\n", status[i]);

    if(status[i] < 0) {
      netslerr(status[i]);
      die(EXIT_FAILURE);
    }
  }

  failures=0;

  for(i=0;i<num_req;i++) {
    int reqstatus;

    printf("Probing for request %d\n", status[i]);
    reqstatus = netslpr(status[i]);
    fprintf(stderr,".");
    while(reqstatus == NetSolveNotReady)
    {
      sleep(1);
      fprintf(stderr,".");
      fflush(stderr);
      reqstatus = netslpr(status[i]);
    }
    fprintf(stderr,"\n");
    if (reqstatus == NetSolveOK)
    {
      reqstatus = netslwt(status[i]);
    }

    fflush(stdout);
    if(reqstatus != NetSolveOK) {
      netslerr(reqstatus);
      die(EXIT_FAILURE);
    }

    if(foo[i] != d+1.0)
      failures++;
  }

  if(failures > 0) {
    printf("%d of the requests returned the wrong value\n", failures);
    die(EXIT_FAILURE);
  }

  die(EXIT_SUCCESS);
}
