/**
 * @file
 * 
 * --SmartGridRPC implementation of mm2_gs.c --
 * 
 * This program makes a number (n) of GridRPC calls to the 
 * "dgesv" routine..
 * The output of the previous dgesv routine is the input to 
 * the next.
 *
 * --------------------------------------------------------
 *  In the SmartGridRPC implementation these outputs will
 *  be sent directly to the destination server or cached.
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "gs_smart_clib.h"
#include "grpc.h"
#include "totaltest.h"


double
elapsed_time(const struct timeval*, const struct timeval*);


int
main(int argc, char *argv[])
{
  grpc_function_handle_t * handles;
 int * status;

  struct timeval t1, t2;
  int i, n, nrhs, lda, *ipiv, ldb, info, num_iters;
  double *a, *b, et;
  int iter;
  char * mapper;
  if(argc!=4){
    fprintf(stderr," Error: Incorrect argument parameters. \n Usage: ./mm2_smart mapper n x\n mapper is the name of the mapping heuristic (rwalk_map, greedy_map, basic_map)\nn is the number of matrix multiplications to perform\n matrix dimensions = x \n");
    exit(1); 
  }
  mapper=argv[1];
  num_iters=atoi(argv[2]);
  n=atoi(argv[3]);
 // y=atoi(argv[4]);

  i=0;
  if(num_iters<1){
    fprintf(stderr," Error: Incorrect argument parameters. \n Usage: ./mm2_smart mapper n x\n mapper is the name of the mapping heuristic (rwalk_map, greedy_map, basic_map)\nn is the number of matrix multiplications to perform\n matrix dimensions = x \n");
    exit(1); 
  }
 
  handles=(grpc_function_handle_t *)(calloc(num_iters, sizeof(grpc_function_handle_t )));
  status=(int *)(calloc(num_iters, sizeof(int)));
  a = (double*)malloc(n*n*sizeof(double));
  b = (double*)malloc(n*n*sizeof(double));
  ipiv = (int*)malloc(n*sizeof(int));
  for(i=0;i<n*n;i++) {
    a[i] = (double)i;
    b[i] = (double)(n+i);
  }
      
  for(i=0;i<n;i++){
    ipiv[i] = n-i;
  }
  nrhs = n;
  lda = n;
  ldb = n;
  
  if(grpc_initialize(NULL) != GRPC_NO_ERROR) {
    grpc_perror("grpc_initialize");
    die(EXIT_FAILURE);
  }
 
  for(iter=0; iter<num_iters;iter++){
     if(grpc_function_handle_init(&handles[iter], "bind_servers_at_call_time", "dgesv") != GRPC_NO_ERROR)
    {
        fprintf(stderr,"Error creating function handle\n");
        die(EXIT_FAILURE);

 
    }
  }

  gettimeofday(&t1, 0);
  grpc_map(mapper){
    for(iter=0;iter< num_iters;iter++ ){       
      status[iter] = grpc_call(&handles[iter],  n, nrhs, a, lda, ipiv, b, ldb, &info);

      grpc_local(){
        if(status[iter] != GRPC_NO_ERROR) {
          grpc_perror("grpc_call");
          exit(-1);
        }
      }
    }
  }

  gettimeofday(&t2, 0);
  et = elapsed_time(&t1,&t2);
  
  printf("Elapsed time for %d (n) matrix multiplications : %g\n", num_iters, et);
  printf("Matrix dimensions = %d\n", n);
  printf("Finalizing...\n");
  grpc_finalize();

  printf("Done.\n");

  free(handles);
  free(status);
  free(ipiv);
  free(a);
  free(b);
  die(EXIT_SUCCESS);
}


double
elapsed_time(const struct timeval* t0, const struct timeval* t1)
{
  double ret = t1->tv_sec - t0->tv_sec;
  double uret = t1->tv_usec - t0->tv_usec;
  return ret + uret / 1E6;
}
