/**
 * @file
 *
 * -- GridRPC implementation --
 *
 * This program makes a number (x) of calls to the 
 * dgemm routine.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "gs_smart_clib.h"
#include "grpc.h"
#include "totaltest.h"


double
elapsed_time(const struct timeval*, const struct timeval*);


int
main(int argc, char *argv[])
{
  grpc_function_handle_t * handles;
 int * status;

  struct timeval t1, t2;
  int i, n, m, k, lda, ldb, ldc,  num_iters;
  double *a, *b, *c, et;
  int iter;
  char transa = 'T';
  char transb = 'N';
  double alpha = 1.0;
  double beta = 1.0;
  if(argc!=5){
    fprintf(stderr," Error: Incorrect argument parameters. \n Usage: ./mm3_gs x m n k\n x is the number of matrix multiplications to perform\n m n k specify the dimensions of matrix A B and C  \n");
    exit(1); 
  }
  num_iters=atoi(argv[1]);
  m=atoi(argv[2]);
  n=atoi(argv[3]);
  k=atoi(argv[4]);

  i=0;
  if(num_iters<1){
    fprintf(stderr," Error: Incorrect argument parameters. \n Usage: ./mm3_gs x m n k\n x is the number of matrix multiplications to perform\n m n k specify the dimensions of matrix A B and C  \n");
    exit(1); 
  }

  grpc_sessionid_t sessionIDs[num_iters];
  handles=(grpc_function_handle_t *)(calloc(num_iters, sizeof(grpc_function_handle_t )));
  status=(int *)(calloc(num_iters, sizeof(int)));
  a = (double*)malloc(m*k*sizeof(double));
  b = (double*)malloc(k*n*sizeof(double));
  c = (double*)malloc(m*n*sizeof(double));

  for(i=0;i<m*k;i++) {
    a[i] = (double)i;
  }
      
  for(i=0;i<k*n;i++) {
    b[i] = (double)(n+i);
  }

  lda = m;
  ldb = k;
  ldc=m;
  if(grpc_initialize(NULL) != GRPC_NO_ERROR) {
    grpc_perror("grpc_initialize");
    die(EXIT_FAILURE);
  }
 
  for(iter=0; iter<num_iters;iter++){
     if(grpc_function_handle_init(&handles[iter], "bind_servers_at_call_time", "dgemm") != GRPC_NO_ERROR)
    {
        fprintf(stderr,"Error creating function handle\n");
        die(EXIT_FAILURE);

 
    }
  }

  gettimeofday(&t1, 0);
  for(iter=0;iter< num_iters;iter++ ){       
    status[iter] = grpc_call_async(&handles[iter],&(sessionIDs[iter]),transa,transb,m,n,k,alpha,a,lda,b,ldb,beta,c,ldc);
    if(status[iter] != GRPC_NO_ERROR) {
      grpc_perror("grpc_call");
      exit(-1);
    }
  }
 

  printf("Waiting for tasks to finish.\n");
  for(iter=0;iter<num_iters;iter++) {
    int id;
    if(status[iter] != GRPC_NO_ERROR) {
      grpc_perror("grpc_call");
      return -1;
    }

    status[iter] = grpc_wait_or(sessionIDs, num_iters, &id);

    printf("%d completed\n", id);

    if(status[iter] != GRPC_NO_ERROR) {
      printf("Warning: one or more tasks failed.\n");
      exit(-1);
    }
  }



  gettimeofday(&t2, 0);
  et = elapsed_time(&t1,&t2);
  
  printf("Elapsed time for %d (n) matrix multiplications : %g\n", num_iters, et);
  printf("Matrix dimensions = %d\n", n);
  printf("Finalizing...\n");
  grpc_finalize();

  printf("Done.\n");

  free(handles);
  free(status);
  free(a);
  free(b);
  die(EXIT_SUCCESS);
}


double
elapsed_time(const struct timeval* t0, const struct timeval* t1)
{
  double ret = t1->tv_sec - t0->tv_sec;
  double uret = t1->tv_usec - t0->tv_usec;
  return ret + uret / 1E6;
}
