/**
 * @file
 *
 * This file contains test code for the icl_hash routines.
 *
 */
/* $Id: icl_hash_test.c,v 1.7 2005/09/30 18:41:15 seymour Exp $ */
/* $UTK_Copyright: $ */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "icl_hash.h"

/**
 * Entry point for icl_hash tester.
 *
 * @param argc -- arg count
 * @param argv -- array of command line args
 *
 * @returns EXIT_SUCCESS on success, EXIT_FAILURE on error.
 */

int main(int argc, char** argv)
{
  icl_hash_t* ht = icl_hash_create(64, NULL);
  icl_entry_t* p  = NULL;
  char *str, *foo;

  printf("Inserting Hello: A word\n");
  p = icl_hash_insert(ht, strdup("Hello"), strdup("A word"));
  printf("New node: key:%s data:%s\n", p->key, (char*)p->data);
  str = (char *)icl_hash_find(ht, "Hello");
  if(str) printf("Found %s\n", str);

  p = icl_hash_insert(ht, strdup("n"), strdup("a string for n"));
  printf("New node: key:%s data:%s\n", p->key, (char*)p->data);
  str = (char *)icl_hash_find(ht, "n");
  if(str) printf("Found %s\n", str);

  p = icl_hash_insert(ht, strdup("k"), strdup("a number for k"));
  printf("New node: key:%s data:%s\n", p->key, (char*)p->data);
  str = (char *)icl_hash_find(ht, "k");
  if(str) printf("Found %s\n", str);

  printf("Looking for non-existent node\n");
  str = (char *)icl_hash_find(ht, "foo");
  if(!str) printf("...cool, not found.\n");

  foo = strdup("foo");
  printf("Updating node\n");
  if(!icl_hash_update_insert(ht, foo, strdup("new entry for k"), (void**)&str))
    printf("update failed\n");
  else {
    printf("Looking for non-existent node\n");
    str = (char *)icl_hash_find(ht, foo);
    if(str) printf("Found %s\n", str);
  }

  icl_hash_dump(stderr, ht);

  icl_hash_destroy(ht, free, free);
  
  exit(EXIT_SUCCESS);
}
