/**
 * @file
 *
 * This file contains test code for the icl_list routines.
 *
 */
/* $Id: icl_list_test.c,v 1.3 2004/09/12 05:21:15 seymour Exp $ */
/* $UTK_Copyright: $ */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "icl_list.h"

/* This function can be used as a standin for strcmp, to avoid the
   warnings about incompatible pointer types */
int cmp(void* p1, void* p2) { return strcmp((char*)p1, (char*) p2); }
#undef strcmp
#define strcmp cmp

/**
 * Entry point for icl_list tester.
 *
 * @param argc -- arg count
 * @param argv -- array of command line args
 *
 * @returns EXIT_SUCCESS on success, EXIT_FAILURE on error.
 */

int main(int argc, char** argv)
{
  icl_list_t* head = icl_list_new();
  icl_list_t* l = NULL;
  
  icl_list_prepend(head, strdup("A word"));
  icl_list_prepend(head, strdup("B word"));
  icl_list_prepend(head, strdup("This word"));
  icl_list_prepend(head, strdup("C word"));
  icl_list_prepend(head, strdup("D word"));
  icl_list_prepend(head, strdup("That word"));
  
  printf("size %d\n", icl_list_size(head));
  
  printf("Printing the list\n");
  for (l=icl_list_first(head); l!=NULL; l=icl_list_next(head, l)) {
    printf("%s\n", (char *)l->data);
  }
  
  printf("Search: This word\n");
  l=icl_list_search(head, "This word", strcmp);
  printf("Found %s\n", (char *)l->data);
  
  printf("Insert: After this word\n");
  l=icl_list_insert(head, l, strdup("After this word"));
  
  printf("Search: This man\n");
  l=icl_list_search(head, "This man", strcmp);
  if (l!=NULL)  printf("Found %s\n", (char *)l->data);
  else printf("Not found\n");

  printf("Search: After this word\n");
  l=icl_list_search(head, "After this word", strcmp);
  if (l!=NULL)  printf("Found %s\n", (char *)l->data);
  else printf("Not found\n");

  icl_list_destroy(head, free);

  exit(EXIT_SUCCESS);
}
