/**
 * @file
 *
 * This file contains logging functions for GridSolve.
 *
 */
/* $Id: log_default.c,v 1.6 2007/09/11 15:28:29 yarkhan Exp $ */
/* $UTK_Copyright: $ */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>

#include "log.h"

static char * _GS_DEFAULT_LOG_EVENT = NULL;
static char * _GS_DEFAULT_LOG_IDENT = NULL;

/* static int _GS_DEFAULT_LOG_LEVEL = LOG_DEBUG; */
/* static int _GS_DEFAULT_LOG_OPTION = 0; */
/* static int _GS_DEFAULT_LOG_FACILITY = 0; */

/**
 * Initializes the logging system.
 *
 * @param ident --
 * @param option --
 * @param facility --
 * @param default_event --
 *
 * @returns 0 on success, -1 on failure.
 */

int 
gs_log_init(const char *ident, int option, int facility, const char *default_event)
{
  fprintf(stderr,"Initializing logging %s %s \n", ident, default_event);
  _GS_DEFAULT_LOG_EVENT = strdup(default_event);
  _GS_DEFAULT_LOG_IDENT = strdup(ident);
  fprintf(stderr,"Set logging %s %s \n", _GS_DEFAULT_LOG_IDENT, _GS_DEFAULT_LOG_EVENT);

  atexit((void (*)())gs_log_finalize);
  return 0;
}

/**
 * Writes a message to the log.
 *
 * @param file -- 
 * @param line --
 * @param function --
 * @param level --
 * @param event --
 * @param format --
 * @param ... --
 *
 * @returns 0 on success, -1 on failure.
 */

int 
gs_log_write(const char* file, const int line, const char* function, int level, const char* event, const char* format, ...)
{
  va_list argptr;

  if (event == NULL) {
    fprintf(stderr, "Received NULL event, default %s\n", _GS_DEFAULT_LOG_EVENT);
    event = _GS_DEFAULT_LOG_EVENT;
  }
  fprintf(stderr,"%s:%d [%s] %d %s", file,line,function,level,event);
  
  va_start(argptr, format);
  vfprintf(stderr,format, argptr);
  fflush(stderr);

  return 0;
}

/**
 * Releases any resources allocated by the logging system.
 *
 * @returns 0 on success, -1 on failure.
 */

int
gs_log_finalize()
{
  if(_GS_DEFAULT_LOG_EVENT) 
    free(_GS_DEFAULT_LOG_EVENT);
  _GS_DEFAULT_LOG_EVENT = NULL;

  if(_GS_DEFAULT_LOG_IDENT) 
    free(_GS_DEFAULT_LOG_IDENT);
  _GS_DEFAULT_LOG_IDENT = NULL;

  return 0;
}
