<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<xsl:template match="/">
  <html>
  <title>GridSolve Service Information</title>
  <body style="font-family:Arial,helvetica,sans-serif;font-size:12pt;
        background-color:#EEEEEE">
    <h2>GridSolve Service Information</h2>

    <a href="problemlist?">Service list</a>
    <br/>
    <a href="status?">Server list</a>
    <p/>

    <xsl:for-each select="problem_information/problem_list/problem">
      <xsl:sort select="@name"/>
      <div style="background-color:lightsteelblue;color:white;padding:4px">
        <span style="font-weight:bold;color:white">
        <xsl:value-of select="@name"/></span>
      </div>

      Description:
      <div style="margin-left:20px;margin-bottom:0em;font-size:10pt">
        <xsl:value-of select="@description"/>
      </div>
      <div style="margin-left:20px;margin-bottom:0em;font-size:10pt">
        This is a
        <xsl:choose>
          <xsl:when test="@major = 114">
            row-major
          </xsl:when>
          <xsl:when test="@major = 99">
            column-major
          </xsl:when>
        </xsl:choose>
        <xsl:value-of select="@type"/>.
      </div>

      <xsl:if test="infolist/info">
        <p/>
        Service Attributes:
      </xsl:if>
      <xsl:for-each select="infolist/info">
        <div style="margin-left:20px;margin-bottom:0em;font-size:10pt">
          <xsl:value-of select="@type"/> = <xsl:value-of select="@value"/>
        </div>
      </xsl:for-each>

      <p/>
      Arguments:
      <xsl:for-each select="arglist/arg">
        <div style="margin-left:20px;margin-bottom:0em;font-size:10pt">
          <xsl:value-of select="@name"/>
          <xsl:choose>
            <xsl:when test="@objecttype = 'vector'">
              [<xsl:value-of select="@rowexp"/>]
            </xsl:when>
            <xsl:when test="@objecttype = 'matrix'">
              [<xsl:value-of select="@rowexp"/>]
              [<xsl:value-of select="@colexp"/>]
            </xsl:when>
            <xsl:when test="@objecttype = 'file'">
              &lt;File&gt;
            </xsl:when>
            <xsl:when test="@objecttype = 'packedfile'">
              &lt;Packed file&gt;
            </xsl:when>
          </xsl:choose>
          --
          Description: <xsl:value-of select="@description"/>
        </div>
        <div style="margin-left:40px;margin-bottom:0em;font-size:10pt">
          Passing mode:
          <xsl:choose>
            <xsl:when test="@inout = 'in'"> Input-only </xsl:when>
            <xsl:when test="@inout = 'inout'"> Input/output </xsl:when>
            <xsl:when test="@inout = 'out'"> Output-only </xsl:when>
            <xsl:when test="@inout = 'workspace'"> Server-allocated workspace </xsl:when>
            <xsl:when test="@inout = 'varout'"> Variable-length output </xsl:when>
          </xsl:choose>
        </div>
        <div style="margin-left:40px;margin-bottom:0em;font-size:10pt">
          Data type: <xsl:value-of select="@datatype"/>
        </div>
      </xsl:for-each>
    </xsl:for-each>

    <xsl:if test="problem_information/server_list/server">
      <p/>
      Servers implementing this service:
    </xsl:if>
    <xsl:for-each select="problem_information/server_list/server">
      <xsl:variable name="cid" select="normalize-space(componentid)"/>
      <div style="margin-left:20px;margin-bottom:0em;font-size:10pt">
        <a href="serverinfo?{$cid}">
        <xsl:value-of select="hostname"/>:<xsl:value-of select="port"/></a>
      </div>
    </xsl:for-each>
    <p/>
    C Code Example:
    <xsl:for-each select="problem_information/problem_list/problem">
<pre>
<![CDATA[
/* This is an automatically generated code example, so arguments are
 * initialized with some arbitrary values that may not be valid for
 * the routine.  It is just intended to show the proper calling 
 * sequence (whether arguments should be passed by reference, how
 * they could be declared, etc) and simple GridRPC initialization, 
 * calling, and error handling.
 */

#include <stdio.h>
#include <stdlib.h>

#include "grpc.h"

int main()
{
  grpc_function_handle_t __handle;
  grpc_error_t __status;
]]>
</pre>
      <code>
      <xsl:for-each select="arglist/arg">
        <xsl:if test="@inout != 'workspace'">
          <div style="margin-left:20px;margin-bottom:0em">
          <xsl:value-of select="@datatype"/>
          <xsl:if test="@objecttype = 'vector' or @objecttype = 'matrix' 
            or @objecttype = 'sparsematrix' or @objecttype = 'file'"> 
            * 
          </xsl:if>
          <xsl:if test="@objecttype = 'packedfile'"> 
            ** 
          </xsl:if>
          <xsl:text> </xsl:text><xsl:value-of select="@name"/>;
          </div>
        </xsl:if>
      </xsl:for-each>
      <br/>
      <xsl:for-each select="arglist/arg">
        <xsl:variable name="varname" select="normalize-space(@name)"/>
        <xsl:variable name="is_sparse_attr" select="preceding-sibling::*[$varname=@nnzexp or $varname=@indices or $varname=@pointer] | following-sibling::*[$varname=@nnzexp or $varname=@indices or $varname=@pointer]"/>
        <xsl:if test="$is_sparse_attr=false()">
          <xsl:if test="@inout != 'workspace'">
            <xsl:if test="@objecttype = 'scalar' and @inout != 'out'">
              <div style="margin-left:20px;margin-bottom:0em">
              <xsl:value-of select="@name"/> = 
              <xsl:call-template name="ns_const">
                <xsl:with-param name="type">
                  <xsl:value-of select="@datatype"/>
                </xsl:with-param>
              </xsl:call-template>; 
              </div>
            </xsl:if>
            <xsl:if test="@objecttype = 'file'">
              <div style="margin-left:20px;margin-bottom:0em">
              <xsl:value-of select="@name"/> = &quot;foo&quot;;
              </div>
            </xsl:if>
          </xsl:if>
        </xsl:if>


      </xsl:for-each>
      <br/>
      <xsl:for-each select="arglist/arg">
        <xsl:variable name="varname" select="normalize-space(@name)"/>
        <xsl:variable name="is_sparse_attr" select="preceding-sibling::*[$varname=@nnzexp or $varname=@indices or $varname=@pointer] | following-sibling::*[$varname=@nnzexp or $varname=@indices or $varname=@pointer]"/>
        <xsl:if test="$is_sparse_attr=false()">
          <xsl:if test="@inout != 'workspace'">
            <xsl:if test="@objecttype = 'vector' or @objecttype = 'matrix' 
              or @objecttype = 'packedfile'"> 
              <div style="margin-left:20px;margin-bottom:0em">
              <xsl:value-of select="@name"/> = 
              ( <xsl:value-of select="@datatype"/>
              <xsl:choose>
                <xsl:when test="@objecttype = 'packedfile'">
                  **
                </xsl:when>
                <xsl:otherwise> * </xsl:otherwise>
              </xsl:choose>
              ) malloc(( <xsl:value-of select="@rowexp"/> )*
                ( <xsl:value-of select="@colexp"/> ) * 
                sizeof(*<xsl:value-of select="@name"/> ));
              </div>
            </xsl:if>
          </xsl:if>
        </xsl:if>
      </xsl:for-each>
      <br/>
      <xsl:for-each select="arglist/arg">
        <xsl:variable name="varname" select="normalize-space(@name)"/>
        <xsl:variable name="is_sparse_attr" select="preceding-sibling::*[$varname=@nnzexp or $varname=@indices or $varname=@pointer] | following-sibling::*[$varname=@nnzexp or $varname=@indices or $varname=@pointer]"/>
        <xsl:if test="$is_sparse_attr=false()">
          <xsl:if test="(@objecttype = 'vector' or @objecttype = 'matrix') and @inout != 'out' and @inout != 'workspace'">
            <div style="margin-left:20px;margin-bottom:0em">
            { <br/>
              <div style="margin-left:20px;margin-bottom:0em">
              int __i; <br/>
              for(__i=0;__i&lt;(
            <xsl:value-of select="@rowexp"/> )*
            ( <xsl:value-of select="@colexp"/> ); __i++)
            <br/>
              </div>
            <div style="margin-left:40px;margin-bottom:0em">
            <xsl:value-of select="@name"/>[__i] = 
            <xsl:call-template name="ns_const">
              <xsl:with-param name="type">
                <xsl:value-of select="@datatype"/>
              </xsl:with-param>
            </xsl:call-template>;
            </div>
            }
            </div>
          </xsl:if>
          <xsl:if test="@objecttype = 'packedfile'">
            <div style="margin-left:20px;margin-bottom:0em">
            { <br/>
            <div style="margin-left:20px;margin-bottom:0em">
              int __i; <br/>
              for(__i=0;__i&lt;(
              <xsl:value-of select="@rowexp"/> )
              ; __i++)
            <br/>
            </div>
            <div style="margin-left:40px;margin-bottom:0em">
            <xsl:value-of select="@name"/>[__i] = &quot;foo&quot;
            ;
            </div>
            }
            </div>
          </xsl:if>
          <xsl:if test="@objecttype = 'sparsematrix'">
            <div style="margin-left:20px;margin-bottom:0em">
            /* initialize sparse matrix */ <br/>
            if(( <xsl:value-of select="@nnzexp"/> = ns_gen_sparse_mat_<xsl:value-of select="@datatype"/>(
            <xsl:value-of select="@rowexp"/>,
            <xsl:value-of select="@rowexp"/>,
            2 * <xsl:value-of select="@rowexp"/>,
            &amp; <xsl:value-of select="@name"/>,
            &amp; <xsl:value-of select="@indices"/>,
            &amp; <xsl:value-of select="@pointer"/>
            )) &lt; 0) { <br/>
            <div style="margin-left:20px;margin-bottom:0em">
              fprintf(stderr,"Error creating sparse matrix\n"); <br/>
              exit(EXIT_FAILURE); <br/>
            </div>
            } <br/>
            </div>
          </xsl:if>
        </xsl:if>
      </xsl:for-each>
      <pre>
  if(grpc_initialize(NULL) != GRPC_NO_ERROR) {
    grpc_perror(&quot;grpc_initialize&quot;);
    exit(EXIT_FAILURE);
  }

  if(grpc_function_handle_default(&amp;__handle, &quot;<xsl:value-of select="@name"/>&quot;) != GRPC_NO_ERROR) {
    fprintf(stderr,&quot;Error creating function handle\n&quot;);
    exit(EXIT_FAILURE);
  }
      </pre>
      <div style="margin-left:20px;margin-bottom:0em">
      __status = grpc_call(&amp;__handle,
      <xsl:for-each select="arglist/arg">
        <xsl:if test="@inout != 'workspace'">
          <xsl:variable name="npos" select="position()"/>
          <xsl:if test="(@objecttype = 'scalar' and @inout != 'in') or @inout = 'varout'">
            &amp;
          </xsl:if>
          <xsl:value-of select="@name"/>
          <xsl:for-each select="following-sibling::*[@inout != 'workspace']">
            <xsl:if test="position() = last()">
               ,
            </xsl:if>
          </xsl:for-each>
        </xsl:if>
      </xsl:for-each>
      );
      </div>
      <pre>
  if(__status != GRPC_NO_ERROR) {
    printf(&quot;GRPC error __status  = %d\n&quot;, __status);
    grpc_perror(&quot;grpc_call&quot;);
    exit(__status);
  }

  printf(&quot;GridRPC call completed successfully.\n&quot;);

  grpc_finalize();
  exit(EXIT_SUCCESS);

}
      </pre>
      </code>
    </xsl:for-each>
  </body>
  </html>
</xsl:template>

<xsl:template name="ns_const">
  <xsl:param name="type"/>
  <xsl:choose>
    <xsl:when test="$type = 'int'">10</xsl:when>
    <xsl:when test="$type = 'float'">5.0</xsl:when>
    <xsl:when test="$type = 'double'">10.0</xsl:when>
    <xsl:when test="$type = 'ns_scomplex'">(ns_scomplex) {4.0, 6.0}</xsl:when>
    <xsl:when test="$type = 'ns_dcomplex'">(ns_dcomplex) {4.0, 6.0}</xsl:when>
    <xsl:when test="$type = 'char'">'n'</xsl:when>
    <xsl:otherwise> bad const </xsl:otherwise>
  </xsl:choose>
</xsl:template>

</xsl:stylesheet>
